@@ -1,4 +1,4 @@
-/* $Id: config.h 1799 2007-02-25 00:33:13Z peter $ */
+/* $Id: config.h 1881 2007-06-06 03:24:02Z peter $ */
 
 #define yasm__splitpath(path, tail)	yasm__splitpath_win(path, tail)
 #define yasm__abspath(path)		yasm__abspath_win(path)
@@ -121,22 +121,22 @@
 #define PACKAGE_BUGREPORT "bug-yasm@tortall.net"
 
 /* Define to build version of this package. */
-#define PACKAGE_BUILD "1798"
+#define PACKAGE_BUILD "BRANCH"
 
 /* Define to internal version of this package. */
-#define PACKAGE_INTVER "0.6.0"
+#define PACKAGE_INTVER "0.6.1"
 
 /* Define to the full name of this package. */
 #define PACKAGE_NAME "yasm"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "yasm 0.6.0"
+#define PACKAGE_STRING "yasm 0.6.1"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "yasm"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "0.6.0"
+#define PACKAGE_VERSION "0.6.1"
 
 /* Define if the C compiler supports function prototypes. */
 #define PROTOTYPES 1
@@ -160,7 +160,7 @@
 #define STDC_HEADERS 1
 
 /* Version number of package */
-#define VERSION "0.6.0"
+#define VERSION "0.6.1"
 
 /* Define if using the dmalloc debugging malloc package */
 /* #undef WITH_DMALLOC */
