@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: yasm-options.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: yasm-options.c 1795 2007-02-24 22:39:55Z peter $");
 
 #include "yasm-options.h"
 
@@ -58,6 +58,16 @@ parse_cmdline(int argc, char **argv, opt_option *options, size_t nopts,
 	if (argv[0][0] == '-') {	/* opt */
 	    got_it = 0;
 	    if (argv[0][1] == '-') {	/* lopt */
+		if (argv[0][2] == '\0') {   /* --, end of options */
+		    /* Handle rest of args as non-options */
+		    while (--argc) {
+			argv++;
+			if (not_an_option_handler(argv[0]))
+			    errors++;
+		    }
+		    return errors;
+		}
+
 		for (i = 0; i < nopts; i++) {
 		    if (options[i].lopt &&
 			strncmp(&argv[0][2], options[i].lopt,
@@ -85,13 +95,17 @@ parse_cmdline(int argc, char **argv, opt_option *options, size_t nopts,
 			break;
 		    }
 		}
+		if (!got_it && !other_option_handler(argv[0]))
+		    got_it = 1;
 		if (!got_it) {
 		    print_error(_("warning: unrecognized option `%s'"),
 				argv[0]);
 		    warnings++;
 		}
+	    } else if (argv[0][1] == '\0') {   /* just -, is non-option */
+		if (not_an_option_handler(argv[0]))
+		    errors++;
 	    } else {		/* sopt */
-
 		for (i = 0; i < nopts; i++) {
 		    if (argv[0][1] == options[i].sopt) {
 			char *cmd = &argv[0][1];
@@ -119,6 +133,8 @@ parse_cmdline(int argc, char **argv, opt_option *options, size_t nopts,
 			break;
 		    }
 		}
+		if (!got_it && !other_option_handler(argv[0]))
+		    got_it = 1;
 		if (!got_it) {
 		    print_error(_("warning: unrecognized option `%s'"),
 				argv[0]);
@@ -145,8 +161,8 @@ help_msg(const char *msg, const char *tail, opt_option *options, size_t nopts)
     printf("%s", gettext(msg));
 
     for (i = 0; i < nopts; i++) {
-	int shortopt_len = 0;
-	int longopt_len = 0;
+	size_t shortopt_len = 0;
+	size_t longopt_len = 0;
 
 	optbuf[0] = 0;
 	optopt[0] = 0;
