@@ -1,4 +1,4 @@
-/* $Id: yasm-options.h 1591 2006-07-13 03:25:38Z peter $
+/* $Id: yasm-options.h 1636 2006-09-30 08:16:39Z peter $
  * Generic Options Support Header File
  *
  * Copyright (c) 2001  Stanislav Karchebny <berk@madfire.net>
@@ -55,6 +55,9 @@ typedef struct opt_option_s
 /* handle everything that is not an option */
 int not_an_option_handler(char *param);
 
+/* handle possibly other special-case options; no parameters allowed */
+int other_option_handler(char *option);
+
 /* parse command line calling handlers when appropriate
  * argc, argv - pass directly from main(argc,argv)
  * options - array of options
