@@ -1,7 +1,7 @@
 /*
  * Program entry point, command line parsing
  *
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: yasm.c 1760 2007-02-08 05:02:37Z peter $");
+/*@unused@*/ RCSID("$Id: yasm.c 1876 2007-06-02 21:50:51Z peter $");
 
 #include <ctype.h>
 #include <libyasm/compat-queue.h>
@@ -56,10 +56,8 @@ static int special_options = 0;
 /*@null@*/ /*@dependent@*/ static const yasm_preproc_module *
     cur_preproc_module = NULL;
 /*@null@*/ static char *objfmt_keyword = NULL;
-/*@null@*/ /*@dependent@*/ static yasm_objfmt *cur_objfmt = NULL;
 /*@null@*/ /*@dependent@*/ static const yasm_objfmt_module *
     cur_objfmt_module = NULL;
-/*@null@*/ /*@dependent@*/ static yasm_dbgfmt *cur_dbgfmt = NULL;
 /*@null@*/ /*@dependent@*/ static const yasm_dbgfmt_module *
     cur_dbgfmt_module = NULL;
 /*@null@*/ /*@dependent@*/ static yasm_listfmt *cur_listfmt = NULL;
@@ -69,6 +67,8 @@ static int preproc_only = 0;
 static unsigned int force_strict = 0;
 static int generate_make_dependencies = 0;
 static int warning_error = 0;	/* warnings being treated as errors */
+static FILE *errfile;
+/*@null@*/ /*@only@*/ static char *error_filename = NULL;
 static enum {
     EWSTYLE_GNU = 0,
     EWSTYLE_VC
@@ -77,7 +77,8 @@ static enum {
 /*@null@*/ /*@dependent@*/ static FILE *open_file(const char *filename,
 						  const char *mode);
 static void check_errors(/*@only@*/ yasm_errwarns *errwarns,
-			 /*@only@*/ yasm_object *object);
+			 /*@only@*/ yasm_object *object,
+			 /*@only@*/ yasm_linemap *linemap);
 static void cleanup(/*@null@*/ /*@only@*/ yasm_object *object);
 
 /* Forward declarations: cmd line parser handlers */
@@ -93,6 +94,8 @@ static int opt_objfile_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_machine_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_strict_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_warning_handler(char *cmd, /*@null@*/ char *param, int extra);
+static int opt_error_file(char *cmd, /*@null@*/ char *param, int extra);
+static int opt_error_stdout(char *cmd, /*@null@*/ char *param, int extra);
 static int preproc_only_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_include_option(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_preproc_option(char *cmd, /*@null@*/ char *param, int extra);
@@ -160,6 +163,10 @@ static opt_option options[] =
       N_("enables/disables warning"), NULL },
     { 'M', NULL, 0, opt_makedep_handler, 0,
       N_("generate Makefile dependencies on stdout"), NULL },
+    { 'E', NULL, 1, opt_error_file, 0,
+      N_("redirect error messages to file"), N_("file") },
+    { 's', NULL, 0, opt_error_stdout, 0,
+      N_("redirect error messages to stdout"), NULL },
     { 'e', "preproc-only", 0, preproc_only_handler, 0,
       N_("preprocess only (writes output to stdout by default)"), NULL },
     { 'i', NULL, 1, opt_include_option, 0,
@@ -218,7 +225,7 @@ do_preproc_only(FILE *in)
     FILE *out = NULL;
     yasm_errwarns *errwarns = yasm_errwarns_create();
 
-    /* Initialize line manager */
+    /* Initialize line map */
     linemap = yasm_linemap_create();
     yasm_linemap_set(linemap, in_filename, 1, 1);
 
@@ -313,13 +320,17 @@ static int
 do_assemble(FILE *in)
 {
     yasm_object *object;
-    yasm_section *def_sect;
     const char *base_filename;
     /*@null@*/ FILE *obj = NULL;
     yasm_arch_create_error arch_error;
+    yasm_linemap *linemap;
     yasm_errwarns *errwarns = yasm_errwarns_create();
     int i, matched;
 
+    /* Initialize line map */
+    linemap = yasm_linemap_create();
+    yasm_linemap_set(linemap, in_filename, 1, 1);
+
     /* determine the object filename if not specified */
     if (!obj_filename) {
 	if (in == stdin)
@@ -375,16 +386,18 @@ do_assemble(FILE *in)
     }
 
     /* Create object */
-    object = yasm_object_create(in_filename, obj_filename);
-    yasm_linemap_set(yasm_object_get_linemap(object), in_filename, 1, 1);
+    object = yasm_object_create(in_filename, obj_filename, cur_arch,
+				cur_objfmt_module, cur_dbgfmt_module);
+    if (!object) {
+	yasm_error_class eclass;
+	unsigned long xrefline;
+	/*@only@*/ /*@null@*/ char *estr, *xrefstr;
+
+	yasm_error_fetch(&eclass, &estr, &xrefline, &xrefstr);
+	print_error("%s: %s", _("FATAL"), estr);
+	yasm_xfree(estr);
+	yasm_xfree(xrefstr);
 
-    /* Initialize the object format */
-    cur_objfmt = yasm_objfmt_create(cur_objfmt_module, object, cur_arch);
-    if (!cur_objfmt) {
-	print_error(
-	    _("%s: object format `%s' does not support architecture `%s' machine `%s'"),
-	    _("FATAL"), cur_objfmt_module->keyword, cur_arch_module->keyword,
-	    machine_name);
 	if (in != stdin)
 	    fclose(in);
 	cleanup(object);
@@ -392,41 +405,7 @@ do_assemble(FILE *in)
     }
 
     /* Get a fresh copy of objfmt_module as it may have changed. */
-    cur_objfmt_module = ((yasm_objfmt_base *)cur_objfmt)->module;
-
-    /* Add an initial "default" section to object */
-    def_sect = yasm_objfmt_add_default_section(cur_objfmt);
-
-    /* Check to see if the requested debug format is in the allowed list
-     * for the active object format.
-     */
-    matched = 0;
-    for (i=0; cur_objfmt_module->dbgfmt_keywords[i]; i++)
-	if (yasm__strcasecmp(cur_objfmt_module->dbgfmt_keywords[i],
-			     cur_dbgfmt_module->keyword) == 0)
-	    matched = 1;
-    if (!matched) {
-	print_error(_("%s: `%s' is not a valid %s for %s `%s'"),
-	    _("FATAL"), cur_dbgfmt_module->keyword, _("debug format"),
-	    _("object format"), cur_objfmt_module->keyword);
-	if (in != stdin)
-	    fclose(in);
-	cleanup(object);
-	return EXIT_FAILURE;
-    }
-
-    /* Initialize the debug format */
-    cur_dbgfmt = yasm_dbgfmt_create(cur_dbgfmt_module, object, cur_objfmt,
-				    cur_arch);
-    if (!cur_dbgfmt) {
-	print_error(
-	    _("%s: debug format `%s' does not work with object format `%s'"),
-	    _("FATAL"), cur_dbgfmt_module->keyword,
-	    cur_objfmt_module->keyword);
-	if (in != stdin)
-	    fclose(in);
-	return EXIT_FAILURE;
-    }
+    cur_objfmt_module = ((yasm_objfmt_base *)object->objfmt)->module;
 
     /* Check to see if the requested preprocessor is in the allowed list
      * for the active parser.
@@ -446,8 +425,7 @@ do_assemble(FILE *in)
 	return EXIT_FAILURE;
     }
 
-    cur_preproc = cur_preproc_module->create(in, in_filename,
-					     yasm_object_get_linemap(object),
+    cur_preproc = cur_preproc_module->create(in, in_filename, linemap,
 					     errwarns);
 
     apply_preproc_builtins();
@@ -462,34 +440,26 @@ do_assemble(FILE *in)
     yasm_arch_set_var(cur_arch, "force_strict", force_strict);
 
     /* Parse! */
-    cur_parser_module->do_parse(object, cur_preproc, cur_arch, cur_objfmt,
-				cur_dbgfmt, in, in_filename,
-				list_filename != NULL, def_sect, errwarns);
+    cur_parser_module->do_parse(object, cur_preproc, in, list_filename != NULL,
+				linemap, errwarns);
 
     /* Close input file */
     if (in != stdin)
 	fclose(in);
 
-    check_errors(errwarns, object);
-
-    /* Check for undefined symbols */
-    yasm_symtab_parser_finalize(yasm_object_get_symtab(object),
-				strcmp(cur_parser_module->keyword, "gas")==0 ||
-				strcmp(cur_parser_module->keyword, "gnu")==0,
-				cur_objfmt, errwarns);
-    check_errors(errwarns, object);
+    check_errors(errwarns, object, linemap);
 
     /* Finalize parse */
     yasm_object_finalize(object, errwarns);
-    check_errors(errwarns, object);
+    check_errors(errwarns, object, linemap);
 
     /* Optimize */
-    yasm_object_optimize(object, cur_arch, errwarns);
-    check_errors(errwarns, object);
+    yasm_object_optimize(object, errwarns);
+    check_errors(errwarns, object, linemap);
 
     /* generate any debugging information */
-    yasm_dbgfmt_generate(cur_dbgfmt, errwarns);
-    check_errors(errwarns, object);
+    yasm_dbgfmt_generate(object, linemap, errwarns);
+    check_errors(errwarns, object, linemap);
 
     /* open the object file for output (if not already opened by dbg objfmt) */
     if (!obj && strcmp(cur_objfmt_module->keyword, "dbg") != 0) {
@@ -501,9 +471,8 @@ do_assemble(FILE *in)
     }
 
     /* Write the object file */
-    yasm_objfmt_output(cur_objfmt, obj?obj:stderr,
-		       strcmp(cur_dbgfmt_module->keyword, "null"), cur_dbgfmt,
-		       errwarns);
+    yasm_objfmt_output(object, obj?obj:stderr,
+		       strcmp(cur_dbgfmt_module->keyword, "null"), errwarns);
 
     /* Close object file */
     if (obj)
@@ -514,7 +483,7 @@ do_assemble(FILE *in)
      */
     if (yasm_errwarns_num_errors(errwarns, warning_error) > 0)
 	remove(obj_filename);
-    check_errors(errwarns, object);
+    check_errors(errwarns, object, linemap);
 
     /* Open and write the list file */
     if (list_filename) {
@@ -526,15 +495,14 @@ do_assemble(FILE *in)
 	/* Initialize the list format */
 	cur_listfmt = yasm_listfmt_create(cur_listfmt_module, in_filename,
 					  obj_filename);
-	yasm_listfmt_output(cur_listfmt, list,
-			    yasm_object_get_linemap(object), cur_arch);
+	yasm_listfmt_output(cur_listfmt, list, linemap, cur_arch);
 	fclose(list);
     }
 
-    yasm_errwarns_output_all(errwarns, yasm_object_get_linemap(object),
-			     warning_error, print_yasm_error,
-			     print_yasm_warning);
+    yasm_errwarns_output_all(errwarns, linemap, warning_error,
+			     print_yasm_error, print_yasm_warning);
 
+    yasm_linemap_destroy(linemap);
     yasm_errwarns_destroy(errwarns);
     cleanup(object);
     return EXIT_SUCCESS;
@@ -548,6 +516,8 @@ main(int argc, char *argv[])
     /*@null@*/ FILE *in = NULL;
     size_t i;
 
+    errfile = stderr;
+
 #if defined(HAVE_SETLOCALE) && defined(HAVE_LC_MESSAGES)
     setlocale(LC_MESSAGES, "");
 #endif
@@ -586,6 +556,13 @@ main(int argc, char *argv[])
 	    return EXIT_SUCCESS;
     }
 
+    /* Open error file if specified. */
+    if (error_filename) {
+	errfile = open_file(error_filename, "wt");
+	if (!errfile)
+	    return EXIT_FAILURE;
+    }
+
     /* Initialize BitVector (needed for intnum/floatnum). */
     if (BitVector_Boot() != ErrCode_Ok) {
 	print_error(_("%s: could not initialize BitVector"), _("FATAL"));
@@ -620,7 +597,7 @@ main(int argc, char *argv[])
 
     /* Check for arch help */
     if (machine_name && strcmp(machine_name, "help") == 0) {
-	yasm_arch_machine *m = cur_arch_module->machines;
+	const yasm_arch_machine *m = cur_arch_module->machines;
 	printf(_("Available %s for %s `%s':\n"), _("machines"),
 	       _("architecture"), cur_arch_module->keyword);
 	while (m->keyword && m->name) {
@@ -717,12 +694,13 @@ open_file(const char *filename, const char *mode)
 }
 
 static void
-check_errors(yasm_errwarns *errwarns, yasm_object *object)
+check_errors(yasm_errwarns *errwarns, yasm_object *object,
+	     yasm_linemap *linemap)
 {
     if (yasm_errwarns_num_errors(errwarns, warning_error) > 0) {
-	yasm_errwarns_output_all(errwarns, yasm_object_get_linemap(object),
-				 warning_error, print_yasm_error,
-				 print_yasm_warning);
+	yasm_errwarns_output_all(errwarns, linemap, warning_error,
+				 print_yasm_error, print_yasm_warning);
+	yasm_linemap_destroy(linemap);
 	yasm_errwarns_destroy(errwarns);
 	cleanup(object);
 	exit(EXIT_FAILURE);
@@ -740,18 +718,12 @@ static void
 cleanup(yasm_object *object)
 {
     if (DO_FREE) {
-	if (cur_objfmt)
-	    yasm_objfmt_destroy(cur_objfmt);
-	if (cur_dbgfmt)
-	    yasm_dbgfmt_destroy(cur_dbgfmt);
 	if (cur_listfmt)
 	    yasm_listfmt_destroy(cur_listfmt);
 	if (cur_preproc)
 	    yasm_preproc_destroy(cur_preproc);
 	if (object)
 	    yasm_object_destroy(object);
-	if (cur_arch)
-	    yasm_arch_destroy(cur_arch);
 
 	yasm_floatnum_cleanup();
 	yasm_intnum_cleanup();
@@ -773,6 +745,9 @@ cleanup(yasm_object *object)
 	if (objfmt_keyword)
 	    yasm_xfree(objfmt_keyword);
     }
+
+    if (errfile != stderr && errfile != stdout)
+	fclose(errfile);
 }
 
 /*
@@ -1024,12 +999,42 @@ opt_warning_handler(char *cmd, /*@unused@*/ char *param, int extra)
 	action(YASM_WARN_ORPHAN_LABEL);
     else if (strcmp(cmd, "uninit-contents") == 0)
 	action(YASM_WARN_UNINIT_CONTENTS);
+    else if (strcmp(cmd, "size-override") == 0)
+	action(YASM_WARN_SIZE_OVERRIDE);
     else
 	return 1;
 
     return 0;
 }
 
+static int
+opt_error_file(/*@unused@*/ char *cmd, char *param, /*@unused@*/ int extra)
+{
+    if (error_filename) {
+	print_error(
+	    _("warning: can output to only one error file, last specified used"));
+	yasm_xfree(error_filename);
+    }
+
+    assert(param != NULL);
+    error_filename = yasm__xstrdup(param);
+
+    return 0;
+}
+
+static int
+opt_error_stdout(/*@unused@*/ char *cmd, /*@unused@*/ char *param,
+		 /*@unused@*/ int extra)
+{
+    /* Clear any specified error filename */
+    if (error_filename) {
+	yasm_xfree(error_filename);
+	error_filename = NULL;
+    }
+    errfile = stdout;
+    return 0;
+}
+
 static int
 preproc_only_handler(/*@unused@*/ char *cmd, /*@unused@*/ char *param,
 		     /*@unused@*/ int extra)
@@ -1184,11 +1189,11 @@ static void
 print_error(const char *fmt, ...)
 {
     va_list va;
-    fprintf(stderr, "yasm: ");
+    fprintf(errfile, "yasm: ");
     va_start(va, fmt);
-    vfprintf(stderr, fmt, va);
+    vfprintf(errfile, fmt, va);
     va_end(va);
-    fputc('\n', stderr);
+    fputc('\n', errfile);
 }
 
 static /*@exits@*/ void
@@ -1206,9 +1211,9 @@ handle_yasm_int_error(const char *file, unsigned int line, const char *message)
 static /*@exits@*/ void
 handle_yasm_fatal(const char *fmt, va_list va)
 {
-    fprintf(stderr, "yasm: %s: ", _("FATAL"));
-    vfprintf(stderr, gettext(fmt), va);
-    fputc('\n', stderr);
+    fprintf(errfile, "yasm: %s: ", _("FATAL"));
+    vfprintf(errfile, gettext(fmt), va);
+    fputc('\n', errfile);
     exit(EXIT_FAILURE);
 }
 
@@ -1234,15 +1239,16 @@ print_yasm_error(const char *filename, unsigned long line, const char *msg,
 		 const char *xref_msg)
 {
     if (line)
-	fprintf(stderr, fmt[ewmsg_style], filename, line, "", msg);
+	fprintf(errfile, fmt[ewmsg_style], filename, line, "", msg);
     else
-	fprintf(stderr, fmt_noline[ewmsg_style], filename, "", msg);
+	fprintf(errfile, fmt_noline[ewmsg_style], filename, "", msg);
 
     if (xref_fn && xref_msg) {
 	if (xref_line)
-	    fprintf(stderr, fmt[ewmsg_style], xref_fn, xref_line, "", xref_msg);
+	    fprintf(errfile, fmt[ewmsg_style], xref_fn, xref_line, "",
+		    xref_msg);
 	else
-	    fprintf(stderr, fmt_noline[ewmsg_style], xref_fn, "", xref_msg);
+	    fprintf(errfile, fmt_noline[ewmsg_style], xref_fn, "", xref_msg);
     }
 }
 
@@ -1250,7 +1256,9 @@ static void
 print_yasm_warning(const char *filename, unsigned long line, const char *msg)
 {
     if (line)
-	fprintf(stderr, fmt[ewmsg_style], filename, line, _("warning: "), msg);
+	fprintf(errfile, fmt[ewmsg_style], filename, line, _("warning: "),
+		msg);
     else
-	fprintf(stderr, fmt_noline[ewmsg_style], filename, _("warning: "), msg);
+	fprintf(errfile, fmt_noline[ewmsg_style], filename, _("warning: "),
+		msg);
 }
