@@ -3,11 +3,11 @@
  * \brief YASM core types and utility functions.
  *
  * \rcs
- * $Id: coretype.h 1795 2007-02-24 22:39:55Z peter $
+ * $Id: coretype.h 1876 2007-06-02 21:50:51Z peter $
  * \endrcs
  *
  * \license
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,6 +47,11 @@ typedef struct yasm_dbgfmt yasm_dbgfmt;
 /** List format interface.  \see listfmt.h for details. */
 typedef struct yasm_listfmt yasm_listfmt;
 
+/** Object format module interface.  \see objfmt.h for details. */
+typedef struct yasm_objfmt_module yasm_objfmt_module;
+/** Debug format module interface.  \see dbgfmt.h for details. */
+typedef struct yasm_dbgfmt_module yasm_dbgfmt_module;
+
 /** YASM associated data callback structure.  Many data structures can have
  * arbitrary data associated with them.
  */
@@ -75,7 +80,7 @@ typedef struct yasm_errwarns yasm_errwarns;
  */
 typedef struct yasm_bytecode yasm_bytecode;
 
-/** Object (opaque type).  \see section.h for related functions. */
+/** Object.  \see section.h for details and related functions. */
 typedef struct yasm_object yasm_object;
 
 /** Section (opaque type).  \see section.h for related functions. */
@@ -182,6 +187,10 @@ typedef struct yasm_valparam yasm_valparam;
  * \see valparam.h for related functions.
  */
 typedef struct yasm_valparamhead yasm_valparamhead;
+/** Directive list entry.
+ * \see valparam.h for details and related functions.
+ */
+typedef struct yasm_directive yasm_directive;
 
 /** A list of instruction operands (opaque type).
  * The list goes from left-to-right as parsed.
@@ -227,18 +236,6 @@ typedef enum yasm_expr_op {
     YASM_EXPR_SEGOFF	/**< The ':' in segment:offset. */
 } yasm_expr_op;
 
-/** Symbol record visibility.
- * \see symrec.h for related functions.
- * \note YASM_SYM_EXTERN and YASM_SYM_COMMON are mutually exclusive.
- */
-typedef enum yasm_sym_vis {
-    YASM_SYM_LOCAL = 0,		/**< Default, local only */
-    YASM_SYM_GLOBAL = 1 << 0,	/**< If symbol is declared GLOBAL */
-    YASM_SYM_COMMON = 1 << 1,	/**< If symbol is declared COMMON */
-    YASM_SYM_EXTERN = 1 << 2,	/**< If symbol is declared EXTERN */
-    YASM_SYM_DLOCAL = 1 << 3	/**< If symbol is explicitly declared LOCAL */
-} yasm_sym_vis;
-
 /** Convert yasm_value to its byte representation.  Usually implemented by
  * object formats to keep track of relocations and verify legal expressions.
  * Must put the value into the least significant bits of the destination,
@@ -328,11 +325,11 @@ int yasm__strncasecmp(const char *s1, const char *s2, size_t n);
 
 /** strndup() implementation using yasm_xmalloc().
  * \internal
- * \param str	string
- * \param len	maximum number of characters to copy
+ * \param str   string
+ * \param max   maximum number of characters to copy
  * \return Newly allocated duplicate string.
  */
-/*@only@*/ char *yasm__xstrndup(const char *str, size_t len);
+/*@only@*/ char *yasm__xstrndup(const char *str, size_t max);
 
 /** Error-checking memory allocation.  A default implementation is provided
  * that calls yasm_fatal() on allocation errors.
