@@ -1,7 +1,7 @@
 /*
  * Floating point number functions.
  *
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  *  Based on public-domain x86 assembly code by Randall Hyde (8/28/91).
  *
@@ -28,7 +28,7 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: floatnum.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: floatnum.c 1876 2007-06-02 21:50:51Z peter $");
 
 #include <ctype.h>
 
@@ -482,8 +482,11 @@ yasm_floatnum_create(const char *str)
 	}
     }
 
-    /* Round the result. (Don't round underflow or overflow). */
-    if ((flt->exponent != EXP_INF) && (flt->exponent != EXP_ZERO))
+    /* Round the result. (Don't round underflow or overflow).  Also don't
+     * increment if this would cause the mantissa to wrap.
+     */
+    if ((flt->exponent != EXP_INF) && (flt->exponent != EXP_ZERO) &&
+        !BitVector_is_full(flt->mantissa))
 	BitVector_increment(flt->mantissa);
 
     return flt;
