@@ -1,7 +1,7 @@
 /*
  * Hash Array Mapped Trie (HAMT) implementation
  *
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  *  Based on the paper "Ideal Hash Tries" by Phil Bagwell [2000].
  *  One algorithmic change from that described in the paper: we use the LSB's
@@ -31,7 +31,9 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: hamt.c 1786 2007-02-24 04:16:05Z peter $");
+/*@unused@*/ RCSID("$Id: hamt.c 1876 2007-06-02 21:50:51Z peter $");
+
+#include <ctype.h>
 
 #include "libyasm-stdint.h"
 #include "coretype.h"
@@ -45,7 +47,7 @@ struct HAMTEntry {
 
 typedef struct HAMTNode {
     unsigned long BitMapKey;		/* 32 bits, bitmap or hash key */
-    void *BaseValue;			/* Base of HAMTNode list or value */
+    uintptr_t BaseValue;		/* Base of HAMTNode list or value */
 } HAMTNode;
 
 struct HAMT {
@@ -53,20 +55,28 @@ struct HAMT {
     HAMTNode *root;
     /*@exits@*/ void (*error_func) (const char *file, unsigned int line,
 				    const char *message);
+    unsigned long (*HashKey) (const char *key);
+    unsigned long (*ReHashKey) (const char *key, int Level);
 };
 
 /* XXX make a portable version of this.  This depends on the pointer being
  * 4 or 2-byte aligned (as it uses the LSB of the pointer variable to store
  * the subtrie flag!
  */
-#define IsSubTrie(n)		((uintptr_t)((n)->BaseValue) & 1)
+#define IsSubTrie(n)		((n)->BaseValue & 1)
 #define SetSubTrie(h, n, v)	do {				\
-	if ((uintptr_t)(v) & 1)				\
+	if ((uintptr_t)(v) & 1)					\
 	    h->error_func(__FILE__, __LINE__,			\
 			  N_("Subtrie is seen as subtrie before flag is set (misaligned?)"));	\
-	(n)->BaseValue = (void *)((uintptr_t)(v) | 1);	\
+	(n)->BaseValue = (uintptr_t)(v) | 1;	\
     } while (0)
-#define GetSubTrie(n)		(HAMTNode *)((uintptr_t)((n)->BaseValue)&~1UL)
+#define SetValue(h, n, v)	do {				\
+	if ((uintptr_t)(v) & 1)					\
+	    h->error_func(__FILE__, __LINE__,			\
+			  N_("Value is seen as subtrie (misaligned?)")); \
+	(n)->BaseValue = (uintptr_t)(v);	\
+    } while (0)
+#define GetSubTrie(n)		(HAMTNode *)(((n)->BaseValue | 1) ^ 1)
 
 static unsigned long
 HashKey(const char *key)
@@ -86,8 +96,26 @@ ReHashKey(const char *key, int Level)
     return vHash;
 }
 
+static unsigned long
+HashKey_nocase(const char *key)
+{
+    unsigned long a=31415, b=27183, vHash;
+    for (vHash=0; *key; key++, a*=b)
+	vHash = a*vHash + tolower(*key);
+    return vHash;
+}
+
+static unsigned long
+ReHashKey_nocase(const char *key, int Level)
+{
+    unsigned long a=31415, b=27183, vHash;
+    for (vHash=0; *key; key++, a*=b)
+	vHash = a*vHash*(unsigned long)Level + tolower(*key);
+    return vHash;
+}
+
 HAMT *
-HAMT_create(/*@exits@*/ void (*error_func)
+HAMT_create(int nocase, /*@exits@*/ void (*error_func)
     (const char *file, unsigned int line, const char *message))
 {
     /*@out@*/ HAMT *hamt = yasm_xmalloc(sizeof(HAMT));
@@ -98,10 +126,17 @@ HAMT_create(/*@exits@*/ void (*error_func)
 
     for (i=0; i<32; i++) {
 	hamt->root[i].BitMapKey = 0;
-	hamt->root[i].BaseValue = NULL;
+	hamt->root[i].BaseValue = 0;
     }
 
     hamt->error_func = error_func;
+    if (nocase) {
+	hamt->HashKey = HashKey_nocase;
+	hamt->ReHashKey = ReHashKey_nocase;
+    } else {
+	hamt->HashKey = HashKey;
+	hamt->ReHashKey = ReHashKey;
+    }
 
     return hamt;
 }
@@ -189,7 +224,7 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
     int keypartbits = 0;
     int level = 0;
 
-    key = HashKey(str);
+    key = hamt->HashKey(str);
     keypart = key & 0x1F;
     node = &hamt->root[keypart];
 
@@ -199,7 +234,7 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
 	entry->str = str;
 	entry->data = data;
 	STAILQ_INSERT_TAIL(&hamt->entries, entry, next);
-	node->BaseValue = entry;
+	SetValue(hamt, node, entry);
 	if (IsSubTrie(node))
 	    hamt->error_func(__FILE__, __LINE__,
 			     N_("Data is seen as subtrie (misaligned?)"));
@@ -230,9 +265,9 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
 		    keypartbits += 5;
 		    if (keypartbits > 30) {
 			/* Exceeded 32 bits: rehash */
-			key = ReHashKey(str, level);
-			key2 = ReHashKey(((HAMTEntry *)(node->BaseValue))->str,
-					 level);
+			key = hamt->ReHashKey(str, level);
+			key2 = hamt->ReHashKey(
+			    ((HAMTEntry *)(node->BaseValue))->str, level);
 			keypartbits = 0;
 		    }
 		    keypart = (key >> keypartbits) & 0x1F;
@@ -261,10 +296,10 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
 			if (keypart2 < keypart) {
 			    newnodes[0] = *node;    /* structure copy */
 			    newnodes[1].BitMapKey = key;
-			    newnodes[1].BaseValue = entry;
+			    SetValue(hamt, &newnodes[1], entry);
 			} else {
 			    newnodes[0].BitMapKey = key;
-			    newnodes[0].BaseValue = entry;
+			    SetValue(hamt, &newnodes[0], entry);
 			    newnodes[1] = *node;    /* structure copy */
 			}
 
@@ -282,7 +317,7 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
 	keypartbits += 5;
 	if (keypartbits > 30) {
 	    /* Exceeded 32 bits of current key: rehash */
-	    key = ReHashKey(str, level);
+	    key = hamt->ReHashKey(str, level);
 	    keypartbits = 0;
 	}
 	keypart = (key >> keypartbits) & 0x1F;
@@ -315,7 +350,7 @@ HAMT_insert(HAMT *hamt, const char *str, void *data, int *replace,
 	    entry->str = str;
 	    entry->data = data;
 	    STAILQ_INSERT_TAIL(&hamt->entries, entry, next);
-	    newnodes[Map].BaseValue = entry;
+	    SetValue(hamt, &newnodes[Map], entry);
 	    SetSubTrie(hamt, node, newnodes);
 
 	    *replace = 1;
@@ -341,7 +376,7 @@ HAMT_search(HAMT *hamt, const char *str)
     int keypartbits = 0;
     int level = 0;
     
-    key = HashKey(str);
+    key = hamt->HashKey(str);
     keypart = key & 0x1F;
     node = &hamt->root[keypart];
 
@@ -360,7 +395,7 @@ HAMT_search(HAMT *hamt, const char *str)
 	keypartbits += 5;
 	if (keypartbits > 30) {
 	    /* Exceeded 32 bits of current key: rehash */
-	    key = ReHashKey(str, level);
+	    key = hamt->ReHashKey(str, level);
 	    keypartbits = 0;
 	}
 	keypart = (key >> keypartbits) & 0x1F;
