@@ -3,11 +3,11 @@
  * \brief Hash Array Mapped Trie (HAMT) functions.
  *
  * \rcs
- * $Id: hamt.h 1495 2006-04-16 22:46:22Z peter $
+ * $Id: hamt.h 1876 2007-06-02 21:50:51Z peter $
  * \endrcs
  *
  * \license
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,10 +41,11 @@ typedef struct HAMTEntry HAMTEntry;
 
 /** Create new, empty, HAMT.  error_func() is called when an internal error is
  * encountered--it should NOT return to the calling function.
+ * \param   nocase	    nonzero if HAMT should be case-insensitive
  * \param   error_func	    function called on internal error
  * \return New, empty, hash array mapped trie.
  */
-HAMT *HAMT_create(/*@exits@*/ void (*error_func)
+HAMT *HAMT_create(int nocase, /*@exits@*/ void (*error_func)
     (const char *file, unsigned int line, const char *message));
 
 /** Delete HAMT and all data associated with it.  Uses deletefunc() to delete
