@@ -33,7 +33,7 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: mergesort.c 1591 2006-07-13 03:25:38Z peter $");
+/*@unused@*/ RCSID("$Id: mergesort.c 1795 2007-02-24 22:39:55Z peter $");
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)merge.c	8.2 (Berkeley) 2/14/94";
@@ -278,7 +278,8 @@ setup(unsigned char *list1, unsigned char *list2, size_t n, size_t size,
 {
 	size_t i;
 	unsigned int tmp;
-	int length, size2, sense;
+	int length, sense;
+	size_t size2;
 	unsigned char *f1, *f2, *s, *l2, *last, *p2;
 
 	size2 = size*2;
