@@ -3,11 +3,11 @@
  * \brief YASM parser module interface.
  *
  * \rcs
- * $Id: parser.h 1521 2006-05-04 07:55:32Z peter $
+ * $Id: parser.h 1876 2007-06-02 21:50:51Z peter $
  * \endrcs
  *
  * \license
- *  Copyright (C) 2001  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -54,23 +54,16 @@ typedef struct yasm_parser_module {
     /** Parse a source file into an object.
      * \param object	object to parse into (already created)
      * \param pp	preprocessor
-     * \param a		architecture; architecture-specific directives are
-     *			obtained from this.
-     * \param of	object format; objfmt-specific directives and segment
-     *			names are obtained from this.
      * \param f		initial starting file
-     * \param in_filename	initial starting file's filename
      * \param save_input	nonzero if the parser should save the original
      *				lines of source into the object's linemap (via
      *				yasm_linemap_add_data()).
-     * \param def_sect	default (starting) section in the object
      * \param errwarns	error/warning set
      * \note Parse errors and warnings are stored into errwarns.
      */
     void (*do_parse)
-	(yasm_object *object, yasm_preproc *pp, yasm_arch *a, yasm_objfmt *of,
-	 yasm_dbgfmt *df, FILE *f, const char *in_filename, int save_input,
-	 yasm_section *def_sect, yasm_errwarns *errwarns);
+	(yasm_object *object, yasm_preproc *pp, FILE *f, int save_input,
+	 yasm_linemap *linemap, yasm_errwarns *errwarns);
 } yasm_parser_module;
 
 #endif
