@@ -1,7 +1,7 @@
 /*
  * Section utility functions
  *
- *  Copyright (C) 2001-2005  Peter Johnson
+ *  Copyright (C) 2001-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,12 +26,13 @@
  */
 #define YASM_LIB_INTERNAL
 #include "util.h"
-/*@unused@*/ RCSID("$Id: section.c 1786 2007-02-24 04:16:05Z peter $");
+/*@unused@*/ RCSID("$Id: section.c 1876 2007-06-02 21:50:51Z peter $");
 
 #include <limits.h>
 
 #include "libyasm-stdint.h"
 #include "coretype.h"
+#include "hamt.h"
 #include "valparam.h"
 #include "assocdat.h"
 
@@ -45,6 +46,8 @@
 #include "bytecode.h"
 #include "arch.h"
 #include "section.h"
+
+#include "dbgfmt.h"
 #include "objfmt.h"
 
 #include "expr-int.h"
@@ -53,36 +56,12 @@
 #include "inttree.h"
 
 
-struct yasm_object {
-    /*@owned@*/ char *src_filename;
-    /*@owned@*/ char *obj_filename;
-
-    yasm_symtab	*symtab;
-    yasm_linemap *linemap;
-
-    /*@reldef@*/ STAILQ_HEAD(yasm_sectionhead, yasm_section) sections;
-};
-
 struct yasm_section {
     /*@reldef@*/ STAILQ_ENTRY(yasm_section) link;
 
     /*@dependent@*/ yasm_object *object;    /* Pointer to parent object */
 
-    enum { SECTION_GENERAL, SECTION_ABSOLUTE } type;
-
-    union {
-	/* SECTION_GENERAL data */
-	struct {
-	    /*@owned@*/ char *name;	/* strdup()'ed name (given by user) */
-	} general;
-	/* SECTION_ABSOLUTE data */
-	struct {
-	    /* Internally created first symrec in section.  Used by
-	     * yasm_expr__level_tree during absolute reference expansion.
-	     */
-	    /*@dependent@*/ yasm_symrec *first;
-	} absolute;
-    } data;
+    /*@owned@*/ char *name;     /* strdup()'ed name (given by user) */
 
     /* associated data; NULL if none */
     /*@null@*/ /*@only@*/ yasm__assoc_data *assoc_data;
@@ -109,24 +88,216 @@ struct yasm_section {
 
 static void yasm_section_destroy(/*@only@*/ yasm_section *sect);
 
+/* Wrapper around directive for HAMT insertion */
+typedef struct yasm_directive_wrap {
+    const yasm_directive *directive;
+} yasm_directive_wrap;
+
+/*
+ * Standard "builtin" object directives.
+ */
+
+static void
+dir_extern(yasm_object *object, yasm_valparamhead *valparams,
+	   yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_valparam *vp = yasm_vps_first(valparams);
+    yasm_symrec *sym;
+    sym = yasm_symtab_declare(object->symtab, yasm_vp_id(vp), YASM_SYM_EXTERN,
+                              line);
+    if (objext_valparams) {
+	yasm_valparamhead *vps = yasm_vps_create();
+	*vps = *objext_valparams;   /* structure copy */
+	yasm_vps_initialize(objext_valparams);	/* don't double-free */
+	yasm_symrec_set_objext_valparams(sym, vps);
+    }
+}
+
+static void
+dir_global(yasm_object *object, yasm_valparamhead *valparams,
+	   yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_valparam *vp = yasm_vps_first(valparams);
+    yasm_symrec *sym;
+    sym = yasm_symtab_declare(object->symtab, yasm_vp_id(vp), YASM_SYM_GLOBAL,
+                              line);
+    if (objext_valparams) {
+	yasm_valparamhead *vps = yasm_vps_create();
+	*vps = *objext_valparams;   /* structure copy */
+	yasm_vps_initialize(objext_valparams);	/* don't double-free */
+	yasm_symrec_set_objext_valparams(sym, vps);
+    }
+}
+
+static void
+dir_common(yasm_object *object, yasm_valparamhead *valparams,
+	   yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_valparam *vp = yasm_vps_first(valparams);
+    yasm_valparam *vp2 = yasm_vps_next(vp);
+    yasm_expr *size = yasm_vp_expr(vp2, object->symtab, line);
+    yasm_symrec *sym;
+
+    if (!size) {
+	yasm_error_set(YASM_ERROR_SYNTAX,
+		       N_("no size specified in %s declaration"), "COMMON");
+	return;
+    }
+    sym = yasm_symtab_declare(object->symtab, yasm_vp_id(vp), YASM_SYM_COMMON,
+                              line);
+    yasm_symrec_set_common_size(sym, size);
+    if (objext_valparams) {
+	yasm_valparamhead *vps = yasm_vps_create();
+	*vps = *objext_valparams;   /* structure copy */
+	yasm_vps_initialize(objext_valparams);	/* don't double-free */
+	yasm_symrec_set_objext_valparams(sym, vps);
+    }
+}
+
+static void
+dir_section(yasm_object *object, yasm_valparamhead *valparams,
+	    yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_section *new_section =
+	yasm_objfmt_section_switch(object, valparams, objext_valparams, line);
+    if (new_section)
+	object->cur_section = new_section;
+    else
+	yasm_error_set(YASM_ERROR_SYNTAX,
+		       N_("invalid argument to directive `%s'"), "SECTION");
+}
+
+static const yasm_directive object_directives[] = {
+    { ".extern",	"gas",	dir_extern,	YASM_DIR_ID_REQUIRED },
+    { ".global",	"gas",	dir_global,	YASM_DIR_ID_REQUIRED },
+    { ".globl",		"gas",	dir_global,	YASM_DIR_ID_REQUIRED },
+    { "extern",		"nasm",	dir_extern,	YASM_DIR_ID_REQUIRED },
+    { "global",		"nasm",	dir_global,	YASM_DIR_ID_REQUIRED },
+    { "common",		"nasm",	dir_common,	YASM_DIR_ID_REQUIRED },
+    { "section",	"nasm",	dir_section,	YASM_DIR_ARG_REQUIRED },
+    { "segment",	"nasm",	dir_section,	YASM_DIR_ARG_REQUIRED },
+    { NULL, NULL, NULL, 0 }
+};
+
+static void
+directive_level2_delete(/*@only@*/ void *data)
+{
+    yasm_xfree(data);
+}
+
+static void
+directive_level1_delete(/*@only@*/ void *data)
+{
+    HAMT_destroy(data, directive_level2_delete);
+}
+
+static void
+directives_add(yasm_object *object, /*@null@*/ const yasm_directive *dir)
+{
+    if (!dir)
+	return;
+
+    while (dir->name) {
+	HAMT *level2 = HAMT_search(object->directives, dir->parser);
+	int replace;
+	yasm_directive_wrap *wrap = yasm_xmalloc(sizeof(yasm_directive_wrap));
+
+	if (!level2) {
+	    replace = 0;
+	    level2 = HAMT_insert(object->directives, dir->parser,
+				 HAMT_create(1, yasm_internal_error_),
+				 &replace, directive_level1_delete);
+	}
+	replace = 0;
+	wrap->directive = dir;
+	HAMT_insert(level2, dir->name, wrap, &replace,
+		    directive_level2_delete);
+	dir++;
+    }
+}
 
 /*@-compdestroy@*/
 yasm_object *
-yasm_object_create(const char *src_filename, const char *obj_filename)
+yasm_object_create(const char *src_filename, const char *obj_filename,
+		   /*@kept@*/ yasm_arch *arch,
+		   const yasm_objfmt_module *objfmt_module,
+		   const yasm_dbgfmt_module *dbgfmt_module)
 {
     yasm_object *object = yasm_xmalloc(sizeof(yasm_object));
+    int matched, i;
 
     object->src_filename = yasm__xstrdup(src_filename);
     object->obj_filename = yasm__xstrdup(obj_filename);
 
-    /* Create empty symtab and linemap */
+    /* Create empty symbol table */
     object->symtab = yasm_symtab_create();
-    object->linemap = yasm_linemap_create();
 
     /* Initialize sections linked list */
     STAILQ_INIT(&object->sections);
 
+    /* Create directives HAMT */
+    object->directives = HAMT_create(1, yasm_internal_error_);
+
+    /* Initialize the target architecture */
+    object->arch = arch;
+
+    /* Initialize things to NULL in case of error */
+    object->dbgfmt = NULL;
+
+    /* Initialize the object format */
+    object->objfmt = yasm_objfmt_create(objfmt_module, object);
+    if (!object->objfmt) {
+	yasm_error_set(YASM_ERROR_GENERAL,
+	    N_("object format `%s' does not support architecture `%s' machine `%s'"),
+	    objfmt_module->keyword, ((yasm_arch_base *)arch)->module->keyword,
+	    yasm_arch_get_machine(arch));
+	goto error;
+    }
+
+    /* Get a fresh copy of objfmt_module as it may have changed. */
+    objfmt_module = ((yasm_objfmt_base *)object->objfmt)->module;
+
+    /* Add an initial "default" section to object */
+    object->cur_section = yasm_objfmt_add_default_section(object);
+
+    /* Check to see if the requested debug format is in the allowed list
+     * for the active object format.
+     */
+    matched = 0;
+    for (i=0; objfmt_module->dbgfmt_keywords[i]; i++)
+	if (yasm__strcasecmp(objfmt_module->dbgfmt_keywords[i],
+			     dbgfmt_module->keyword) == 0)
+	    matched = 1;
+    if (!matched) {
+	yasm_error_set(YASM_ERROR_GENERAL,
+	    N_("`%s' is not a valid debug format for object format `%s'"),
+	    dbgfmt_module->keyword, objfmt_module->keyword);
+	goto error;
+    }
+
+    /* Initialize the debug format */
+    object->dbgfmt = yasm_dbgfmt_create(dbgfmt_module, object);
+    if (!object->dbgfmt) {
+	yasm_error_set(YASM_ERROR_GENERAL,
+	    N_("debug format `%s' does not work with object format `%s'"),
+	    dbgfmt_module->keyword, objfmt_module->keyword);
+	goto error;
+    }
+
+    /* Add directives to HAMT.  Note ordering here determines priority. */
+    directives_add(object,
+		   ((yasm_objfmt_base *)object->objfmt)->module->directives);
+    directives_add(object,
+		   ((yasm_dbgfmt_base *)object->dbgfmt)->module->directives);
+    directives_add(object,
+		   ((yasm_arch_base *)object->arch)->module->directives);
+    directives_add(object, object_directives);
+
     return object;
+
+error:
+    yasm_object_destroy(object);
+    return NULL;
 }
 /*@=compdestroy@*/
 
@@ -143,8 +314,7 @@ yasm_object_get_general(yasm_object *object, const char *name,
      * that name.
      */
     STAILQ_FOREACH(s, &object->sections, link) {
-	if (s->type == SECTION_GENERAL &&
-	    strcmp(s->data.general.name, name) == 0) {
+	if (strcmp(s->name, name) == 0) {
 	    *isnew = 0;
 	    return s;
 	}
@@ -157,8 +327,7 @@ yasm_object_get_general(yasm_object *object, const char *name,
     STAILQ_INSERT_TAIL(&object->sections, s, link);
 
     s->object = object;
-    s->type = SECTION_GENERAL;
-    s->data.general.name = yasm__xstrdup(name);
+    s->name = yasm__xstrdup(name);
     s->assoc_data = NULL;
     if (start)
 	s->start = start;
@@ -190,44 +359,27 @@ yasm_object_get_general(yasm_object *object, const char *name,
 }
 /*@=onlytrans@*/
 
-/*@-onlytrans@*/
-yasm_section *
-yasm_object_create_absolute(yasm_object *object, yasm_expr *start,
-			    unsigned long line)
+int
+yasm_object_directive(yasm_object *object, const char *name,
+		      const char *parser, yasm_valparamhead *valparams,
+		      yasm_valparamhead *objext_valparams,
+		      unsigned long line)
 {
-    yasm_section *s;
-    yasm_bytecode *bc;
+    HAMT *level2;
+    yasm_directive_wrap *wrap;
 
-    s = yasm_xcalloc(1, sizeof(yasm_section));
-    STAILQ_INSERT_TAIL(&object->sections, s, link);
+    level2 = HAMT_search(object->directives, parser);
+    if (!level2)
+	return 1;
 
-    s->object = object;
-    s->type = SECTION_ABSOLUTE;
-    s->start = start;
+    wrap = HAMT_search(level2, name);
+    if (!wrap)
+	return 1;
 
-    /* Initialize bytecodes with one empty bytecode (acts as "prior" for first
-     * real bytecode in section.
-     */
-    STAILQ_INIT(&s->bcs);
-    bc = yasm_bc_create_common(NULL, NULL, 0);
-    bc->section = s;
-    bc->offset = 0;
-    STAILQ_INSERT_TAIL(&s->bcs, bc, link);
-
-    /* Initialize relocs */
-    STAILQ_INIT(&s->relocs);
-    s->destroy_reloc = NULL;
-
-    s->data.absolute.first =
-	yasm_symtab_define_label(object->symtab, ".absstart", bc, 0, 0);
-
-    s->code = 0;
-    s->res_only = 1;
-    s->def = 0;
-
-    return s;
+    yasm_call_directive(wrap->directive, object, valparams, objext_valparams,
+			line);
+    return 0;
 }
-/*@=onlytrans@*/
 
 void
 yasm_object_set_source_fn(yasm_object *object, const char *src_filename)
@@ -236,36 +388,6 @@ yasm_object_set_source_fn(yasm_object *object, const char *src_filename)
     object->src_filename = yasm__xstrdup(src_filename);
 }
 
-const char *
-yasm_object_get_source_fn(const yasm_object *object)
-{
-    return object->src_filename;
-}
-
-const char *
-yasm_object_get_object_fn(const yasm_object *object)
-{
-    return object->obj_filename;
-}
-
-yasm_symtab *
-yasm_object_get_symtab(const yasm_object *object)
-{
-    return object->symtab;
-}
-
-yasm_linemap *
-yasm_object_get_linemap(const yasm_object *object)
-{
-    return object->linemap;
-}
-
-int
-yasm_section_is_absolute(yasm_section *sect)
-{
-    return (sect->type == SECTION_ABSOLUTE);
-}
-
 int
 yasm_section_is_code(yasm_section *sect)
 {
@@ -321,6 +443,14 @@ yasm_object_destroy(yasm_object *object)
 {
     yasm_section *cur, *next;
 
+    /* Delete object format, debug format, and arch.  This can be called
+     * due to an error in yasm_object_create(), so look out for NULLs.
+     */
+    if (object->objfmt)
+	yasm_objfmt_destroy(object->objfmt);
+    if (object->dbgfmt)
+	yasm_dbgfmt_destroy(object->dbgfmt);
+
     /* Delete sections */
     cur = STAILQ_FIRST(&object->sections);
     while (cur) {
@@ -329,13 +459,19 @@ yasm_object_destroy(yasm_object *object)
 	cur = next;
     }
 
+    /* Delete directives HAMT */
+    HAMT_destroy(object->directives, directive_level1_delete);
+
     /* Delete associated filenames */
     yasm_xfree(object->src_filename);
     yasm_xfree(object->obj_filename);
 
-    /* Delete symbol table and line mappings */
+    /* Delete symbol table */
     yasm_symtab_destroy(object->symtab);
-    yasm_linemap_destroy(object->linemap);
+
+    /* Delete architecture */
+    if (object->arch)
+	yasm_arch_destroy(object->arch);
 
     yasm_xfree(object);
 }
@@ -403,8 +539,7 @@ yasm_object_find_general(yasm_object *object, const char *name)
     yasm_section *cur;
 
     STAILQ_FOREACH(cur, &object->sections, link) {
-	if (cur->type == SECTION_GENERAL &&
-	    strcmp(cur->data.general.name, name) == 0)
+	if (strcmp(cur->name, name) == 0)
 	    return cur;
     }
     return NULL;
@@ -496,17 +631,7 @@ yasm_section_bcs_traverse(yasm_section *sect,
 const char *
 yasm_section_get_name(const yasm_section *sect)
 {
-    if (sect->type == SECTION_GENERAL)
-	return sect->data.general.name;
-    return NULL;
-}
-
-yasm_symrec *
-yasm_section_abs_get_sym(const yasm_section *sect)
-{
-    if (sect->type == SECTION_ABSOLUTE)
-	return sect->data.absolute.first;
-    return NULL;
+    return sect->name;
 }
 
 void
@@ -545,9 +670,7 @@ yasm_section_destroy(yasm_section *sect)
     if (!sect)
 	return;
 
-    if (sect->type == SECTION_GENERAL) {
-	yasm_xfree(sect->data.general.name);
-    }
+    yasm_xfree(sect->name);
     yasm__assoc_data_destroy(sect->assoc_data);
     yasm_expr_destroy(sect->start);
 
@@ -580,17 +703,7 @@ yasm_section_print(const yasm_section *sect, FILE *f, int indent_level,
 	return;
     }
 
-    fprintf(f, "%*stype=", indent_level, "");
-    switch (sect->type) {
-	case SECTION_GENERAL:
-	    fprintf(f, "general\n%*sname=%s\n", indent_level, "",
-		    sect->data.general.name);
-	    break;
-	case SECTION_ABSOLUTE:
-	    fprintf(f, "absolute\n");
-	    break;
-    }
-
+    fprintf(f, "%*sname=%s\n", indent_level, "", sect->name);
     fprintf(f, "%*sstart=", indent_level, "");
     yasm_expr_print(sect->start, f);
     fprintf(f, "\n");
@@ -1145,8 +1258,7 @@ optimize_term_expand(IntervalTreeNode *node, void *d)
 }
 
 void
-yasm_object_optimize(yasm_object *object, yasm_arch *arch,
-		     yasm_errwarns *errwarns)
+yasm_object_optimize(yasm_object *object, yasm_errwarns *errwarns)
 {
     yasm_section *sect;
     unsigned long bc_index = 0;
