@@ -1,7 +1,7 @@
 /*
  * CodeView debugging formats implementation for Yasm
  *
- *  Copyright (C) 2006  Peter Johnson
+ *  Copyright (C) 2006-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-dbgfmt.c 1598 2006-08-10 04:02:59Z peter $");
+/*@unused@*/ RCSID("$Id: cv-dbgfmt.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -37,19 +37,13 @@ yasm_dbgfmt_module yasm_cv8_LTX_dbgfmt;
 
 
 static /*@null@*/ /*@only@*/ yasm_dbgfmt *
-cv_dbgfmt_create(yasm_object *object, yasm_objfmt *of, yasm_arch *a,
-		 yasm_dbgfmt_module *module, int version)
+cv_dbgfmt_create(yasm_object *object, yasm_dbgfmt_module *module, int version)
 {
     yasm_dbgfmt_cv *dbgfmt_cv = yasm_xmalloc(sizeof(yasm_dbgfmt_cv));
     size_t i;
 
     dbgfmt_cv->dbgfmt.module = module;
 
-    dbgfmt_cv->object = object;
-    dbgfmt_cv->symtab = yasm_object_get_symtab(object);
-    dbgfmt_cv->linemap = yasm_object_get_linemap(object);
-    dbgfmt_cv->arch = a;
-
     dbgfmt_cv->filenames_allocated = 32;
     dbgfmt_cv->filenames_size = 0;
     dbgfmt_cv->filenames =
@@ -67,9 +61,9 @@ cv_dbgfmt_create(yasm_object *object, yasm_objfmt *of, yasm_arch *a,
 }
 
 static /*@null@*/ /*@only@*/ yasm_dbgfmt *
-cv8_dbgfmt_create(yasm_object *object, yasm_objfmt *of, yasm_arch *a)
+cv8_dbgfmt_create(yasm_object *object)
 {
-    return cv_dbgfmt_create(object, of, a, &yasm_cv8_LTX_dbgfmt, 8);
+    return cv_dbgfmt_create(object, &yasm_cv8_LTX_dbgfmt, 8);
 }
 
 static void
@@ -98,29 +92,19 @@ yasm_cv__append_bc(yasm_section *sect, yasm_bytecode *bc)
 }
 
 static void
-cv_dbgfmt_generate(yasm_dbgfmt *dbgfmt, yasm_errwarns *errwarns)
+cv_dbgfmt_generate(yasm_object *object, yasm_linemap *linemap,
+		   yasm_errwarns *errwarns)
 {
-    yasm_dbgfmt_cv *dbgfmt_cv = (yasm_dbgfmt_cv *)dbgfmt;
-
-    yasm_cv__generate_symline(dbgfmt_cv, errwarns);
-
-    yasm_cv__generate_type(dbgfmt_cv);
-}
-
-static int
-cv_dbgfmt_directive(yasm_dbgfmt *dbgfmt, const char *name,
-		    yasm_section *sect, yasm_valparamhead *valparams,
-		    unsigned long line)
-{
-    return 1;	    /* TODO */
+    yasm_cv__generate_symline(object, linemap, errwarns);
+    yasm_cv__generate_type(object);
 }
 
 /* Define dbgfmt structure -- see dbgfmt.h for details */
 yasm_dbgfmt_module yasm_cv8_LTX_dbgfmt = {
     "CodeView debugging format for VC8",
     "cv8",
+    NULL,   /* no directives */
     cv8_dbgfmt_create,
     cv_dbgfmt_destroy,
-    cv_dbgfmt_directive,
     cv_dbgfmt_generate
 };
