@@ -1,7 +1,7 @@
 /*
  * CodeView debugging format - symbol and line information
  *
- *  Copyright (C) 2006  Peter Johnson
+ *  Copyright (C) 2006-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-symline.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: cv-symline.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -107,7 +107,6 @@ enum cv_symtype {
 };
 
 typedef struct cv8_symhead {
-    yasm_dbgfmt_cv *dbgfmt_cv;
     enum cv8_symheadtype type;
     yasm_bytecode *start_prevbc;
     yasm_bytecode *end_prevbc;
@@ -115,7 +114,6 @@ typedef struct cv8_symhead {
 } cv8_symhead;
 
 typedef struct cv8_fileinfo {
-    yasm_dbgfmt_cv *dbgfmt_cv;
     const cv_filename *fn;
 } cv8_fileinfo;
 
@@ -134,7 +132,6 @@ typedef struct cv8_lineset {
 
 typedef struct cv8_lineinfo {
     STAILQ_ENTRY(cv8_lineinfo) link;
-    yasm_dbgfmt_cv *dbgfmt_cv;
     const cv_filename *fn;	/* filename associated with line numbers */
     yasm_section *sect;		/* section line numbers are for */
     yasm_symrec *sectsym;	/* symbol for beginning of sect */
@@ -153,7 +150,6 @@ typedef struct cv8_lineinfo {
  * 'Z' : 0-terminated string (pointer)
  */
 typedef struct cv_sym {
-    yasm_dbgfmt_cv *dbgfmt_cv;
     enum cv_symtype type;
     const char *format;
     union {
@@ -243,24 +239,21 @@ static const yasm_bytecode_callback cv_sym_bc_callback = {
     0
 };
 
-static cv8_symhead *cv8_add_symhead(yasm_dbgfmt_cv *dbgfmt_cv,
-				    yasm_section *sect, unsigned long type,
+static cv8_symhead *cv8_add_symhead(yasm_section *sect, unsigned long type,
 				    int first);
 static void cv8_set_symhead_end(cv8_symhead *head, yasm_bytecode *end_prevbc);
 
 static yasm_bytecode *cv8_add_fileinfo
-    (yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect, const cv_filename *fn);
+    (yasm_section *sect, const cv_filename *fn);
 
 static unsigned long cv_sym_size(const cv_sym *cvs);
 
 
 static cv_sym *
-cv8_add_sym_objname(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
-		    /*@keep@*/ char *objname)
+cv8_add_sym_objname(yasm_section *sect, /*@keep@*/ char *objname)
 {
     yasm_bytecode *bc;
     cv_sym *cvs = yasm_xmalloc(sizeof(cv_sym));
-    cvs->dbgfmt_cv = dbgfmt_cv;
     cvs->type = CV8_S_OBJNAME;
     cvs->format = "wZ";
     cvs->args[0].i = 0;		/* signature (0=asm) */
@@ -273,19 +266,18 @@ cv8_add_sym_objname(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
 }
 
 static cv_sym *
-cv8_add_sym_compile(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
+cv8_add_sym_compile(yasm_object *object, yasm_section *sect,
 		    /*@keep@*/ char *creator)
 {
     yasm_bytecode *bc;
     cv_sym *cvs = yasm_xmalloc(sizeof(cv_sym));
-    cvs->dbgfmt_cv = dbgfmt_cv;
     cvs->type = CV8_S_COMPILE;
     cvs->format = "wwwwZh";
     cvs->args[0].i = 3;		/* language (3=Masm) */
 
     /* target processor; 0xD0 = AMD64 */
-    if (strcmp(yasm_arch_keyword(dbgfmt_cv->arch), "x86") == 0) {
-	if (strcmp(yasm_arch_get_machine(dbgfmt_cv->arch), "amd64") == 0)
+    if (strcmp(yasm_arch_keyword(object->arch), "x86") == 0) {
+	if (strcmp(yasm_arch_get_machine(object->arch), "amd64") == 0)
 	    cvs->args[1].i = 0xD0;
 	else
 	    cvs->args[1].i = 0x6;	/* 686, FIXME */
@@ -304,12 +296,10 @@ cv8_add_sym_compile(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
 }
 
 static cv_sym *
-cv8_add_sym_label(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
-		  yasm_symrec *sym)
+cv8_add_sym_label(yasm_section *sect, yasm_symrec *sym)
 {
     yasm_bytecode *bc;
     cv_sym *cvs = yasm_xmalloc(sizeof(cv_sym));
-    cvs->dbgfmt_cv = dbgfmt_cv;
     cvs->type = CV8_S_LABEL32;
     cvs->format = "YbZ";
     cvs->args[0].p = sym;	/* symrec for label */
@@ -323,12 +313,11 @@ cv8_add_sym_label(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
 }
 
 static cv_sym *
-cv8_add_sym_data(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
-		 unsigned long type, yasm_symrec *sym, int is_global)
+cv8_add_sym_data(yasm_section *sect, unsigned long type, yasm_symrec *sym,
+		 int is_global)
 {
     yasm_bytecode *bc;
     cv_sym *cvs = yasm_xmalloc(sizeof(cv_sym));
-    cvs->dbgfmt_cv = dbgfmt_cv;
     cvs->type = is_global ? CV8_S_GDATA32 : CV8_S_LDATA32;
     cvs->format = "wYZ";
     cvs->args[0].i = type;	/* type index */
@@ -423,7 +412,9 @@ cv_append_str(yasm_section *sect, const char *str)
 
 typedef struct cv_line_info {
     yasm_section *debug_symline;
+    yasm_object *object;
     yasm_dbgfmt_cv *dbgfmt_cv;
+    yasm_linemap *linemap;
     yasm_errwarns *errwarns;
     unsigned int num_lineinfos;
     STAILQ_HEAD(, cv8_lineinfo) cv8_lineinfos;
@@ -445,7 +436,7 @@ cv_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
     if (nextbc && bc->offset == nextbc->offset)
 	return 0;
 
-    yasm_linemap_lookup(dbgfmt_cv->linemap, bc->line, &filename, &line);
+    yasm_linemap_lookup(info->linemap, bc->line, &filename, &line);
 
     if (!info->cv8_cur_li
 	|| strcmp(filename, info->cv8_cur_li->fn->filename) != 0) {
@@ -477,7 +468,6 @@ cv_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 
 	/* and create new lineinfo structure */
 	info->cv8_cur_li = yasm_xmalloc(sizeof(cv8_lineinfo));
-	info->cv8_cur_li->dbgfmt_cv = dbgfmt_cv;
 	info->cv8_cur_li->fn = &dbgfmt_cv->filenames[i];
 	info->cv8_cur_li->sect = sect;
 	sectbc = yasm_section_bcs_first(sect);
@@ -486,7 +476,7 @@ cv_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 	else {
 	    sprintf(symname, ".%06u", info->num_lineinfos++);
 	    info->cv8_cur_li->sectsym =
-		yasm_symtab_define_label(dbgfmt_cv->symtab, symname, sectbc,
+		yasm_symtab_define_label(info->object->symtab, symname, sectbc,
 					 1, 0);
 	}
 	info->cv8_cur_li->num_linenums = 0;
@@ -532,8 +522,7 @@ cv_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
 static int
 cv_generate_filename(const char *filename, void *d)
 {
-    yasm_dbgfmt_cv *dbgfmt_cv = (yasm_dbgfmt_cv *)d;
-    cv_dbgfmt_add_file(dbgfmt_cv, 0, filename);
+    cv_dbgfmt_add_file((yasm_dbgfmt_cv *)d, 0, filename);
     return 0;
 }
 
@@ -553,16 +542,18 @@ cv_generate_sym(yasm_symrec *sym, void *d)
 
     /* TODO: add data types; until then, just mark everything as UBYTE */
     if (yasm_section_is_code(yasm_bc_get_section(precbc)))
-	cv8_add_sym_label(info->dbgfmt_cv, info->debug_symline, sym);
+	cv8_add_sym_label(info->debug_symline, sym);
     else
-	cv8_add_sym_data(info->dbgfmt_cv, info->debug_symline, 0x20, sym,
+	cv8_add_sym_data(info->debug_symline, 0x20, sym,
 	    yasm_symrec_get_visibility(sym) & YASM_SYM_GLOBAL?1:0);
     return 0;
 }
 
 yasm_section *
-yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
+yasm_cv__generate_symline(yasm_object *object, yasm_linemap *linemap,
+			  yasm_errwarns *errwarns)
 {
+    yasm_dbgfmt_cv *dbgfmt_cv = (yasm_dbgfmt_cv *)object->dbgfmt;
     cv_line_info info;
     int new;
     size_t i;
@@ -572,21 +563,22 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
     unsigned long off;
 
     /* Generate filenames based on linemap */
-    yasm_linemap_traverse_filenames(dbgfmt_cv->linemap, dbgfmt_cv,
+    yasm_linemap_traverse_filenames(linemap, dbgfmt_cv,
 				    cv_generate_filename);
 
+    info.object = object;
     info.dbgfmt_cv = dbgfmt_cv;
+    info.linemap = linemap;
     info.errwarns = errwarns;
-    info.debug_symline = yasm_object_get_general(dbgfmt_cv->object,
-						 ".debug$S", 0, 1, 0, 0, &new,
-						 0);
+    info.debug_symline =
+	yasm_object_get_general(object, ".debug$S", 0, 1, 0, 0, &new, 0);
     info.num_lineinfos = 0;
     STAILQ_INIT(&info.cv8_lineinfos);
     info.cv8_cur_li = NULL;
     info.cv8_cur_ls = NULL;
 
     /* source filenames string table */
-    head = cv8_add_symhead(dbgfmt_cv, info.debug_symline, CV8_FILE_STRTAB, 1);
+    head = cv8_add_symhead(info.debug_symline, CV8_FILE_STRTAB, 1);
     cv_append_str(info.debug_symline, "");
     off = 1;
     for (i=0; i<dbgfmt_cv->filenames_size; i++) {
@@ -611,13 +603,12 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
     yasm_bc_calc_len(bc, NULL, NULL);
 
     /* source file info table */
-    head = cv8_add_symhead(dbgfmt_cv, info.debug_symline, CV8_FILE_INFO, 0);
+    head = cv8_add_symhead(info.debug_symline, CV8_FILE_INFO, 0);
     off = 0;
     for (i=0; i<dbgfmt_cv->filenames_size; i++) {
 	if (!dbgfmt_cv->filenames[i].pathname)
 	    continue;
-	bc = cv8_add_fileinfo(dbgfmt_cv, info.debug_symline,
-			      &dbgfmt_cv->filenames[i]);
+	bc = cv8_add_fileinfo(info.debug_symline, &dbgfmt_cv->filenames[i]);
 	dbgfmt_cv->filenames[i].info_off = off;
 	off += bc->len;
     }
@@ -626,13 +617,12 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
     /* Already aligned 4 */
 
     /* Generate line numbers for sections */
-    yasm_object_sections_traverse(dbgfmt_cv->object, (void *)&info,
+    yasm_object_sections_traverse(object, (void *)&info,
 				  cv_generate_line_section);
 
     /* Output line numbers for sections */
     STAILQ_FOREACH(li, &info.cv8_lineinfos, link) {
-	head = cv8_add_symhead(dbgfmt_cv, info.debug_symline, CV8_LINE_NUMS,
-			       0);
+	head = cv8_add_symhead(info.debug_symline, CV8_LINE_NUMS, 0);
 	bc = yasm_bc_create_common(&cv8_lineinfo_bc_callback, li, 0);
 	bc->len = 24+li->num_linenums*8;
 	yasm_cv__append_bc(info.debug_symline, bc);
@@ -642,15 +632,15 @@ yasm_cv__generate_symline(yasm_dbgfmt_cv *dbgfmt_cv, yasm_errwarns *errwarns)
     /* Already aligned 4 */
 
     /* Output debugging symbols */
-    head = cv8_add_symhead(dbgfmt_cv, info.debug_symline, CV8_DEBUG_SYMS, 0);
+    head = cv8_add_symhead(info.debug_symline, CV8_DEBUG_SYMS, 0);
     /* add object and compile flag first */
-    cv8_add_sym_objname(dbgfmt_cv, info.debug_symline,
-	yasm__abspath(yasm_object_get_object_fn(dbgfmt_cv->object)));
-    cv8_add_sym_compile(dbgfmt_cv, info.debug_symline,
+    cv8_add_sym_objname(info.debug_symline,
+			yasm__abspath(object->obj_filename));
+    cv8_add_sym_compile(object, info.debug_symline,
 			yasm__xstrdup(PACKAGE_NAME " " PACKAGE_INTVER "."
 				      PACKAGE_BUILD));
     /* then iterate through symbol table */
-    yasm_symtab_traverse(dbgfmt_cv->symtab, &info, cv_generate_sym);
+    yasm_symtab_traverse(object->symtab, &info, cv_generate_sym);
     cv8_set_symhead_end(head, yasm_section_bcs_last(info.debug_symline));
 
     /* Align 4 at end */
@@ -683,14 +673,12 @@ cv_out_sym(yasm_symrec *sym, unsigned long off, yasm_bytecode *bc,
 }
 
 static cv8_symhead *
-cv8_add_symhead(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
-		unsigned long type, int first)
+cv8_add_symhead(yasm_section *sect, unsigned long type, int first)
 {
     cv8_symhead *head;
     yasm_bytecode *bc;
 
     head = yasm_xmalloc(sizeof(cv8_symhead));
-    head->dbgfmt_cv = dbgfmt_cv;
     head->type = type;
     head->first = first;
     head->start_prevbc = yasm_section_bcs_last(sect);
@@ -738,28 +726,28 @@ cv8_symhead_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 		       yasm_output_value_func output_value,
 		       yasm_output_reloc_func output_reloc)
 {
+    yasm_object *object = yasm_section_get_object(bc->section);
     cv8_symhead *head = (cv8_symhead *)bc->contents;
-    yasm_dbgfmt_cv *dbgfmt_cv = head->dbgfmt_cv;
     unsigned char *buf = *bufp;
     yasm_intnum *intn, *cval;
 
     cval = yasm_intnum_create_uint(4);
     /* Output "version" if first */
     if (head->first) {
-	yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+	yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
 	buf += 4;
     }
 
     /* Type contained - 4 bytes */
     yasm_intnum_set_uint(cval, head->type);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Total length of info (following this field) - 4 bytes */
     yasm_intnum_set_uint(cval, bc->len);
     intn = yasm_calc_bc_dist(head->start_prevbc, head->end_prevbc);
     yasm_intnum_calc(intn, YASM_EXPR_SUB, cval);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, intn, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, intn, buf, 4, 32, 0, bc, 0);
     buf += 4;
     yasm_intnum_destroy(intn);
 
@@ -770,14 +758,12 @@ cv8_symhead_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 }
 
 static yasm_bytecode *
-cv8_add_fileinfo(yasm_dbgfmt_cv *dbgfmt_cv, yasm_section *sect,
-		 const cv_filename *fn)
+cv8_add_fileinfo(yasm_section *sect, const cv_filename *fn)
 {
     cv8_fileinfo *fi;
     yasm_bytecode *bc;
 
     fi = yasm_xmalloc(sizeof(cv8_fileinfo));
-    fi->dbgfmt_cv = dbgfmt_cv;
     fi->fn = fn;
 
     bc = yasm_bc_create_common(&cv8_fileinfo_bc_callback, fi, 0);
@@ -813,20 +799,20 @@ cv8_fileinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 			yasm_output_value_func output_value,
 			yasm_output_reloc_func output_reloc)
 {
+    yasm_object *object = yasm_section_get_object(bc->section);
     cv8_fileinfo *fi = (cv8_fileinfo *)bc->contents;
-    yasm_dbgfmt_cv *dbgfmt_cv = fi->dbgfmt_cv;
     unsigned char *buf = *bufp;
     yasm_intnum *cval;
     int i;
 
     /* Offset in filename string table */
     cval = yasm_intnum_create_uint(fi->fn->str_off);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Checksum type/length */
     yasm_intnum_set_uint(cval, 0x0110);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 2, 16, 0, bc, 0);
     buf += 2;
 
     /* Checksum */
@@ -880,8 +866,8 @@ cv8_lineinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 			yasm_output_value_func output_value,
 			yasm_output_reloc_func output_reloc)
 {
+    yasm_object *object = yasm_section_get_object(bc->section);
     cv8_lineinfo *li = (cv8_lineinfo *)bc->contents;
-    yasm_dbgfmt_cv *dbgfmt_cv = li->dbgfmt_cv;
     unsigned char *buf = *bufp;
     yasm_intnum *cval;
     unsigned long i;
@@ -897,22 +883,22 @@ cv8_lineinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     /* Section length covered by line number info */
     cval = yasm_calc_bc_dist(yasm_section_bcs_first(li->sect),
 			     yasm_section_bcs_last(li->sect));
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Offset of source file in info table */
     yasm_intnum_set_uint(cval, li->fn->info_off);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Number of line number pairs */
     yasm_intnum_set_uint(cval, li->num_linenums);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Number of bytes of line number pairs + 12 (no, I don't know why) */
     yasm_intnum_set_uint(cval, li->num_linenums*8+12);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
     buf += 4;
 
     /* Offset / line number pairs */
@@ -922,14 +908,12 @@ cv8_lineinfo_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 	for (j=0; i<li->num_linenums && j<126; i++, j++) {
 	    /* offset in section */
 	    yasm_intnum_set_uint(cval, ls->pairs[j].offset);
-	    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc,
-				     0);
+	    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
 	    buf += 4;
 
 	    /* line number in file */
 	    yasm_intnum_set_uint(cval, ls->pairs[j].line);
-	    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0, bc,
-				     0);
+	    yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0, bc, 0);
 	    buf += 4;
 	}
     }
@@ -1037,8 +1021,8 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 		  yasm_output_value_func output_value,
 		  yasm_output_reloc_func output_reloc)
 {
+    yasm_object *object = yasm_section_get_object(bc->section);
     cv_sym *cvs = (cv_sym *)bc->contents;
-    yasm_dbgfmt_cv *dbgfmt_cv = cvs->dbgfmt_cv;
     unsigned char *buf = *bufp;
     yasm_intnum *cval;
     const char *ch = cvs->format;
@@ -1047,12 +1031,12 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 
     /* Total length of record (following this field) - 2 bytes */
     cval = yasm_intnum_create_uint(bc->len-2);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 1);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 2, 16, 0, bc, 1);
     buf += 2;
 
     /* Type contained - 2 bytes */
     yasm_intnum_set_uint(cval, cvs->type);
-    yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0, bc, 0);
+    yasm_arch_intnum_tobytes(object->arch, cval, buf, 2, 16, 0, bc, 0);
     buf += 2;
 
     while (*ch) {
@@ -1063,13 +1047,13 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 		break;
 	    case 'h':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
-		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 2, 16, 0,
+		yasm_arch_intnum_tobytes(object->arch, cval, buf, 2, 16, 0,
 					 bc, 0);
 		buf += 2;
 		break;
 	    case 'w':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
-		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0,
+		yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0,
 					 bc, 0);
 		buf += 4;
 		break;
@@ -1080,7 +1064,7 @@ cv_sym_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 		break;
 	    case 'T':
 		yasm_intnum_set_uint(cval, cvs->args[arg++].i);
-		yasm_arch_intnum_tobytes(dbgfmt_cv->arch, cval, buf, 4, 32, 0,
+		yasm_arch_intnum_tobytes(object->arch, cval, buf, 4, 32, 0,
 					 bc, 0);
 		buf += 4;	/* XXX: will be 2 in CV4 */
 		break;
