@@ -1,7 +1,7 @@
 /*
  * DWARF2 debugging format - line information
  *
- *  Copyright (C) 2006  Peter Johnson
+ *  Copyright (C) 2006-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dwarf2-line.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: dwarf2-line.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -111,7 +111,6 @@ typedef struct dwarf2_line_state {
 } dwarf2_line_state;
 
 typedef struct dwarf2_spp {
-    yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2;
     yasm_bytecode *line_start_prevbc;
     yasm_bytecode *line_end_prevbc;
 } dwarf2_spp;
@@ -445,6 +444,8 @@ dwarf2_dbgfmt_gen_line_op(yasm_section *debug_line, dwarf2_line_state *state,
 
 typedef struct dwarf2_line_bc_info {
     yasm_section *debug_line;
+    yasm_object *object;
+    yasm_linemap *linemap;
     yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2;
     dwarf2_line_state *state;
     dwarf2_loc loc;
@@ -475,8 +476,7 @@ dwarf2_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 	}
     }
 
-    yasm_linemap_lookup(dbgfmt_dwarf2->linemap, bc->line, &filename,
-			&info->loc.line);
+    yasm_linemap_lookup(info->linemap, bc->line, &filename, &info->loc.line);
     /* Find file index; just linear search it unless it was the last used */
     if (info->lastfile > 0
 	&& strcmp(filename, dbgfmt_dwarf2->filenames[info->lastfile-1].pathname)
@@ -500,6 +500,8 @@ dwarf2_generate_line_bc(yasm_bytecode *bc, /*@null@*/ void *d)
 
 typedef struct dwarf2_line_info {
     yasm_section *debug_line;	/* section to which line number info goes */
+    yasm_object *object;
+    yasm_linemap *linemap;
     yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2;
     yasm_errwarns *errwarns;
 
@@ -552,6 +554,8 @@ dwarf2_generate_line_section(yasm_section *sect, /*@null@*/ void *d)
 	dwarf2_line_bc_info bcinfo;
 
 	bcinfo.debug_line = info->debug_line;
+	bcinfo.object = info->object;
+	bcinfo.linemap = info->linemap;
 	bcinfo.dbgfmt_dwarf2 = dbgfmt_dwarf2;
 	bcinfo.state = &state;
 	bcinfo.lastfile = 0;
@@ -618,11 +622,12 @@ dwarf2_generate_filename(const char *filename, void *d)
 }
 
 yasm_section *
-yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
+yasm_dwarf2__generate_line(yasm_object *object, yasm_linemap *linemap,
 			   yasm_errwarns *errwarns, int asm_source,
 			   /*@out@*/ yasm_section **main_code,
 			   /*@out@*/ size_t *num_line_sections)
 {
+    yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2 = (yasm_dbgfmt_dwarf2 *)object->dbgfmt;
     dwarf2_line_info info;
     int new;
     size_t i;
@@ -632,17 +637,18 @@ yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 
     if (asm_source) {
 	/* Generate dirs and filenames based on linemap */
-	yasm_linemap_traverse_filenames(dbgfmt_dwarf2->linemap, dbgfmt_dwarf2,
+	yasm_linemap_traverse_filenames(linemap, dbgfmt_dwarf2,
 					dwarf2_generate_filename);
     }
 
     info.num_sections = 0;
     info.last_code = NULL;
     info.asm_source = asm_source;
+    info.object = object;
+    info.linemap = linemap;
     info.dbgfmt_dwarf2 = dbgfmt_dwarf2;
-    info.debug_line = yasm_object_get_general(dbgfmt_dwarf2->object,
-					      ".debug_line", 0, 1, 0, 0, &new,
-					      0);
+    info.debug_line = yasm_object_get_general(object, ".debug_line", 0, 1, 0,
+					      0, &new, 0);
     last = yasm_section_bcs_last(info.debug_line);
 
     /* header */
@@ -650,7 +656,6 @@ yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
 
     /* statement program prologue */
     spp = yasm_xmalloc(sizeof(dwarf2_spp));
-    spp->dbgfmt_dwarf2 = dbgfmt_dwarf2;
     sppbc = yasm_bc_create_common(&dwarf2_spp_bc_callback, spp, 0);
     sppbc->len = dbgfmt_dwarf2->sizeof_offset + 5 +
 	NELEMS(line_opcode_num_operands);
@@ -676,7 +681,7 @@ yasm_dwarf2__generate_line(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
     yasm_dwarf2__append_bc(info.debug_line, sppbc);
 
     /* statement program */
-    yasm_object_sections_traverse(dbgfmt_dwarf2->object, (void *)&info,
+    yasm_object_sections_traverse(object, (void *)&info,
 				  dwarf2_generate_line_section);
 
     /* mark end of line information */
@@ -716,8 +721,8 @@ dwarf2_spp_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
 		      yasm_output_value_func output_value,
 		      yasm_output_reloc_func output_reloc)
 {
-    dwarf2_spp *spp = (dwarf2_spp *)bc->contents;
-    yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2 = spp->dbgfmt_dwarf2;
+    yasm_object *object = yasm_section_get_object(bc->section);
+    yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2 = (yasm_dbgfmt_dwarf2 *)object->dbgfmt;
     unsigned char *buf = *bufp;
     yasm_intnum *cval;
     size_t i, len;
@@ -725,7 +730,7 @@ dwarf2_spp_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     /* Prologue length (following this field) */
     cval = yasm_intnum_create_uint(bc->len - (unsigned long)(buf-*bufp) -
 				   dbgfmt_dwarf2->sizeof_offset);
-    yasm_arch_intnum_tobytes(dbgfmt_dwarf2->arch, cval, buf,
+    yasm_arch_intnum_tobytes(object->arch, cval, buf,
 			     dbgfmt_dwarf2->sizeof_offset,
 			     dbgfmt_dwarf2->sizeof_offset*8, 0, bc, 0);
     buf += dbgfmt_dwarf2->sizeof_offset;
@@ -821,186 +826,226 @@ dwarf2_line_op_bc_tobytes(yasm_bytecode *bc, unsigned char **bufp, void *d,
     return 0;
 }
 
-int
-yasm_dwarf2__line_directive(yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2,
-			    const char *name, yasm_section *sect,
-			    yasm_valparamhead *valparams, unsigned long line)
+void
+yasm_dwarf2__dir_loc(yasm_object *object, yasm_valparamhead *valparams,
+		     yasm_valparamhead *objext_valparams, unsigned long line)
 {
     yasm_valparam *vp;
-    if (yasm__strcasecmp(name, "loc") == 0) {
-	/*@dependent@*/ /*@null@*/ const yasm_intnum *intn;
-	dwarf2_section_data *dsd;
-	dwarf2_loc *loc = yasm_xmalloc(sizeof(dwarf2_loc));
+    int in_is_stmt = 0, in_isa = 0;
 
-	/* File number (required) */
-	if (!valparams || !(vp = yasm_vps_first(valparams)) || !vp->param) {
-	    yasm_error_set(YASM_ERROR_SYNTAX, N_("file number required"));
-	    yasm_xfree(loc);
-	    return 0;
-	}
-	intn = yasm_expr_get_intnum(&vp->param, 0);
-	if (!intn) {
-	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			   N_("file number is not a constant"));
-	    yasm_xfree(loc);
-	    return 0;
-	}
-	if (yasm_intnum_sign(intn) != 1) {
-	    yasm_error_set(YASM_ERROR_VALUE,
-			   N_("file number less than one"));
-	    yasm_xfree(loc);
-	    return 0;
-	}
-	loc->file = yasm_intnum_get_uint(intn);
+    /*@dependent@*/ /*@null@*/ const yasm_intnum *intn;
+    dwarf2_section_data *dsd;
+    dwarf2_loc *loc = yasm_xmalloc(sizeof(dwarf2_loc));
 
-	/* Line number (required) */
-	vp = yasm_vps_next(vp);
-	if (!vp || !vp->param) {
-	    yasm_error_set(YASM_ERROR_SYNTAX, N_("line number required"));
-	    yasm_xfree(loc);
-	    return 0;
-	}
-	intn = yasm_expr_get_intnum(&vp->param, 0);
-	if (!intn) {
-	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			   N_("file number is not a constant"));
-	    yasm_xfree(loc);
-	    return 0;
-	}
-	loc->line = yasm_intnum_get_uint(intn);
-
-	/* Generate new section data if it doesn't already exist */
-	dsd = yasm_section_get_data(sect, &yasm_dwarf2__section_data_cb);
-	if (!dsd) {
-	    dsd = yasm_xmalloc(sizeof(dwarf2_section_data));
-	    STAILQ_INIT(&dsd->locs);
-	    yasm_section_add_data(sect, &yasm_dwarf2__section_data_cb, dsd);
-	}
-
-	/* Defaults for optional settings */
-	loc->column = 0;
-	loc->isa_change = 0;
-	loc->isa = 0;
-	loc->is_stmt = IS_STMT_NOCHANGE;
-	loc->basic_block = 0;
-	loc->prologue_end = 0;
-	loc->epilogue_begin = 0;
-
-	/* Optional column number */
-	vp = yasm_vps_next(vp);
-	if (vp && vp->param) {
-	    intn = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!intn) {
-		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			       N_("column number is not a constant"));
-		yasm_xfree(loc);
-		return 0;
-	    }
-	    loc->column = yasm_intnum_get_uint(intn);
-	    vp = yasm_vps_next(vp);
-	}
-
-	/* Other options */
-	while (vp && vp->val) {
-	    if (yasm__strcasecmp(vp->val, "basic_block") == 0)
-		loc->basic_block = 1;
-	    else if (yasm__strcasecmp(vp->val, "prologue_end") == 0)
-		loc->prologue_end = 1;
-	    else if (yasm__strcasecmp(vp->val, "epilogue_begin") == 0)
-		loc->epilogue_begin = 1;
-	    else if (yasm__strcasecmp(vp->val, "is_stmt") == 0) {
-		if (!vp->param) {
-		    yasm_error_set(YASM_ERROR_SYNTAX,
-				   N_("is_stmt requires value"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-		intn = yasm_expr_get_intnum(&vp->param, 0);
-		if (!intn) {
-		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-				   N_("is_stmt value is not a constant"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-		if (yasm_intnum_is_zero(intn))
-		    loc->is_stmt = IS_STMT_SET;
-		else if (yasm_intnum_is_pos1(intn))
-		    loc->is_stmt = IS_STMT_CLEAR;
-		else {
-		    yasm_error_set(YASM_ERROR_VALUE,
-				   N_("is_stmt value not 0 or 1"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-	    } else if (yasm__strcasecmp(vp->val, "isa") == 0) {
-		if (!vp->param) {
-		    yasm_error_set(YASM_ERROR_SYNTAX, N_("isa requires value"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-		intn = yasm_expr_get_intnum(&vp->param, 0);
-		if (!intn) {
-		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-				   N_("isa value is not a constant"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-		if (yasm_intnum_sign(intn) < 0) {
-		    yasm_error_set(YASM_ERROR_VALUE,
-				   N_("isa value less than zero"));
-		    yasm_xfree(loc);
-		    return 0;
-		}
-		loc->isa_change = 1;
-		loc->isa = yasm_intnum_get_uint(intn);
-	    } else
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("unrecognized loc option `%s'"), vp->val);
-	}
-
-	/* Append new location */
-	loc->vline = line;
-	loc->bc = NULL;
-	loc->sym = NULL;
-	STAILQ_INSERT_TAIL(&dsd->locs, loc, link);
-
-	return 0;
-    } else if (yasm__strcasecmp(name, "file") == 0) {
-	/*@dependent@*/ /*@null@*/ const yasm_intnum *file_intn;
-	unsigned long filenum;
-
-	if (!valparams) {
-	    yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
-			   "FILE");
-	    return 0;
-	}
-
-	vp = yasm_vps_first(valparams);
-	if (vp->val) {
-	    /* Just a bare filename */
-	    yasm_object_set_source_fn(dbgfmt_dwarf2->object, vp->val);
-	    return 0;
-	}
-
-	/* Otherwise.. first vp is the file number */
-	file_intn = yasm_expr_get_intnum(&vp->param, 0);
-	if (!file_intn) {
-	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			   N_("file number is not a constant"));
-	    return 0;
-	}
-	filenum = yasm_intnum_get_uint(file_intn);
-
-	vp = yasm_vps_next(vp);
-	if (!vp || !vp->val) {
-	    yasm_error_set(YASM_ERROR_SYNTAX,
-			   N_("file number given but no filename"));
-	    return 0;
-	}
-
-	dwarf2_dbgfmt_add_file(dbgfmt_dwarf2, filenum, vp->val);
-	return 0;
+    /* File number (required) */
+    if (!valparams || !(vp = yasm_vps_first(valparams)) ||
+        vp->val || vp->type != YASM_PARAM_EXPR) {
+	yasm_error_set(YASM_ERROR_SYNTAX, N_("file number required"));
+	yasm_xfree(loc);
+	return;
     }
-    return 1;
+    intn = yasm_expr_get_intnum(&vp->param.e, 0);
+    if (!intn) {
+	yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+		       N_("file number is not a constant"));
+	yasm_xfree(loc);
+	return;
+    }
+    if (yasm_intnum_sign(intn) != 1) {
+	yasm_error_set(YASM_ERROR_VALUE, N_("file number less than one"));
+	yasm_xfree(loc);
+	return;
+    }
+    loc->file = yasm_intnum_get_uint(intn);
+
+    /* Line number (required) */
+    vp = yasm_vps_next(vp);
+    if (!vp || vp->val || vp->type != YASM_PARAM_EXPR) {
+	yasm_error_set(YASM_ERROR_SYNTAX, N_("line number required"));
+	yasm_xfree(loc);
+	return;
+    }
+    intn = yasm_expr_get_intnum(&vp->param.e, 0);
+    if (!intn) {
+	yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+                       N_("line number is not a constant"));
+	yasm_xfree(loc);
+	return;
+    }
+    loc->line = yasm_intnum_get_uint(intn);
+
+    /* Generate new section data if it doesn't already exist */
+    if (!object->cur_section) {
+        yasm_error_set(YASM_ERROR_SYNTAX,
+                       N_("[%s] can only be used inside of a section"), "loc");
+        yasm_xfree(loc);
+        return;
+    }
+    dsd = yasm_section_get_data(object->cur_section,
+				&yasm_dwarf2__section_data_cb);
+    if (!dsd) {
+	dsd = yasm_xmalloc(sizeof(dwarf2_section_data));
+	STAILQ_INIT(&dsd->locs);
+	yasm_section_add_data(object->cur_section,
+			      &yasm_dwarf2__section_data_cb, dsd);
+    }
+
+    /* Defaults for optional settings */
+    loc->column = 0;
+    loc->isa_change = 0;
+    loc->isa = 0;
+    loc->is_stmt = IS_STMT_NOCHANGE;
+    loc->basic_block = 0;
+    loc->prologue_end = 0;
+    loc->epilogue_begin = 0;
+
+    /* Optional column number */
+    vp = yasm_vps_next(vp);
+    if (vp && !vp->val && vp->type == YASM_PARAM_EXPR) {
+        intn = yasm_expr_get_intnum(&vp->param.e, 0);
+	if (!intn) {
+	    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			   N_("column number is not a constant"));
+	    yasm_xfree(loc);
+	    return;
+	}
+	loc->column = yasm_intnum_get_uint(intn);
+	vp = yasm_vps_next(vp);
+    }
+
+    /* Other options; note for GAS compatibility we need to support both:
+     * is_stmt=1 (NASM) and
+     * is_stmt 1 (GAS)
+     */
+    while (vp) {
+        /*@null@*/ /*@dependent@*/ const char *s;
+        /*@null@*/ /*@only@*/ yasm_expr *e;
+
+restart:
+        if (in_is_stmt) {
+            in_is_stmt = 0;
+            if (!(e = yasm_vp_expr(vp, object->symtab, line)) ||
+                !(intn = yasm_expr_get_intnum(&e, 0))) {
+		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			       N_("is_stmt value is not a constant"));
+		yasm_xfree(loc);
+                if (e)
+                    yasm_expr_destroy(e);
+		return;
+	    }
+	    if (yasm_intnum_is_zero(intn))
+		loc->is_stmt = IS_STMT_SET;
+	    else if (yasm_intnum_is_pos1(intn))
+		loc->is_stmt = IS_STMT_CLEAR;
+	    else {
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("is_stmt value not 0 or 1"));
+		yasm_xfree(loc);
+                yasm_expr_destroy(e);
+		return;
+	    }
+            yasm_expr_destroy(e);
+        } else if (in_isa) {
+            in_isa = 0;
+            if (!(e = yasm_vp_expr(vp, object->symtab, line)) ||
+                !(intn = yasm_expr_get_intnum(&e, 0))) {
+		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+			       N_("isa value is not a constant"));
+		yasm_xfree(loc);
+                if (e)
+                    yasm_expr_destroy(e);
+		return;
+	    }
+	    if (yasm_intnum_sign(intn) < 0) {
+		yasm_error_set(YASM_ERROR_VALUE,
+			       N_("isa value less than zero"));
+		yasm_xfree(loc);
+                yasm_expr_destroy(e);
+		return;
+	    }
+	    loc->isa_change = 1;
+	    loc->isa = yasm_intnum_get_uint(intn);
+            yasm_expr_destroy(e);
+        } else if (!vp->val && (s = yasm_vp_id(vp))) {
+            if (yasm__strcasecmp(s, "is_stmt") == 0)
+                in_is_stmt = 1;
+            else if (yasm__strcasecmp(s, "isa") == 0)
+                in_isa = 1;
+            else if (yasm__strcasecmp(s, "basic_block") == 0)
+                loc->basic_block = 1;
+            else if (yasm__strcasecmp(s, "prologue_end") == 0)
+                loc->prologue_end = 1;
+            else if (yasm__strcasecmp(s, "epilogue_begin") == 0)
+                loc->epilogue_begin = 1;
+            else
+                yasm_warn_set(YASM_WARN_GENERAL,
+                              N_("unrecognized loc option `%s'"), s);
+        } else if (!vp->val) {
+            yasm_warn_set(YASM_WARN_GENERAL,
+                          N_("unrecognized numeric qualifier"));
+        } else if (yasm__strcasecmp(vp->val, "is_stmt") == 0) {
+            in_is_stmt = 1;
+            goto restart; /* don't go to the next valparam */
+        } else if (yasm__strcasecmp(vp->val, "isa") == 0) {
+            in_isa = 1;
+            goto restart; /* don't go to the next valparam */
+	} else
+	    yasm_warn_set(YASM_WARN_GENERAL,
+			  N_("unrecognized loc option `%s'"), vp->val);
+        vp = yasm_vps_next(vp);
+    }
+
+    if (in_is_stmt || in_isa) {
+        yasm_error_set(YASM_ERROR_SYNTAX, N_("%s requires value"),
+                       in_is_stmt ? "is_stmt" : "isa");
+        yasm_xfree(loc);
+        return;
+    }
+
+    /* Append new location */
+    loc->vline = line;
+    loc->bc = NULL;
+    loc->sym = NULL;
+    STAILQ_INSERT_TAIL(&dsd->locs, loc, link);
 }
 
+void
+yasm_dwarf2__dir_file(yasm_object *object, yasm_valparamhead *valparams,
+		      yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_dbgfmt_dwarf2 *dbgfmt_dwarf2 = (yasm_dbgfmt_dwarf2 *)object->dbgfmt;
+    yasm_valparam *vp;
+    /*@dependent@*/ /*@null@*/ const yasm_intnum *file_intn;
+    unsigned long filenum;
+
+    if (!valparams) {
+	yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
+		       "FILE");
+	return;
+    }
+
+    vp = yasm_vps_first(valparams);
+    if (yasm_vp_string(vp)) {
+	/* Just a bare filename */
+        yasm_object_set_source_fn(object, yasm_vp_string(vp));
+	return;
+    }
+
+    /* Otherwise.. first vp is the file number */
+    if (vp->type != YASM_PARAM_EXPR ||
+        !(file_intn = yasm_expr_get_intnum(&vp->param.e, 0))) {
+	yasm_error_set(YASM_ERROR_NOT_CONSTANT,
+		       N_("file number is not a constant"));
+	return;
+    }
+    filenum = yasm_intnum_get_uint(file_intn);
+
+    vp = yasm_vps_next(vp);
+    if (!yasm_vp_string(vp)) {
+	yasm_error_set(YASM_ERROR_SYNTAX,
+		       N_("file number given but no filename"));
+	return;
+    }
+
+    dwarf2_dbgfmt_add_file(dbgfmt_dwarf2, filenum, yasm_vp_string(vp));
+}
