@@ -1,7 +1,7 @@
 /*
  * Flat-format binary object format
  *
- *  Copyright (C) 2002  Peter Johnson
+ *  Copyright (C) 2002-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: bin-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: bin-objfmt.c 1879 2007-06-05 04:20:37Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -37,23 +37,16 @@
 
 typedef struct yasm_objfmt_bin {
     yasm_objfmt_base objfmt;		/* base structure */
-
-    yasm_object *object;
-    yasm_symtab *symtab;
-    /*@dependent@*/ yasm_arch *arch;
 } yasm_objfmt_bin;
 
 yasm_objfmt_module yasm_bin_LTX_objfmt;
 
 
 static yasm_objfmt *
-bin_objfmt_create(yasm_object *object, yasm_arch *a)
+bin_objfmt_create(yasm_object *object)
 {
     yasm_objfmt_bin *objfmt_bin = yasm_xmalloc(sizeof(yasm_objfmt_bin));
     objfmt_bin->objfmt.module = &yasm_bin_LTX_objfmt;
-    objfmt_bin->object = object;
-    objfmt_bin->symtab = yasm_object_get_symtab(object);
-    objfmt_bin->arch = a;
     return (yasm_objfmt *)objfmt_bin;
 }
 
@@ -91,7 +84,8 @@ bin_objfmt_align_section(yasm_section *sect, yasm_section *prevsect,
 }
 
 typedef struct bin_objfmt_output_info {
-    yasm_objfmt_bin *objfmt_bin;
+    yasm_object *object;
+    yasm_errwarns *errwarns;
     /*@dependent@*/ FILE *f;
     /*@only@*/ unsigned char *buf;
     /*@observer@*/ const yasm_section *sect;
@@ -109,11 +103,13 @@ bin_objfmt_expr_xform(/*@returned@*/ /*@only@*/ yasm_expr *e,
     /*@null@*/ yasm_intnum *dist;
 
     for (i=0; i<e->numterms; i++) {
-	/* Transform symrecs that reference sections into
+        /* Transform symrecs or precbcs that reference sections into
 	 * start expr + intnum(dist).
 	 */
-	if (e->terms[i].type == YASM_EXPR_SYM &&
-	    yasm_symrec_get_label(e->terms[i].data.sym, &precbc) &&
+        if (((e->terms[i].type == YASM_EXPR_SYM &&
+             yasm_symrec_get_label(e->terms[i].data.sym, &precbc)) ||
+            (e->terms[i].type == YASM_EXPR_PRECBC &&
+             (precbc = e->terms[i].data.precbc))) &&
 	    (sect = yasm_bc_get_section(precbc)) &&
 	    (dist = yasm_calc_bc_dist(yasm_section_bcs_first(sect), precbc))) {
 	    const yasm_expr *start = yasm_section_get_start(sect);
@@ -141,17 +137,36 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf,
     assert(info != NULL);
 
     /* Binary objects we need to resolve against object, not against section. */
-    if (value->rel && !value->curpos_rel
-	&& yasm_symrec_get_label(value->rel, &precbc)
-	&& (sect = yasm_bc_get_section(precbc))) {
+    if (value->rel) {
 	unsigned int rshift = (unsigned int)value->rshift;
 	yasm_expr *syme;
-	if (value->rshift > 0)
-	    syme = yasm_expr_create(YASM_EXPR_SHR, yasm_expr_sym(value->rel),
-		yasm_expr_int(yasm_intnum_create_uint(rshift)), bc->line);
-	else
-	    syme = yasm_expr_create_ident(yasm_expr_sym(value->rel), bc->line);
 
+        if (yasm_symrec_is_abs(value->rel)) {
+            syme = yasm_expr_create_ident(yasm_expr_int(
+                yasm_intnum_create_uint(0)), bc->line);
+        } else if (yasm_symrec_get_label(value->rel, &precbc)
+                   && (sect = yasm_bc_get_section(precbc))) {
+            syme = yasm_expr_create_ident(yasm_expr_sym(value->rel), bc->line);
+        } else
+            goto done;
+
+        /* Handle PC-relative */
+        if (value->curpos_rel) {
+            yasm_expr *sube;
+            sube = yasm_expr_create(YASM_EXPR_SUB, yasm_expr_precbc(bc),
+                yasm_expr_int(yasm_intnum_create_uint(bc->len*bc->mult_int)),
+                bc->line);
+            syme = yasm_expr_create(YASM_EXPR_SUB, yasm_expr_expr(syme),
+                                    yasm_expr_expr(sube), bc->line);
+            value->curpos_rel = 0;
+            value->ip_rel = 0;
+        } else
+
+	if (value->rshift > 0)
+            syme = yasm_expr_create(YASM_EXPR_SHR, yasm_expr_expr(syme),
+		yasm_expr_int(yasm_intnum_create_uint(rshift)), bc->line);
+
+        /* Add into absolute portion */
 	if (!value->abs)
 	    value->abs = syme;
 	else
@@ -161,7 +176,7 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf,
 	value->rel = NULL;
 	value->rshift = 0;
     }
-
+done:
     /* Simplify absolute portion of value, transforming symrecs */
     if (value->abs)
 	value->abs = yasm_expr__level_tree
@@ -169,7 +184,7 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf,
 
     /* Output */
     switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
-				    info->objfmt_bin->arch)) {
+				    info->object->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -178,36 +193,6 @@ bin_objfmt_output_value(yasm_value *value, unsigned char *buf,
 	    return 0;
     }
 
-    /* Absolute value; handle it here as output_basic won't understand it */
-    if (value->rel && yasm_symrec_is_abs(value->rel)) {
-	if (value->curpos_rel) {
-	    /* Calculate value relative to current assembly position */
-	    /*@only@*/ yasm_intnum *outval;
-	    unsigned int valsize = value->size;
-	    int retval = 0;
-
-	    outval = yasm_intnum_create_uint(bc->offset + info->abs_start);
-	    yasm_intnum_calc(outval, YASM_EXPR_NEG, NULL);
-
-	    if (value->rshift > 0) {
-		/*@only@*/ yasm_intnum *shamt =
-		    yasm_intnum_create_uint((unsigned long)value->rshift);
-		yasm_intnum_calc(outval, YASM_EXPR_SHR, shamt);
-		yasm_intnum_destroy(shamt);
-	    }
-	    /* Add in absolute portion */
-	    if (value->abs)
-		yasm_intnum_calc(outval, YASM_EXPR_ADD,
-				 yasm_expr_get_intnum(&value->abs, 1));
-	    /* Output! */
-	    if (yasm_arch_intnum_tobytes(info->objfmt_bin->arch, outval, buf,
-					 destsize, valsize, 0, bc, warn))
-		retval = 1;
-	    yasm_intnum_destroy(outval);
-	    return retval;
-	}
-    }
-
     /* Couldn't output, assume it contains an external reference. */
     yasm_error_set(YASM_ERROR_GENERAL,
 	N_("binary object format does not support external references"));
@@ -259,11 +244,33 @@ bin_objfmt_output_bytecode(yasm_bytecode *bc, /*@null@*/ void *d)
     return 0;
 }
 
+static int
+bin_objfmt_check_sym(yasm_symrec *sym, /*@null@*/ void *d)
+{
+    /*@null@*/ bin_objfmt_output_info *info = (bin_objfmt_output_info *)d;
+    yasm_sym_vis vis = yasm_symrec_get_visibility(sym);
+    assert(info != NULL);
+
+    if (vis & YASM_SYM_EXTERN) {
+	yasm_warn_set(YASM_WARN_GENERAL,
+	    N_("binary object format does not support extern variables"));
+	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
+    } else if (vis & YASM_SYM_GLOBAL) {
+	yasm_warn_set(YASM_WARN_GENERAL,
+	    N_("binary object format does not support global variables"));
+	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
+    } else if (vis & YASM_SYM_COMMON) {
+	yasm_error_set(YASM_ERROR_TYPE,
+	    N_("binary object format does not support common variables"));
+	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
+    }
+    return 0;
+}
+
 static void
-bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
-		  /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
+bin_objfmt_output(yasm_object *object, FILE *f, /*@unused@*/ int all_syms,
+		  yasm_errwarns *errwarns)
 {
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     /*@observer@*/ /*@null@*/ yasm_section *text, *data, *bss, *prevsect;
     /*@null@*/ yasm_expr *startexpr;
     /*@dependent@*/ /*@null@*/ const yasm_intnum *startnum;
@@ -273,13 +280,17 @@ bin_objfmt_output(yasm_objfmt *objfmt, FILE *f, /*@unused@*/ int all_syms,
     unsigned long i;
     bin_objfmt_output_info info;
 
-    info.objfmt_bin = objfmt_bin;
+    info.object = object;
+    info.errwarns = errwarns;
     info.f = f;
     info.buf = yasm_xmalloc(REGULAR_OUTBUF_SIZE);
 
-    text = yasm_object_find_general(objfmt_bin->object, ".text");
-    data = yasm_object_find_general(objfmt_bin->object, ".data");
-    bss = yasm_object_find_general(objfmt_bin->object, ".bss");
+    /* Check symbol table */
+    yasm_symtab_traverse(object->symtab, &info, bin_objfmt_check_sym);
+
+    text = yasm_object_find_general(object, ".text");
+    data = yasm_object_find_general(object, ".data");
+    bss = yasm_object_find_general(object, ".bss");
 
     if (!text)
 	yasm_internal_error(N_("No `.text' section in bin objfmt output"));
@@ -361,51 +372,56 @@ bin_objfmt_destroy(yasm_objfmt *objfmt)
 }
 
 static void
-bin_objfmt_init_new_section(yasm_objfmt_bin *objfmt_bin, yasm_section *sect,
+bin_objfmt_init_new_section(yasm_object *object, yasm_section *sect,
 			    const char *sectname, unsigned long line)
 {
-    yasm_symtab_define_label(
-	yasm_object_get_symtab(objfmt_bin->object), sectname,
-	yasm_section_bcs_first(sect), 1, line);
+    yasm_symtab_define_label(object->symtab, sectname,
+			     yasm_section_bcs_first(sect), 1, line);
 }
 
 static yasm_section *
-bin_objfmt_add_default_section(yasm_objfmt *objfmt)
+bin_objfmt_add_default_section(yasm_object *object)
 {
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     yasm_section *retval;
     int isnew;
 
-    retval = yasm_object_get_general(objfmt_bin->object, ".text", 0, 16, 1, 0,
-				     &isnew, 0);
+    retval = yasm_object_get_general(object, ".text", 0, 16, 1, 0, &isnew, 0);
     if (isnew) {
-	bin_objfmt_init_new_section(objfmt_bin, retval, ".text", 0);
+	bin_objfmt_init_new_section(object, retval, ".text", 0);
 	yasm_section_set_default(retval, 1);
     }
     return retval;
 }
 
 static /*@observer@*/ /*@null@*/ yasm_section *
-bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
+bin_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
 			  /*@unused@*/ /*@null@*/
 			  yasm_valparamhead *objext_valparams,
 			  unsigned long line)
 {
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     yasm_valparam *vp;
     yasm_section *retval;
     int isnew;
     unsigned long start;
-    char *sectname;
+    const char *sectname;
     int resonly = 0;
+    /*@only@*/ /*@null@*/ yasm_intnum *align_intn = NULL;
     unsigned long align = 4;
     int have_align = 0;
 
-    if ((vp = yasm_vps_first(valparams)) && !vp->param && vp->val != NULL) {
+    static const yasm_dir_help help[] = {
+        { "align", 1, yasm_dir_helper_intn, 0, 0 }
+    };
+
+    vp = yasm_vps_first(valparams);
+    sectname = yasm_vp_string(vp);
+    if (!sectname)
+        return NULL;
+    vp = yasm_vps_next(vp);
+
 	/* If it's the first section output (.text) start at 0, otherwise
 	 * make sure the start is > 128.
 	 */
-	sectname = vp->val;
 	if (strcmp(sectname, ".text") == 0)
 	    start = 0;
 	else if (strcmp(sectname, ".data") == 0)
@@ -420,52 +436,31 @@ bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    return NULL;
 	}
 
-	/* Check for ALIGN qualifier */
-	while ((vp = yasm_vps_next(vp))) {
-	    if (!vp->val) {
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized numeric qualifier"));
-		continue;
-	    }
+    have_align = yasm_dir_helper(object, vp, line, help, NELEMS(help),
+                                 &align_intn, yasm_dir_helper_valparam_warn);
+    if (have_align < 0)
+        return NULL;    /* error occurred */
 
-	    if (yasm__strcasecmp(vp->val, "align") == 0 && vp->param) {
-		/*@dependent@*/ /*@null@*/ const yasm_intnum *align_expr;
-
-		if (strcmp(sectname, ".text") == 0) {
-		    yasm_error_set(YASM_ERROR_GENERAL,
-			N_("cannot specify an alignment to the `%s' section"),
-			sectname);
-		    return NULL;
-		}
-		
-		align_expr = yasm_expr_get_intnum(&vp->param, 0);
-		if (!align_expr) {
-		    yasm_error_set(YASM_ERROR_VALUE,
-				N_("argument to `%s' is not an integer"),
-				vp->val);
-		    return NULL;
-		}
-		align = yasm_intnum_get_uint(align_expr);
+    if (align_intn) {
+        align = yasm_intnum_get_uint(align_intn);
+        yasm_intnum_destroy(align_intn);
 
 		/* Alignments must be a power of two. */
 		if (!is_exp2(align)) {
 		    yasm_error_set(YASM_ERROR_VALUE,
 				N_("argument to `%s' is not a power of two"),
-				vp->val);
+                           "align");
 		    return NULL;
 		}
-
-		have_align = 1;
 	    }
-	}
 
-	retval = yasm_object_get_general(objfmt_bin->object, sectname,
+	retval = yasm_object_get_general(object, sectname,
 	    yasm_expr_create_ident(
 		yasm_expr_int(yasm_intnum_create_uint(start)), line), align,
 	    strcmp(sectname, ".text") == 0, resonly, &isnew, line);
 
 	if (isnew)
-	    bin_objfmt_init_new_section(objfmt_bin, retval, sectname, line);
+	    bin_objfmt_init_new_section(object, retval, sectname, line);
 
 	if (isnew || yasm_section_is_default(retval)) {
 	    yasm_section_set_default(retval, 0);
@@ -475,105 +470,33 @@ bin_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 		N_("alignment value ignored on section redeclaration"));
 
 	return retval;
-    } else
-	return NULL;
 }
 
-static yasm_symrec *
-bin_objfmt_extern_declare(yasm_objfmt *objfmt, const char *name,
-			  /*@unused@*/ /*@null@*/
-			  yasm_valparamhead *objext_valparams,
-			  unsigned long line)
+static void
+bin_objfmt_dir_org(yasm_object *object,
+		   /*@null@*/ yasm_valparamhead *valparams,
+		   /*@unused@*/ /*@null@*/
+		   yasm_valparamhead *objext_valparams, unsigned long line)
 {
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
-    yasm_symrec *sym;
-
-    yasm_warn_set(YASM_WARN_GENERAL,
-		  N_("binary object format does not support extern variables"));
-
-    sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_EXTERN, line);
-    return sym;
-}
-
-static yasm_symrec *
-bin_objfmt_global_declare(yasm_objfmt *objfmt, const char *name,
-			  /*@unused@*/ /*@null@*/
-			  yasm_valparamhead *objext_valparams,
-			  unsigned long line)
-{
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
-    yasm_symrec *sym;
-
-    yasm_warn_set(YASM_WARN_GENERAL,
-		  N_("binary object format does not support global variables"));
-
-    sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_GLOBAL, line);
-    return sym;
-}
-
-static yasm_symrec *
-bin_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
-			  /*@only@*/ yasm_expr *size, /*@unused@*/ /*@null@*/
-			  yasm_valparamhead *objext_valparams,
-			  unsigned long line)
-{
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
-    yasm_symrec *sym;
-
-    yasm_expr_destroy(size);
-    yasm_error_set(YASM_ERROR_TYPE,
-	N_("binary object format does not support common variables"));
-
-    sym = yasm_symtab_declare(objfmt_bin->symtab, name, YASM_SYM_COMMON, line);
-    return sym;
-}
-
-static int
-bin_objfmt_directive(yasm_objfmt *objfmt, const char *name,
-		     /*@null@*/ yasm_valparamhead *valparams,
-		     /*@unused@*/ /*@null@*/
-		     yasm_valparamhead *objext_valparams, unsigned long line)
-{
-    yasm_objfmt_bin *objfmt_bin = (yasm_objfmt_bin *)objfmt;
     yasm_section *sect;
     yasm_valparam *vp;
+    /*@only@*/ /*@null@*/ yasm_expr *start;
 
-    if (yasm__strcasecmp(name, "org") == 0) {
-	/*@null@*/ yasm_expr *start = NULL;
+    /* ORG takes just a simple integer as param */
+    vp = yasm_vps_first(valparams);
+    start = yasm_vp_expr(vp, object->symtab, line);
+    if (!start) {
+	yasm_error_set(YASM_ERROR_SYNTAX,
+		       N_("argument to ORG must be expression"));
+	return;
+    }
 
-	if (!valparams) {
-	    yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
-			   "ORG");
-	    return 0;
-	}
-
-	/* ORG takes just a simple integer as param */
-	vp = yasm_vps_first(valparams);
-	if (vp->val)
-	    start = yasm_expr_create_ident(yasm_expr_sym(yasm_symtab_use(
-		yasm_object_get_symtab(objfmt_bin->object), vp->val, line)),
-					   line);
-	else if (vp->param) {
-	    start = vp->param;
-	    vp->param = NULL;	/* Don't let valparams delete it */
-	}
-
-	if (!start) {
-	    yasm_error_set(YASM_ERROR_SYNTAX,
-			   N_("argument to ORG must be expression"));
-	    return 0;
-	}
-
-	/* ORG changes the start of the .text section */
-	sect = yasm_object_find_general(objfmt_bin->object, ".text");
-	if (!sect)
-	    yasm_internal_error(
-		N_("bin objfmt: .text section does not exist before ORG is called?"));
-	yasm_section_set_start(sect, start, line);
-
-	return 0;	    /* directive recognized */
-    } else
-	return 1;	    /* directive unrecognized */
+    /* ORG changes the start of the .text section */
+    sect = yasm_object_find_general(object, ".text");
+    if (!sect)
+	yasm_internal_error(
+	    N_("bin objfmt: .text section does not exist before ORG is called?"));
+    yasm_section_set_start(sect, start, line);
 }
 
 
@@ -583,6 +506,11 @@ static const char *bin_objfmt_dbgfmt_keywords[] = {
     NULL
 };
 
+static const yasm_directive bin_objfmt_directives[] = {
+    { "org",	"nasm",	bin_objfmt_dir_org,	YASM_DIR_ARG_REQUIRED },
+    { NULL, NULL, NULL, 0 }
+};
+
 /* Define objfmt structure -- see objfmt.h for details */
 yasm_objfmt_module yasm_bin_LTX_objfmt = {
     "Flat format binary",
@@ -591,13 +519,10 @@ yasm_objfmt_module yasm_bin_LTX_objfmt = {
     16,
     bin_objfmt_dbgfmt_keywords,
     "null",
+    bin_objfmt_directives,
     bin_objfmt_create,
     bin_objfmt_output,
     bin_objfmt_destroy,
     bin_objfmt_add_default_section,
-    bin_objfmt_section_switch,
-    bin_objfmt_extern_declare,
-    bin_objfmt_global_declare,
-    bin_objfmt_common_declare,
-    bin_objfmt_directive
+    bin_objfmt_section_switch
 };
