@@ -1,7 +1,7 @@
-/* $Id: elf.h 1521 2006-05-04 07:55:32Z peter $
+/* $Id: elf.h 1876 2007-06-02 21:50:51Z peter $
  * ELF object format helpers
  *
- *  Copyright (C) 2003  Michael Urman
+ *  Copyright (C) 2003-2007  Michael Urman
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -391,7 +391,7 @@ struct elf_symtab_entry {
     yasm_section	*sect;
     elf_strtab_entry	*name;
     elf_address		 value;
-    yasm_expr		*xsize;
+    /*@dependent@*/ yasm_expr *xsize;
     elf_size		 size;
     elf_section_index	 index;
     elf_symbol_binding	 bind;
