@@ -103,7 +103,7 @@
 */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: macho-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: macho-objfmt.c 1875 2007-06-02 21:48:49Z peter $");
 
 /* optional: automatically prefix underscores to global exported symbols */
 /*#define AUTO_UNDERSCORE*/
@@ -281,8 +281,8 @@ typedef struct macho_reloc {
 typedef struct macho_section_data {
     /*@dependent@*/ yasm_symrec *sym; /* symbol created for this section */
     long scnum;			/* section number (0=first section) */
-    const char *segname;	/* segment name in file */
-    const char *sectname;	/* section name in file */
+    /*@only@*/ char *segname;   /* segment name in file */
+    /*@only@*/ char *sectname;  /* section name in file */
     unsigned long flags;	/* S_* flags */
     unsigned long size;		/* size of raw data (section data) in bytes */
     unsigned long offset;	/* offset in raw data within file in bytes */
@@ -293,7 +293,6 @@ typedef struct macho_section_data {
 
 
 typedef struct macho_symrec_data {
-    /*@owned@*/ /*@null@*/ yasm_expr *size; /* size if COMMON declaration */
     unsigned long index;	/* index in output order */
     yasm_intnum *value;		/* valid after writing symtable to file */
     unsigned long length;	/* length + 1 (plus auto underscore) */
@@ -306,14 +305,11 @@ typedef struct yasm_objfmt_macho {
 
     long parse_scnum;		/* sect numbering in parser */
     int bits;			/* 32 / 64 */
-
-    yasm_object *object;
-    yasm_symtab *symtab;
-    /*@dependent@ */ yasm_arch *arch;
 } yasm_objfmt_macho;
 
 
 typedef struct macho_objfmt_output_info {
+    yasm_object *object;
     yasm_objfmt_macho *objfmt_macho;
     yasm_errwarns *errwarns;
     /*@dependent@ */ FILE *f;
@@ -360,27 +356,25 @@ yasm_objfmt_module yasm_macho32_LTX_objfmt;
 yasm_objfmt_module yasm_macho64_LTX_objfmt;
 
 static yasm_objfmt *
-macho_objfmt_create_common(yasm_object *object, yasm_arch *a,
-			   yasm_objfmt_module *module, int bits_pref)
+macho_objfmt_create_common(yasm_object *object, yasm_objfmt_module *module,
+			   int bits_pref)
 {
     yasm_objfmt_macho *objfmt_macho = yasm_xmalloc(sizeof(yasm_objfmt_macho));
 
     objfmt_macho->objfmt.module = module;
-    objfmt_macho->object = object;
-    objfmt_macho->symtab = yasm_object_get_symtab(object);
-    objfmt_macho->arch = a;
 
     /* Only support x86 arch for now */
-    if (yasm__strcasecmp(yasm_arch_keyword(a), "x86") != 0) {
+    if (yasm__strcasecmp(yasm_arch_keyword(object->arch), "x86") != 0) {
 	yasm_xfree(objfmt_macho);
 	return NULL;
     }
 
     /* Support x86 and amd64 machines of x86 arch */
-    if (yasm__strcasecmp(yasm_arch_get_machine(a), "x86") == 0 &&
+    if (yasm__strcasecmp(yasm_arch_get_machine(object->arch), "x86") == 0 &&
 	(bits_pref == 0 || bits_pref == 32))
 	objfmt_macho->bits = 32;
-    else if (yasm__strcasecmp(yasm_arch_get_machine(a), "amd64") == 0 &&
+    else if (yasm__strcasecmp(yasm_arch_get_machine(object->arch),
+			      "amd64") == 0 &&
 	     (bits_pref == 0 || bits_pref == 64))
 	objfmt_macho->bits = 64;
     else {
@@ -393,12 +387,12 @@ macho_objfmt_create_common(yasm_object *object, yasm_arch *a,
 }
 
 static yasm_objfmt *
-macho_objfmt_create(yasm_object *object, yasm_arch *a)
+macho_objfmt_create(yasm_object *object)
 {
     yasm_objfmt *objfmt;
     yasm_objfmt_macho *objfmt_macho;
 
-    objfmt = macho_objfmt_create_common(object, a, &yasm_macho_LTX_objfmt, 0);
+    objfmt = macho_objfmt_create_common(object, &yasm_macho_LTX_objfmt, 0);
     if (objfmt) {
 	objfmt_macho = (yasm_objfmt_macho *)objfmt;
 	/* Figure out which bitness of object format to use */
@@ -411,15 +405,15 @@ macho_objfmt_create(yasm_object *object, yasm_arch *a)
 }
 
 static yasm_objfmt *
-macho32_objfmt_create(yasm_object *object, yasm_arch *a)
+macho32_objfmt_create(yasm_object *object)
 {
-    return macho_objfmt_create_common(object, a, &yasm_macho32_LTX_objfmt, 32);
+    return macho_objfmt_create_common(object, &yasm_macho32_LTX_objfmt, 32);
 }
 
 static yasm_objfmt *
-macho64_objfmt_create(yasm_object *object, yasm_arch *a)
+macho64_objfmt_create(yasm_object *object)
 {
-    return macho_objfmt_create_common(object, a, &yasm_macho64_LTX_objfmt, 64);
+    return macho_objfmt_create_common(object, &yasm_macho64_LTX_objfmt, 64);
 }
 
 static int
@@ -446,7 +440,7 @@ macho_objfmt_output_value(yasm_value *value, unsigned char *buf,
      * cross-section, or non-PC-relative reference (those are handled below).
      */
     switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
-				    info->objfmt_macho->arch)) {
+				    info->object->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -580,7 +574,7 @@ macho_objfmt_output_value(yasm_value *value, unsigned char *buf,
 	yasm_intnum_calc(intn, YASM_EXPR_ADD, intn2);
     }
 
-    retval = yasm_arch_intnum_tobytes(objfmt_macho->arch, intn, buf, destsize,
+    retval = yasm_arch_intnum_tobytes(info->object->arch, intn, buf, destsize,
 				      valsize, 0, bc, warn);
     /*printf("val %ld\n",yasm_intnum_get_int(intn));*/
     yasm_intnum_destroy(intn);
@@ -640,10 +634,6 @@ macho_objfmt_output_section(yasm_section *sect, /*@null@ */ void *d)
 	(macho_objfmt_output_info *) d;
     /*@dependent@ *//*@null@ */ macho_section_data *msd;
 
-    /* FIXME: Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     msd = yasm_section_get_data(sect, &macho_section_data_cb);
     assert(msd != NULL);
@@ -665,10 +655,6 @@ macho_objfmt_output_relocs(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ macho_section_data *msd;
     macho_reloc *reloc;
 
-    /* FIXME: Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     reloc = (macho_reloc *)yasm_section_relocs_first(sect);
     while (reloc) {
 	unsigned char *localbuf = info->buf;
@@ -734,11 +720,15 @@ macho_objfmt_is_section_label(yasm_symrec *sym)
 	 * If there is not a section, leave as debugging symbol.
 	 */
 	if (sect) {
-	    if (strcmp(yasm_symrec_get_name(sym),
-		       yasm_section_get_name(sect)) == 0)
+            /*@dependent@*/ /*@null@*/ macho_section_data *msd;
+
+            msd = yasm_section_get_data(sect, &macho_section_data_cb);
+            if (msd) {
+                if (msd->sym == sym)
 		return 1;	/* don't store section names */
 	}
     }
+    }
     return 0;
 }
 
@@ -750,10 +740,6 @@ macho_objfmt_output_secthead(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ macho_section_data *msd;
     unsigned char *localbuf;
 
-    /* Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     objfmt_macho = info->objfmt_macho;
     msd = yasm_section_get_data(sect, &macho_section_data_cb);
@@ -761,8 +747,12 @@ macho_objfmt_output_secthead(yasm_section *sect, /*@null@*/ void *d)
 
     localbuf = info->buf;
 
-    fwrite(msd->sectname, 16, 1, info->f);
-    fwrite(msd->segname, 16, 1, info->f);
+    memset(localbuf, 0, 16);
+    strncpy((char *)localbuf, msd->sectname, 16);
+    localbuf += 16;
+    memset(localbuf, 0, 16);
+    strncpy((char *)localbuf, msd->segname, 16);
+    localbuf += 16;
     /* section address, size depend on 32/64 bit mode */
     YASM_WRITE_32_L(localbuf, msd->vmoff);	/* address in memory */
     if (info->is_64)
@@ -800,9 +790,9 @@ macho_objfmt_output_secthead(yasm_section *sect, /*@null@*/ void *d)
     YASM_WRITE_32_L(localbuf, 0);	/* reserved 2 */
 
     if (info->is_64)
-	fwrite(info->buf, MACHO_SECTCMD64_SIZE - 32, 1, info->f);	/* 2*16 byte strings already written */
+	fwrite(info->buf, MACHO_SECTCMD64_SIZE, 1, info->f);
     else
-	fwrite(info->buf, MACHO_SECTCMD_SIZE - 32, 1, info->f);	/* 2*16 byte strings already written */
+	fwrite(info->buf, MACHO_SECTCMD_SIZE, 1, info->f);
 
     return 0;
 }
@@ -886,36 +876,17 @@ macho_objfmt_output_symtable(yasm_symrec *sym, /*@null@*/ void *d)
 	    if (sect) {
 		/*@dependent@*/ /*@null@*/ macho_section_data *msd;
 
-		if (strcmp(yasm_symrec_get_name(sym),
-			   yasm_section_get_name(sect)) == 0) {
+                msd = yasm_section_get_data(sect, &macho_section_data_cb);
+                if (msd) {
+                    if (msd->sym == sym) {
 		    /* don't store section names */
 		    yasm_intnum_destroy(val);
 		    return 0;
 		}
-		msd = yasm_section_get_data(sect, &macho_section_data_cb);
-		if (msd) {
 		    scnum = msd->scnum;
 		    n_type = N_SECT;
-		} else {
-		    if (yasm_section_is_absolute(sect)) {
-			yasm_expr *abs_start;
-
-			abs_start =
-			    yasm_expr_copy(yasm_section_get_start(sect));
-			intn = yasm_expr_get_intnum(&abs_start, 1);
-			if (!intn) {
-			    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-				N_("absolute section start not an integer expression"));
-			    yasm_errwarn_propagate(info->errwarns,
-						   abs_start->line);
-			} else
-			    value = yasm_intnum_get_uint(intn);
-			yasm_expr_destroy(abs_start);
-
-			scnum = -2;	/* -2 = absolute symbol */
-		    } else
-			yasm_internal_error(N_("didn't understand section"));
-		}
+		} else
+		    yasm_internal_error(N_("didn't understand section"));
 		if (precbc)
 		    value += yasm_bc_next_offset(precbc);
 		/* all values are subject to correction: base offset is first
@@ -949,13 +920,14 @@ macho_objfmt_output_symtable(yasm_symrec *sym, /*@null@*/ void *d)
 	    scnum = -1;
 	    /*n_desc = REFERENCE_FLAG_UNDEFINED_LAZY;	* FIXME: see definition of REFERENCE_FLAG_* above */
 	} else if (vis & YASM_SYM_COMMON) {
+	    yasm_expr **csize = yasm_symrec_get_common_size(sym);
 	    n_type = N_UNDF | N_EXT;
-	    if (symd) {
-		intn = yasm_expr_get_intnum(&symd->size, 1);
+	    if (csize) {
+		intn = yasm_expr_get_intnum(csize, 1);
 		if (!intn) {
 		    yasm_error_set(YASM_ERROR_NOT_CONSTANT,
 				   N_("COMMON data size not an integer expression"));
-		    yasm_errwarn_propagate(info->errwarns, symd->size->line);
+		    yasm_errwarn_propagate(info->errwarns, (*csize)->line);
 		} else
 		    yasm_intnum_set_uint(val, yasm_intnum_get_uint(intn));
 	    }
@@ -1018,10 +990,6 @@ macho_objfmt_calc_sectsize(yasm_section *sect, /*@null@ */ void *d)
 	(macho_objfmt_output_info *) d;
     /*@dependent@ *//*@null@ */ macho_section_data *msd;
 
-    /* FIXME: Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     msd = yasm_section_get_data(sect, &macho_section_data_cb);
     assert(msd != NULL);
@@ -1042,10 +1010,10 @@ macho_objfmt_calc_sectsize(yasm_section *sect, /*@null@ */ void *d)
 
 /* write object */
 static void
-macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
-		    /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
+macho_objfmt_output(yasm_object *object, FILE *f, int all_syms,
+		    yasm_errwarns *errwarns)
 {
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *) objfmt;
+    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)object->objfmt;
     macho_objfmt_output_info info;
     unsigned char *localbuf;
     unsigned long symtab_count = 0;
@@ -1058,6 +1026,7 @@ macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     unsigned long long_int_bytes;
     const char pad_data[3] = "\0\0\0";
 
+    info.object = object;
     info.objfmt_macho = objfmt_macho;
     info.errwarns = errwarns;
     info.f = f;
@@ -1106,7 +1075,7 @@ macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     info.strlength = 1;		/* string table starts with a zero byte */
     info.all_syms = all_syms || info.is_64;
     /*info.all_syms = 1;		* force all syms into symbol table */
-    yasm_symtab_traverse(objfmt_macho->symtab, &info, macho_objfmt_count_sym);
+    yasm_symtab_traverse(object->symtab, &info, macho_objfmt_count_sym);
     symtab_count = info.indx;
 
     /* write raw section data first */
@@ -1122,12 +1091,10 @@ macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     info.vmsize = 0;
     info.filesize = 0;
     info.offset = headsize;
-    yasm_object_sections_traverse(objfmt_macho->object, &info,
-				  macho_objfmt_calc_sectsize);
+    yasm_object_sections_traverse(object, &info, macho_objfmt_calc_sectsize);
 
     /* output sections to file */
-    yasm_object_sections_traverse(objfmt_macho->object, &info,
-				  macho_objfmt_output_section);
+    yasm_object_sections_traverse(object, &info, macho_objfmt_output_section);
 
     fileoff_sections = ftell(f);
 
@@ -1223,8 +1190,7 @@ macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     /* offset to relocs for first section */
     info.rel_base = align32((long)fileoffset + (long)info.filesize);
     info.s_reloff = 0;		/* offset for relocs of following sections */
-    yasm_object_sections_traverse(objfmt_macho->object, &info,
-				  macho_objfmt_output_secthead);
+    yasm_object_sections_traverse(object, &info, macho_objfmt_output_secthead);
 
     localbuf = info.buf;
     /* write out symbol command */
@@ -1256,25 +1222,16 @@ macho_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     }
 
     /* relocation data */
-    yasm_object_sections_traverse(objfmt_macho->object, &info,
-				  macho_objfmt_output_relocs);
+    yasm_object_sections_traverse(object, &info, macho_objfmt_output_relocs);
 
     /* symbol table (NLIST) */
     info.indx = 1;		/* restart symbol table indices */
-    yasm_symtab_traverse(objfmt_macho->symtab, &info,
-			 macho_objfmt_output_symtable);
+    yasm_symtab_traverse(object->symtab, &info, macho_objfmt_output_symtable);
 
     /* symbol strings */
     fwrite(pad_data, 1, 1, f);
-    yasm_symtab_traverse(objfmt_macho->symtab, &info,
-			 macho_objfmt_output_str);
-#if 0
-    /* relocation fixup: set internal symbol locations into byte code within
-     * file.
-     */
-    yasm_object_sections_traverse(objfmt_macho->object, &info,
-				  macho_objfmt_fixup_relocs);
-#endif
+    yasm_symtab_traverse(object->symtab, &info, macho_objfmt_output_str);
+
     yasm_intnum_destroy(val);
     yasm_xfree(info.buf);
 }
@@ -1286,44 +1243,44 @@ macho_objfmt_destroy(yasm_objfmt *objfmt)
 }
 
 static macho_section_data *
-macho_objfmt_init_new_section(yasm_objfmt_macho * objfmt_macho,
-			      yasm_section *sect, const char *sectname,
-			      unsigned long line)
+macho_objfmt_init_new_section(yasm_object *object, yasm_section *sect,
+			      const char *sectname, unsigned long line)
 {
+    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)object->objfmt;
     macho_section_data *data;
     yasm_symrec *sym;
 
     data = yasm_xmalloc(sizeof(macho_section_data));
     data->scnum = objfmt_macho->parse_scnum++;
-    data->segname = "";
-    data->sectname = "";
+    data->segname = NULL;
+    data->sectname = NULL;
     data->flags = S_REGULAR;
     data->size = 0;
-    data->nreloc = 0;
     data->offset = 0;
+    data->vmoff = 0;
+    data->nreloc = 0;
+    data->extreloc = 0;
     yasm_section_add_data(sect, &macho_section_data_cb, data);
 
-    sym = yasm_symtab_define_label(objfmt_macho->symtab, sectname,
+    sym = yasm_symtab_define_label(object->symtab, sectname,
 				   yasm_section_bcs_first(sect), 1, line);
     data->sym = sym;
     return data;
 }
 
 static yasm_section *
-macho_objfmt_add_default_section(yasm_objfmt *objfmt)
+macho_objfmt_add_default_section(yasm_object *object)
 {
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *) objfmt;
     yasm_section *retval;
     macho_section_data *msd;
     int isnew;
 
-    retval =
-	yasm_object_get_general(objfmt_macho->object, ".text", 0, 0, 1, 0,
-				&isnew, 0);
+    retval = yasm_object_get_general(object, "LC_SEGMENT.__TEXT.__text", 0, 0, 1, 0,
+                                     &isnew, 0);
     if (isnew) {
-	msd = macho_objfmt_init_new_section(objfmt_macho, retval, ".text", 0);
-	msd->segname = "__TEXT";
-	msd->sectname = "__text";
+	msd = macho_objfmt_init_new_section(object, retval, ".text", 0);
+        msd->segname = yasm__xstrdup("__TEXT");
+        msd->sectname = yasm__xstrdup("__text");
 	msd->flags = S_ATTR_PURE_INSTRUCTIONS;
 	yasm_section_set_align(retval, 0, 0);
 	yasm_section_set_default(retval, 1);
@@ -1332,20 +1289,20 @@ macho_objfmt_add_default_section(yasm_objfmt *objfmt)
 }
 
 static /*@observer@*/ /*@null@*/ yasm_section *
-macho_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
+macho_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
 			    /*@unused@*/ /*@null@*/
 			    yasm_valparamhead *objext_valparams,
 			    unsigned long line)
 {
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *) objfmt;
-    yasm_valparam *vp = yasm_vps_first(valparams);
+    yasm_valparam *vp;
     yasm_section *retval;
     int isnew;
-    const char *f_segname, *f_sectname;
+    /*@only@*/ char *f_sectname;
     unsigned long flags;
     unsigned long align;
     int flags_override = 0;
-    char *sectname;
+    const char *sectname;
+    char *realname;
     int resonly = 0;
     macho_section_data *msd;
     size_t i;
@@ -1415,10 +1372,26 @@ macho_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    S_ATTR_NO_DEAD_STRIP, 0}
     };
 
-    if (!vp || vp->param || !vp->val)
-	return NULL;
+    struct macho_section_switch_data {
+        /*@only@*/ /*@null@*/ char *f_segname;
+        /*@only@*/ /*@null@*/ yasm_intnum *align_intn;
+    } data;
 
-    sectname = vp->val;
+    static const yasm_dir_help help[] = {
+        { "segname", 1, yasm_dir_helper_string,
+          offsetof(struct macho_section_switch_data, f_segname), 0 },
+        { "align", 1, yasm_dir_helper_intn,
+          offsetof(struct macho_section_switch_data, align_intn), 0 }
+    };
+
+    data.f_segname = NULL;
+    data.align_intn = NULL;
+
+    vp = yasm_vps_first(valparams);
+    sectname = yasm_vp_string(vp);
+    if (!sectname)
+        return NULL;
+    vp = yasm_vps_next(vp);
 
     /* translate .text,.data,.bss to __text,__data,__bss... */
     for (i=0; i<NELEMS(section_name_translation); i++) {
@@ -1427,35 +1400,46 @@ macho_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
     }
 
     if (i == NELEMS(section_name_translation)) {
+        const char *s;
+        if (vp && !vp->val && (s = yasm_vp_string(vp))) {
+            /* Treat as SEGNAME, SECTNAME */
+            if (strlen(sectname) > 16)
 	yasm_warn_set(YASM_WARN_GENERAL,
-		      N_("Unknown section type, defaulting to .text"));
-	i = 0;
-    }
+                    N_("segment name is too long, max 16 chars; truncating"));
+            data.f_segname = yasm__xstrndup(sectname, 16);
+            if (strlen(s) > 16)
+                yasm_warn_set(YASM_WARN_GENERAL,
+                    N_("section name is too long, max 16 chars; truncating"));
+            f_sectname = yasm__xstrndup(s, 16);
+            flags = S_REGULAR;
+            align = 0;
 
-    f_segname = section_name_translation[i].seg;
-    f_sectname = section_name_translation[i].sect;
+            sectname = s;
+            vp = yasm_vps_next(vp);
+        } else {
+            data.f_segname = NULL;
+            if (strlen(sectname) > 16)
+                yasm_warn_set(YASM_WARN_GENERAL,
+                    N_("section name is too long, max 16 chars; truncating"));
+            f_sectname = yasm__xstrndup(sectname, 16);
+            flags = S_ATTR_SOME_INSTRUCTIONS;
+            align = 0;
+    }
+    } else {
+        data.f_segname = yasm__xstrdup(section_name_translation[i].seg);
+        f_sectname = yasm__xstrdup(section_name_translation[i].sect);
     flags = section_name_translation[i].flags;
     align = section_name_translation[i].align;
+    }
 
-    while ((vp = yasm_vps_next(vp))) {
-	if (!vp->val) {
-	    yasm_warn_set(YASM_WARN_GENERAL,
-			  N_("Unrecognized numeric qualifier"));
-	    continue;
-	}
+    flags_override = yasm_dir_helper(object, vp, line, help, NELEMS(help),
+                                     &data, yasm_dir_helper_valparam_warn);
+    if (flags_override < 0)
+        return NULL;    /* error occurred */
 
-	flags_override = 1;
-	if (yasm__strcasecmp(vp->val, "align") == 0 && vp->param) {
-	    /*@dependent@ *//*@null@ */ const yasm_intnum *align_expr;
-
-	    align_expr = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!align_expr) {
-		yasm_error_set(YASM_ERROR_VALUE,
-			       N_("argument to `%s' is not an integer"),
-			       vp->val);
-		return NULL;
-	    }
-	    align = yasm_intnum_get_uint(align_expr);
+    if (data.align_intn) {
+        align = yasm_intnum_get_uint(data.align_intn);
+        yasm_intnum_destroy(data.align_intn);
 
 	    /* Alignments must be a power of two. */
 	    if (!is_exp2(align)) {
@@ -1471,24 +1455,30 @@ macho_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 		    N_("macho implementation does not support alignments > 16384"));
 		return NULL;
 	    }
-	} else
-	    yasm_warn_set(YASM_WARN_GENERAL,
-			  N_("Unrecognized qualifier `%s'"), vp->val);
     }
 
-    retval =
-	yasm_object_get_general(objfmt_macho->object, sectname, 0, align, 1,
-				resonly, &isnew, line);
+    if (!data.f_segname) {
+        yasm_warn_set(YASM_WARN_GENERAL,
+                      N_("Unknown section name, defaulting to __TEXT segment"));
+        data.f_segname = yasm__xstrdup("__TEXT");
+    }
+
+    /* Build a unique sectname from f_segname and f_sectname. */
+    realname = yasm_xmalloc(strlen("LC_SEGMENT") + 1 + strlen(data.f_segname) + 1 +
+                            strlen(f_sectname) + 1);
+    sprintf(realname, "LC_SEGMENT.%s.%s", data.f_segname, f_sectname);
+    retval = yasm_object_get_general(object, realname, 0, align, 1, resonly,
+				     &isnew, line);
+    yasm_xfree(realname);
 
     if (isnew)
-	msd = macho_objfmt_init_new_section(objfmt_macho, retval, sectname,
-					    line);
+	msd = macho_objfmt_init_new_section(object, retval, sectname, line);
     else
 	msd = yasm_section_get_data(retval, &macho_section_data_cb);
 
     if (isnew || yasm_section_is_default(retval)) {
 	yasm_section_set_default(retval, 0);
-	msd->segname = f_segname;
+        msd->segname = data.f_segname;
 	msd->sectname = f_sectname;
 	msd->flags = flags;
 	yasm_section_set_align(retval, align, line);
@@ -1501,6 +1491,9 @@ macho_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 static void
 macho_section_data_destroy(void *data)
 {
+    macho_section_data *msd = (macho_section_data *) data;
+    yasm_xfree(msd->segname);
+    yasm_xfree(msd->sectname);
     yasm_xfree(data);
 }
 
@@ -1513,61 +1506,15 @@ macho_section_data_print(void *data, FILE *f, int indent_level)
     yasm_symrec_print(msd->sym, f, indent_level + 1);
     fprintf(f, "%*sscnum=%ld\n", indent_level, "", msd->scnum);
     fprintf(f, "%*sflags=0x%lx\n", indent_level, "", msd->flags);
-    fprintf(f, "%*ssize=%ld\n", indent_level, "", msd->size);
-    fprintf(f, "%*snreloc=%ld\n", indent_level, "", msd->nreloc);
-    fprintf(f, "%*soffset=%ld\n", indent_level, "", msd->offset);
-}
-
-static yasm_symrec *
-macho_objfmt_extern_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			    /*@null@*/ yasm_valparamhead *objext_valparams,
-			    unsigned long line)
-{
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)objfmt;
-
-    return yasm_symtab_declare(objfmt_macho->symtab, name, YASM_SYM_EXTERN,
-			       line);
-}
-
-static yasm_symrec *
-macho_objfmt_global_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			    /*@null@*/ yasm_valparamhead *objext_valparams,
-			    unsigned long line)
-{
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)objfmt;
-
-    return yasm_symtab_declare(objfmt_macho->symtab, name, YASM_SYM_GLOBAL,
-			       line);
-}
-
-static yasm_symrec *
-macho_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
-			    /*@only@*/ yasm_expr *size,
-			    /*@unused@*/ /*@null@*/
-			    yasm_valparamhead *objext_valparams,
-			    unsigned long line)
-{
-    yasm_objfmt_macho *objfmt_macho = (yasm_objfmt_macho *)objfmt;
-    yasm_symrec *sym;
-    macho_symrec_data *sym_data;
-
-    sym = yasm_symtab_declare(objfmt_macho->symtab, name, YASM_SYM_COMMON,
-			      line);
-
-    sym_data = yasm_xmalloc(sizeof(macho_symrec_data));
-
-    sym_data->size = size;
-    yasm_symrec_add_data(sym, &macho_symrec_data_cb, sym_data);
-    return sym;
+    fprintf(f, "%*ssize=%lu\n", indent_level, "", msd->size);
+    fprintf(f, "%*snreloc=%lu\n", indent_level, "", msd->nreloc);
+    fprintf(f, "%*soffset=%lu\n", indent_level, "", msd->offset);
+    fprintf(f, "%*sextreloc=%u\n", indent_level, "", msd->extreloc);
 }
 
 static void
 macho_symrec_data_destroy(void *data)
 {
-    macho_symrec_data *csymd = (macho_symrec_data *)data;
-
-    if (csymd->size)
-	yasm_expr_destroy(csymd->size);
     yasm_xfree(data);
 }
 
@@ -1576,12 +1523,6 @@ macho_symrec_data_print(void *data, FILE *f, int indent_level)
 {
     macho_symrec_data *msd = (macho_symrec_data *)data;
 
-    fprintf(f, "%*ssize=", indent_level, "");
-    if (msd->size)
-	yasm_expr_print(msd->size, f);
-    else
-	fprintf(f, "nil");
-    fprintf(f, "\n");
     fprintf(f, "%*sindex=%ld\n", indent_level, "", msd->index);
     fprintf(f, "%*svalue=", indent_level, "");
     if (msd->value)
@@ -1591,18 +1532,6 @@ macho_symrec_data_print(void *data, FILE *f, int indent_level)
 }
 
 
-static int
-macho_objfmt_directive(/*@unused@*/ yasm_objfmt *objfmt,
-		       /*@unused@*/ const char *name,
-		       /*@unused@*/ /*@null@*/ yasm_valparamhead *valparams,
-		       /*@unused@*/ /*@null@*/
-		       yasm_valparamhead *objext_valparams,
-		       /*@unused@*/ unsigned long line)
-{
-    return 1;			/* no objfmt directives */
-}
-
-
 /* Define valid debug formats to use with this object format */
 static const char *macho_objfmt_dbgfmt_keywords[] = {
     "null",
@@ -1617,15 +1546,12 @@ yasm_objfmt_module yasm_macho_LTX_objfmt = {
     32,
     macho_objfmt_dbgfmt_keywords,
     "null",
+    NULL,   /* no directives */
     macho_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
     macho_objfmt_add_default_section,
-    macho_objfmt_section_switch,
-    macho_objfmt_extern_declare,
-    macho_objfmt_global_declare,
-    macho_objfmt_common_declare,
-    macho_objfmt_directive
+    macho_objfmt_section_switch
 };
 
 yasm_objfmt_module yasm_macho32_LTX_objfmt = {
@@ -1635,15 +1561,12 @@ yasm_objfmt_module yasm_macho32_LTX_objfmt = {
     32,
     macho_objfmt_dbgfmt_keywords,
     "null",
+    NULL,   /* no directives */
     macho32_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
     macho_objfmt_add_default_section,
-    macho_objfmt_section_switch,
-    macho_objfmt_extern_declare,
-    macho_objfmt_global_declare,
-    macho_objfmt_common_declare,
-    macho_objfmt_directive
+    macho_objfmt_section_switch
 };
 
 yasm_objfmt_module yasm_macho64_LTX_objfmt = {
@@ -1653,13 +1576,10 @@ yasm_objfmt_module yasm_macho64_LTX_objfmt = {
     64,
     macho_objfmt_dbgfmt_keywords,
     "null",
+    NULL,   /* no directives */
     macho64_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
     macho_objfmt_add_default_section,
-    macho_objfmt_section_switch,
-    macho_objfmt_extern_declare,
-    macho_objfmt_global_declare,
-    macho_objfmt_common_declare,
-    macho_objfmt_directive
+    macho_objfmt_section_switch
 };
