@@ -1,7 +1,7 @@
 /*
  * Relocatable Dynamic Object File Format (RDOFF) version 2 format
  *
- *  Copyright (C) 2006  Peter Johnson
+ *  Copyright (C) 2006-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: rdf-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: rdf-objfmt.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -100,10 +100,6 @@ typedef struct rdf_section_data {
 } rdf_section_data;
 
 typedef struct rdf_symrec_data {
-    /*@owned@*/ /*@null@*/ yasm_expr *size; /* size if COMMON declaration */
-    unsigned long align;		/* alignment if COMMON declaration */
-
-    unsigned int flags;			/* import/export/type flags */
     unsigned int segment;		/* assigned RDF "segment" index */
 } rdf_symrec_data;
 
@@ -118,15 +114,12 @@ typedef struct yasm_objfmt_rdf {
 
     long parse_scnum;		    /* sect numbering in parser */
 
-    yasm_object *object;
-    yasm_symtab *symtab;
-    /*@dependent@*/ yasm_arch *arch;
-
     /*@owned@*/ xdf_str_head module_names;
     /*@owned@*/ xdf_str_head library_names;
 } yasm_objfmt_rdf;
 
 typedef struct rdf_objfmt_output_info {
+    yasm_object *object;
     yasm_objfmt_rdf *objfmt_rdf;
     yasm_errwarns *errwarns;
     /*@dependent@*/ FILE *f;
@@ -159,30 +152,21 @@ yasm_objfmt_module yasm_rdf_LTX_objfmt;
 
 
 static /*@dependent@*/ rdf_symrec_data *
-rdf_objfmt_sym_set_data(yasm_symrec *sym,
-			/*@only@*/ /*@null@*/ yasm_expr *size,
-			unsigned long align, unsigned int flags)
+rdf_objfmt_sym_set_data(yasm_symrec *sym, unsigned int segment)
 {
     rdf_symrec_data *rsymd = yasm_xmalloc(sizeof(rdf_symrec_data));
 
-    rsymd->size = size;
-    rsymd->align = align;
-    rsymd->flags = flags;
-    rsymd->segment = 0;
+    rsymd->segment = segment;
 
     yasm_symrec_add_data(sym, &rdf_symrec_data_cb, rsymd);
     return rsymd;
 }
 
 static yasm_objfmt *
-rdf_objfmt_create(yasm_object *object, yasm_arch *a)
+rdf_objfmt_create(yasm_object *object)
 {
     yasm_objfmt_rdf *objfmt_rdf = yasm_xmalloc(sizeof(yasm_objfmt_rdf));
 
-    objfmt_rdf->object = object;
-    objfmt_rdf->symtab = yasm_object_get_symtab(object);
-    objfmt_rdf->arch = a;
-
     /* We theoretically support all arches, so don't check.
      * Really we only support byte-addressable ones.
      */
@@ -221,7 +205,7 @@ rdf_objfmt_output_value(yasm_value *value, unsigned char *buf,
      * cross-section, or non-PC-relative reference (those are handled below).
      */
     switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
-				    info->objfmt_rdf->arch)) {
+				    info->object->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -308,7 +292,7 @@ rdf_objfmt_output_value(yasm_value *value, unsigned char *buf,
 	yasm_intnum_calc(intn, YASM_EXPR_ADD, intn2);
     }
 
-    retval = yasm_arch_intnum_tobytes(objfmt_rdf->arch, intn, buf, destsize,
+    retval = yasm_arch_intnum_tobytes(info->object->arch, intn, buf, destsize,
 				      valsize, 0, bc, warn);
     yasm_intnum_destroy(intn);
     return retval;
@@ -362,10 +346,6 @@ rdf_objfmt_output_section_mem(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ rdf_section_data *rsd;
     unsigned long size;
 
-    /* Don't output absolute sections */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     rsd = yasm_section_get_data(sect, &rdf_section_data_cb);
     assert(rsd != NULL);
@@ -408,10 +388,6 @@ rdf_objfmt_output_section_reloc(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ rdf_section_data *rsd;
     rdf_reloc *reloc;
 
-    /* Don't output absolute sections */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     rsd = yasm_section_get_data(sect, &rdf_section_data_cb);
     assert(rsd != NULL);
@@ -456,10 +432,6 @@ rdf_objfmt_output_section_file(yasm_section *sect, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ rdf_section_data *rsd;
     unsigned char *localbuf;
 
-    /* Don't output absolute sections */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     rsd = yasm_section_get_data(sect, &rdf_section_data_cb);
     assert(rsd != NULL);
@@ -491,6 +463,61 @@ rdf_objfmt_output_section_file(yasm_section *sect, /*@null@*/ void *d)
     return 0;
 }
 
+#define FLAG_EXT    0x1000
+#define FLAG_GLOB   0x2000
+#define FLAG_SET    0x4000
+#define FLAG_CLR    0x8000
+#define FLAG_MASK   0x0fff
+
+static int
+rdf_helper_flag(void *obj, yasm_valparam *vp, unsigned long line, void *d,
+                uintptr_t flag)
+{
+    yasm_symrec *sym = (yasm_symrec *)obj;
+    yasm_sym_vis vis = yasm_symrec_get_visibility(sym);
+    unsigned int *flags = (unsigned int *)d;
+
+    if (((vis & YASM_SYM_GLOBAL) && (flag & FLAG_GLOB)) ||
+        ((vis & YASM_SYM_EXTERN) && (flag & FLAG_EXT))) {
+        if (flag & FLAG_SET)
+            *flags |= flag & FLAG_MASK;
+        else if (flag & FLAG_CLR)
+            *flags &= ~(flag & FLAG_MASK);
+    }
+    return 0;
+}
+
+static unsigned int
+rdf_parse_flags(yasm_symrec *sym)
+{
+    /*@dependent@*/ /*@null@*/ yasm_valparamhead *objext_valparams =
+	yasm_symrec_get_objext_valparams(sym);
+    unsigned int flags = 0;
+
+    static const yasm_dir_help help[] = {
+        { "data", 0, rdf_helper_flag, 0,
+          FLAG_EXT|FLAG_GLOB|FLAG_SET|SYM_DATA },
+        { "object", 0, rdf_helper_flag, 0,
+          FLAG_EXT|FLAG_GLOB|FLAG_SET|SYM_DATA },
+        { "proc", 0, rdf_helper_flag, 0,
+          FLAG_EXT|FLAG_GLOB|FLAG_SET|SYM_FUNCTION },
+        { "function", 0, rdf_helper_flag, 0,
+          FLAG_EXT|FLAG_GLOB|FLAG_SET|SYM_FUNCTION },
+        { "import", 0, rdf_helper_flag, 0, FLAG_EXT|FLAG_SET|SYM_IMPORT },
+        { "export", 0, rdf_helper_flag, 0, FLAG_GLOB|FLAG_SET|SYM_GLOBAL },
+        { "far", 0, rdf_helper_flag, 0, FLAG_EXT|FLAG_SET|SYM_FAR },
+        { "near", 0, rdf_helper_flag, 0, FLAG_EXT|FLAG_CLR|SYM_FAR }
+    };
+
+    if (!objext_valparams)
+	return 0;
+
+    yasm_dir_helper(sym, yasm_vps_first(objext_valparams), 0, help,
+                    NELEMS(help), &flags, yasm_dir_helper_valparam_warn);
+
+    return flags;
+}
+
 static int
 rdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 {
@@ -503,7 +530,6 @@ rdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
     /*@dependent@*/ /*@null@*/ yasm_section *sect;
     /*@dependent@*/ /*@null@*/ yasm_bytecode *precbc;
     unsigned char *localbuf;
-    rdf_symrec_data *rsymd;
 
     assert(info != NULL);
 
@@ -523,20 +549,15 @@ rdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 
 	/* it's a label: get value and offset. */
 	csectd = yasm_section_get_data(sect, &rdf_section_data_cb);
-	if (csectd) {
+	if (csectd)
 	    scnum = csectd->scnum;
-	} else if (yasm_section_is_absolute(sect)) {
-	    yasm_warn_set(YASM_WARN_GENERAL,
-			  N_("rdf does not support exporting absolutes"));
-	    yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_line(sym));
-	    return 0;
-	} else
+	else
 	    yasm_internal_error(N_("didn't understand section"));
 	value = yasm_bc_next_offset(precbc);
     } else if (yasm_symrec_get_equ(sym)) {
 	yasm_warn_set(YASM_WARN_GENERAL,
-		      N_("rdf does not support exporting EQU/absolute values"));
-	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_line(sym));
+	    N_("rdf does not support exporting EQU/absolute values"));
+	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
 	return 0;
     }
 
@@ -547,50 +568,81 @@ rdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 	yasm_warn_set(YASM_WARN_GENERAL,
 		      N_("label name too long, truncating to %d bytes"),
 		      EXIM_LABEL_MAX);
-	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_line(sym));
 	len = EXIM_LABEL_MAX-1;
     }
 
     localbuf = info->buf;
     if (vis & YASM_SYM_GLOBAL) {
-	rsymd = yasm_symrec_get_data(sym, &rdf_symrec_data_cb);
-	if (!rsymd)
-	    yasm_internal_error(N_("rdf: no symbol data for global symbol"));
 	YASM_WRITE_8(localbuf, RDFREC_GLOBAL);
 	YASM_WRITE_8(localbuf, 6+len+1);	/* record length */
-	YASM_WRITE_8(localbuf, rsymd->flags);	/* flags */
+	YASM_WRITE_8(localbuf, rdf_parse_flags(sym));	/* flags */
 	YASM_WRITE_8(localbuf, scnum);		/* segment referred to */
 	YASM_WRITE_32_L(localbuf, value);	/* offset */
     } else {
-	/* Create new symrec data if it doesn't already exist */
-	rsymd = yasm_symrec_get_data(sym, &rdf_symrec_data_cb);
-	if (!rsymd)
-	    rsymd = rdf_objfmt_sym_set_data(sym, NULL, 0, 0);
-
 	/* Save symbol segment in symrec data (for later reloc gen) */
-	rsymd->segment = info->indx++;
-	scnum = rsymd->segment;
+	scnum = info->indx++;
+	rdf_objfmt_sym_set_data(sym, scnum);
 
 	if (vis & YASM_SYM_COMMON) {
+	    /*@dependent@*/ /*@null@*/ yasm_expr **csize_expr;
 	    const yasm_intnum *intn;
+	    /*@dependent@*/ /*@null@*/ yasm_valparamhead *objext_valparams =
+		yasm_symrec_get_objext_valparams(sym);
+	    unsigned long addralign = 0;
 
 	    YASM_WRITE_8(localbuf, RDFREC_COMMON);
 	    YASM_WRITE_8(localbuf, 8+len+1);	/* record length */
 	    YASM_WRITE_16_L(localbuf, scnum);	/* segment allocated */
 
 	    /* size */
-	    intn = yasm_expr_get_intnum(&rsymd->size, 1);
+	    csize_expr = yasm_symrec_get_common_size(sym);
+	    assert(csize_expr != NULL);
+	    intn = yasm_expr_get_intnum(csize_expr, 1);
 	    if (!intn) {
 		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
 		    N_("COMMON data size not an integer expression"));
-		yasm_errwarn_propagate(info->errwarns,
-				       yasm_symrec_get_line(sym));
 	    } else
 		value = yasm_intnum_get_uint(intn);
 	    YASM_WRITE_32_L(localbuf, value);
-	    YASM_WRITE_16_L(localbuf, rsymd->align);	/* alignment */
+
+	    /* alignment */
+	    if (objext_valparams) {
+		yasm_valparam *vp = yasm_vps_first(objext_valparams);
+		for (; vp; vp = yasm_vps_next(vp)) {
+                    if (!vp->val) {
+                        /*@only@*/ /*@null@*/ yasm_expr *align_expr;
+                        /*@dependent@*/ /*@null@*/
+                        const yasm_intnum *align_intn;
+
+                        if (!(align_expr = yasm_vp_expr(vp,
+                                info->object->symtab,
+                                yasm_symrec_get_decl_line(sym))) ||
+                            !(align_intn = yasm_expr_get_intnum(&align_expr,
+                                                                0))) {
+			    yasm_error_set(YASM_ERROR_VALUE,
+                                N_("argument to `%s' is not an integer"),
+                                vp->val);
+                            if (align_expr)
+                                yasm_expr_destroy(align_expr);
+			    continue;
+			}
+                        addralign = yasm_intnum_get_uint(align_intn);
+                        yasm_expr_destroy(align_expr);
+
+			/* Alignments must be a power of two. */
+			if (!is_exp2(addralign)) {
+			    yasm_error_set(YASM_ERROR_VALUE,
+				N_("alignment constraint is not a power of two"));
+			    continue;
+			}
+                    } else
+			yasm_warn_set(YASM_WARN_GENERAL,
+                            N_("Unrecognized qualifier `%s'"), vp->val);
+		}
+	    }
+	    YASM_WRITE_16_L(localbuf, addralign);
 	} else if (vis & YASM_SYM_EXTERN) {
-	    unsigned int flags = rsymd->flags;
+	    unsigned int flags = rdf_parse_flags(sym);
 	    if (flags & SYM_FAR) {
 		YASM_WRITE_8(localbuf, RDFREC_FARIMPORT);
 		flags &= ~SYM_FAR;
@@ -608,20 +660,23 @@ rdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
     YASM_WRITE_8(localbuf, 0);		/* 0-terminated name */
 
     fwrite(info->buf, (unsigned long)(localbuf-info->buf), 1, info->f);
+
+    yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
     return 0;
 }
 
 static void
-rdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
-		  /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
+rdf_objfmt_output(yasm_object *object, FILE *f, int all_syms,
+		  yasm_errwarns *errwarns)
 {
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
+    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)object->objfmt;
     rdf_objfmt_output_info info;
     unsigned char *localbuf;
     long headerlen, filelen;
     xdf_str *cur;
     size_t len;
 
+    info.object = object;
     info.objfmt_rdf = objfmt_rdf;
     info.errwarns = errwarns;
     info.f = f;
@@ -660,7 +715,7 @@ rdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
 
     /* Output symbol table */
     info.indx = objfmt_rdf->parse_scnum;
-    yasm_symtab_traverse(objfmt_rdf->symtab, &info, rdf_objfmt_output_sym);
+    yasm_symtab_traverse(object->symtab, &info, rdf_objfmt_output_sym);
 
     /* UGH! Due to the fact the relocs go at the beginning of the file, and
      * we only know if we have relocs when we output the sections, we have
@@ -674,12 +729,12 @@ rdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
      *
      * We also calculate the total size of all BSS sections here.
      */
-    if (yasm_object_sections_traverse(objfmt_rdf->object, &info,
+    if (yasm_object_sections_traverse(object, &info,
 				      rdf_objfmt_output_section_mem))
 	return;
 
     /* Output all relocs */
-    if (yasm_object_sections_traverse(objfmt_rdf->object, &info,
+    if (yasm_object_sections_traverse(object, &info,
 				      rdf_objfmt_output_section_reloc))
 	return;
 
@@ -701,7 +756,7 @@ rdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     }
 
     /* Section data (to file) */
-    if (yasm_object_sections_traverse(objfmt_rdf->object, &info,
+    if (yasm_object_sections_traverse(object, &info,
 				      rdf_objfmt_output_section_file))
 	return;
 
@@ -759,9 +814,10 @@ rdf_objfmt_destroy(yasm_objfmt *objfmt)
 }
 
 static rdf_section_data *
-rdf_objfmt_init_new_section(yasm_objfmt_rdf *objfmt_rdf, yasm_section *sect,
+rdf_objfmt_init_new_section(yasm_object *object, yasm_section *sect,
 			    const char *sectname, unsigned long line)
 {
+    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)object->objfmt;
     rdf_section_data *data;
     yasm_symrec *sym;
 
@@ -773,24 +829,22 @@ rdf_objfmt_init_new_section(yasm_objfmt_rdf *objfmt_rdf, yasm_section *sect,
     data->raw_data = NULL;
     yasm_section_add_data(sect, &rdf_section_data_cb, data);
 
-    sym = yasm_symtab_define_label(objfmt_rdf->symtab, sectname,
+    sym = yasm_symtab_define_label(object->symtab, sectname,
 				   yasm_section_bcs_first(sect), 1, line);
     data->sym = sym;
     return data;
 }
 
 static yasm_section *
-rdf_objfmt_add_default_section(yasm_objfmt *objfmt)
+rdf_objfmt_add_default_section(yasm_object *object)
 {
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
     yasm_section *retval;
     rdf_section_data *rsd;
     int isnew;
 
-    retval = yasm_object_get_general(objfmt_rdf->object, ".text", 0, 0, 1, 0,
-				     &isnew, 0);
+    retval = yasm_object_get_general(object, ".text", 0, 0, 1, 0, &isnew, 0);
     if (isnew) {
-	rsd = rdf_objfmt_init_new_section(objfmt_rdf, retval, ".text", 0);
+	rsd = rdf_objfmt_init_new_section(object, retval, ".text", 0);
 	rsd->type = RDF_SECT_CODE;
 	rsd->reserved = 0;
 	yasm_section_set_default(retval, 1);
@@ -798,104 +852,114 @@ rdf_objfmt_add_default_section(yasm_objfmt *objfmt)
     return retval;
 }
 
+static int
+rdf_helper_set_type(void *obj, yasm_valparam *vp, unsigned long line,
+                    void *d, uintptr_t newtype)
+{
+    unsigned int *type = (unsigned int *)d;
+    *type = newtype;
+    return 0;
+}
+
+struct rdf_section_switch_data {
+    /*@only@*/ /*@null@*/ yasm_intnum *reserved_intn;
+    unsigned int type;
+};
+
+static int
+rdf_helper_set_reserved(void *obj, yasm_valparam *vp, unsigned long line,
+                        void *d)
+{
+    struct rdf_section_switch_data *data = (struct rdf_section_switch_data *)d;
+
+    if (!vp->val && vp->type == YASM_PARAM_EXPR)
+        return yasm_dir_helper_intn(obj, vp, line, &data->reserved_intn, 0);
+    else
+        return yasm_dir_helper_valparam_warn(obj, vp, line, d);
+}
+
 static /*@observer@*/ /*@null@*/ yasm_section *
-rdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
-			    /*@unused@*/ /*@null@*/
-			    yasm_valparamhead *objext_valparams,
-			    unsigned long line)
+rdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
+			  /*@unused@*/ /*@null@*/
+			  yasm_valparamhead *objext_valparams,
+			  unsigned long line)
 {
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
     yasm_valparam *vp = yasm_vps_first(valparams);
     yasm_section *retval;
     int isnew;
-    unsigned int type = 0xffff;
     unsigned int reserved = 0;
     int flags_override = 0;
-    char *sectname;
+    const char *sectname;
     rdf_section_data *rsd;
 
-    static const struct {
-	const char *name;
-	unsigned int type;
-    } typenames[] = {
-	{ "bss", RDF_SECT_BSS },
-	{ "code", RDF_SECT_CODE },
-	{ "text", RDF_SECT_CODE },
-	{ "data", RDF_SECT_DATA },
-	{ "comment", RDF_SECT_COMMENT },
-	{ "lcomment", RDF_SECT_LCOMMENT },
-	{ "pcomment", RDF_SECT_PCOMMENT },
-	{ "symdebug", RDF_SECT_SYMDEBUG },
-	{ "linedebug", RDF_SECT_LINEDEBUG },
+    struct rdf_section_switch_data data;
+
+    static const yasm_dir_help help[] = {
+        { "bss", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_BSS },
+        { "code", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_CODE },
+        { "text", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_CODE },
+        { "data", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_DATA },
+        { "comment", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_COMMENT },
+        { "lcomment", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_LCOMMENT },
+        { "pcomment", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_PCOMMENT },
+        { "symdebug", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_SYMDEBUG },
+        { "linedebug", 0, rdf_helper_set_type,
+          offsetof(struct rdf_section_switch_data, type), RDF_SECT_LINEDEBUG },
+        { "reserved", 1, yasm_dir_helper_intn,
+          offsetof(struct rdf_section_switch_data, reserved_intn), 0 }
     };
 
-    if (!vp || vp->param || !vp->val)
-	return NULL;
+    data.reserved_intn = NULL;
+    data.type = 0xffff;
 
-    sectname = vp->val;
+    vp = yasm_vps_first(valparams);
+    sectname = yasm_vp_string(vp);
+    if (!sectname)
+	return NULL;
+    vp = yasm_vps_next(vp);
 
     if (strcmp(sectname, ".text") == 0)
-	type = RDF_SECT_CODE;
+        data.type = RDF_SECT_CODE;
     else if (strcmp(sectname, ".data") == 0)
-	type = RDF_SECT_DATA;
+        data.type = RDF_SECT_DATA;
     else if (strcmp(sectname, ".bss") == 0)
-	type = RDF_SECT_BSS;
+        data.type = RDF_SECT_BSS;
 
-    /* Look for section type */
-    if ((vp = yasm_vps_next(vp))) {
-	size_t i;
-	int match;
-	if (vp->val) {
-	    match = 0;
-	    for (i=0; i<NELEMS(typenames) && !match; i++) {
-		if (yasm__strcasecmp(vp->val, typenames[i].name) == 0) {
-		    type = typenames[i].type;
-		    flags_override = 1;
-		    match = 1;
-		}
-	    }
-	    if (!match)
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized RDF segment type `%s'"),
-			      vp->val);
-	} else
-	    yasm_warn_set(YASM_WARN_GENERAL,
-			  N_("Unrecognized numeric qualifier"));
-    }
+    flags_override = yasm_dir_helper(object, vp, line, help, NELEMS(help),
+                                     &data, rdf_helper_set_reserved);
+    if (flags_override < 0)
+        return NULL;    /* error occurred */
 
-    if (type == 0xffff) {
+    if (data.type == 0xffff) {
 	yasm_error_set(YASM_ERROR_VALUE,
 		       N_("new segment declared without type code"));
-	type = RDF_SECT_DATA;
+        data.type = RDF_SECT_DATA;
     }
 
-    /* Look for reserved value */
-    if (vp && (vp = yasm_vps_next(vp))) {
-        if (!vp->val && vp->param) {
-            /*@dependent@*/ /*@null@*/ const yasm_intnum *reserved_expr;
-
-            reserved_expr = yasm_expr_get_intnum(&vp->param, 0);
-            if (!reserved_expr)
-                yasm_error_set(YASM_ERROR_VALUE,
-		    N_("reserved value must be numeric"));
-            else
-                reserved = yasm_intnum_get_uint(reserved_expr);
-	} else if (vp->val)
-            yasm_warn_set(YASM_WARN_GENERAL, N_("Unrecognized qualifier `%s'"),
-			  vp->val);
+    if (data.reserved_intn) {
+        reserved = yasm_intnum_get_uint(data.reserved_intn);
+        yasm_intnum_destroy(data.reserved_intn);
     }
 
-    retval = yasm_object_get_general(objfmt_rdf->object, sectname, 0, 0, 1,
-				     type == RDF_SECT_BSS, &isnew, line);
+    retval = yasm_object_get_general(object, sectname, 0, 0, 1,
+                                     data.type == RDF_SECT_BSS, &isnew, line);
 
     if (isnew)
-	rsd = rdf_objfmt_init_new_section(objfmt_rdf, retval, sectname, line);
+	rsd = rdf_objfmt_init_new_section(object, retval, sectname, line);
     else
 	rsd = yasm_section_get_data(retval, &rdf_section_data_cb);
 
     if (isnew || yasm_section_is_default(retval)) {
 	yasm_section_set_default(retval, 0);
-	rsd->type = type;
+        rsd->type = data.type;
 	rsd->reserved = reserved;
     } else if (flags_override)
 	yasm_warn_set(YASM_WARN_GENERAL,
@@ -925,166 +989,9 @@ rdf_section_data_print(void *data, FILE *f, int indent_level)
     fprintf(f, "%*ssize=%ld\n", indent_level, "", rsd->size);
 }
 
-static yasm_symrec *
-rdf_objfmt_extern_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			   /*@null@*/ yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
-    yasm_symrec *sym;
-    unsigned int flags = 0;
-
-    static const struct {
-	const char *name;
-	unsigned int flags;
-    } flagnames[] = {
-	{ "data", SYM_DATA },
-	{ "object", SYM_DATA },
-	{ "proc", SYM_FUNCTION },
-	{ "function", SYM_FUNCTION },
-	{ "import", SYM_IMPORT },
-	{ "far", SYM_FAR },
-    };
-
-    sym = yasm_symtab_declare(objfmt_rdf->symtab, name, YASM_SYM_EXTERN, line);
-
-    if (objext_valparams) {
-	yasm_valparam *vp = yasm_vps_first(objext_valparams);
-        for (; vp; vp = yasm_vps_next(vp)) {
-	    size_t i;
-	    int match;
-
-	    if (!vp->val) {
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized numeric qualifier"));
-		continue;
-	    }
-
-	    match = 0;
-	    for (i=0; i<NELEMS(flagnames) && !match; i++) {
-		if (yasm__strcasecmp(vp->val, flagnames[i].name) == 0) {
-		    flags |= flagnames[i].flags;
-		    match = 1;
-		}
-	    }
-
-	    if (yasm__strcasecmp(vp->val, "near") == 0) {
-		flags &= ~SYM_FAR;
-		match = 1;
-	    }
-
-	    if (!match)
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized qualifier `%s'"), vp->val);
-	}
-    }
-
-    /* Remember flags */
-    rdf_objfmt_sym_set_data(sym, NULL, 0, flags);
-    return sym;
-}
-
-static yasm_symrec *
-rdf_objfmt_global_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			   /*@null@*/ yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
-    yasm_symrec *sym;
-    unsigned int flags = 0;
-
-    static const struct {
-	const char *name;
-	unsigned int flags;
-    } flagnames[] = {
-	{ "data", SYM_DATA },
-	{ "object", SYM_DATA },
-	{ "proc", SYM_FUNCTION },
-	{ "function", SYM_FUNCTION },
-	{ "export", SYM_GLOBAL },
-    };
-
-    sym = yasm_symtab_declare(objfmt_rdf->symtab, name, YASM_SYM_GLOBAL, line);
-
-    if (objext_valparams) {
-	yasm_valparam *vp = yasm_vps_first(objext_valparams);
-        for (; vp; vp = yasm_vps_next(vp)) {
-	    size_t i;
-	    int match;
-
-	    if (!vp->val) {
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized numeric qualifier"));
-		continue;
-	    }
-
-	    match = 0;
-	    for (i=0; i<NELEMS(flagnames) && !match; i++) {
-		if (yasm__strcasecmp(vp->val, flagnames[i].name) == 0) {
-		    flags |= flagnames[i].flags;
-		    match = 1;
-		}
-	    }
-	    if (!match)
-		yasm_warn_set(YASM_WARN_GENERAL,
-			      N_("Unrecognized qualifier `%s'"), vp->val);
-	}
-    }
-
-    /* Remember flags */
-    rdf_objfmt_sym_set_data(sym, NULL, 0, flags);
-    return sym;
-}
-
-static yasm_symrec *
-rdf_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
-			   /*@only@*/ yasm_expr *size,
-			   yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
-    yasm_symrec *sym;
-    unsigned long addralign = 0;
-
-    sym = yasm_symtab_declare(objfmt_rdf->symtab, name, YASM_SYM_COMMON, line);
-
-    if (objext_valparams) {
-	yasm_valparam *vp = yasm_vps_first(objext_valparams);
-        for (; vp; vp = yasm_vps_next(vp)) {
-            if (!vp->val && vp->param) {
-                /*@dependent@*/ /*@null@*/ const yasm_intnum *align_expr;
-
-                align_expr = yasm_expr_get_intnum(&vp->param, 0);
-                if (!align_expr) {
-                    yasm_error_set(YASM_ERROR_VALUE,
-			N_("alignment constraint is not an integer"));
-                    return sym;
-                }
-                addralign = yasm_intnum_get_uint(align_expr);
-
-                /* Alignments must be a power of two. */
-                if (!is_exp2(addralign)) {
-                    yasm_error_set(YASM_ERROR_VALUE,
-                        N_("alignment constraint is not a power of two"));
-                    return sym;
-                }
-            } else if (vp->val)
-                yasm_warn_set(YASM_WARN_GENERAL,
-                              N_("Unrecognized qualifier `%s'"), vp->val);
-        }
-    }
-
-    /* Remember size and alignment */
-    rdf_objfmt_sym_set_data(sym, size, addralign, 0);
-    return sym;
-}
-
 static void
 rdf_symrec_data_destroy(void *data)
 {
-    rdf_symrec_data *rsymd = (rdf_symrec_data *)data;
-    if (rsymd->size)
-	yasm_expr_destroy(rsymd->size);
     yasm_xfree(data);
 }
 
@@ -1094,47 +1001,17 @@ rdf_symrec_data_print(void *data, FILE *f, int indent_level)
     rdf_symrec_data *rsymd = (rdf_symrec_data *)data;
 
     fprintf(f, "%*ssymtab segment=%u\n", indent_level, "", rsymd->segment);
-    fprintf(f, "%*ssize=", indent_level, "");
-    if (rsymd->size)
-	yasm_expr_print(rsymd->size, f);
-    else
-	fprintf(f, "nil");
-    fprintf(f, "%*salign=%lu\n", indent_level, "", rsymd->align);
 }
 
-static int
-rdf_objfmt_directive(yasm_objfmt *objfmt, const char *name,
-		     /*@null@*/ yasm_valparamhead *valparams,
-		     /*@unused@*/ /*@null@*/
-		     yasm_valparamhead *objext_valparams, unsigned long line)
+static void
+rdf_objfmt_add_libmodule(yasm_object *object, char *name, int lib)
 {
-    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)objfmt;
-    int lib;
-    yasm_valparam *vp;
+    yasm_objfmt_rdf *objfmt_rdf = (yasm_objfmt_rdf *)object->objfmt;
     xdf_str *str;
 
-    if (yasm__strcasecmp(name, "library") == 0)
-	lib = 1;
-    else if (yasm__strcasecmp(name, "module") == 0)
-	lib = 0;
-    else
-	return 1;
-
-    if (!valparams) {
-	yasm_error_set(YASM_ERROR_SYNTAX, N_("[%s] requires an argument"),
-		       name);
-	return 0;
-    }
-    vp = yasm_vps_first(valparams);
-    if (!vp->val) {
-	yasm_error_set(YASM_ERROR_SYNTAX, N_("argument to [%s] must be name"),
-		       name);
-	return 0;
-    }
-
     /* Add to list */
     str = yasm_xmalloc(sizeof(xdf_str));
-    str->str = vp->val;
+    str->str = name;
     if (lib)
 	STAILQ_INSERT_TAIL(&objfmt_rdf->library_names, str, link);
     else
@@ -1146,11 +1023,23 @@ rdf_objfmt_directive(yasm_objfmt *objfmt, const char *name,
 		      MODLIB_NAME_MAX);
 	str->str[MODLIB_NAME_MAX-1] = '\0';
     }
-
-    vp->val = NULL;	/* don't free it */
-    return 0;
 }
 
+static void
+dir_library(yasm_object *object, yasm_valparamhead *valparams,
+	    yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_valparam *vp = yasm_vps_first(valparams);
+    rdf_objfmt_add_libmodule(object, yasm__xstrdup(yasm_vp_string(vp)), 1);
+}
+
+static void
+dir_module(yasm_object *object, yasm_valparamhead *valparams,
+	   yasm_valparamhead *objext_valparams, unsigned long line)
+{
+    yasm_valparam *vp = yasm_vps_first(valparams);
+    rdf_objfmt_add_libmodule(object, yasm__xstrdup(yasm_vp_string(vp)), 0);
+}
 
 /* Define valid debug formats to use with this object format */
 static const char *rdf_objfmt_dbgfmt_keywords[] = {
@@ -1158,6 +1047,12 @@ static const char *rdf_objfmt_dbgfmt_keywords[] = {
     NULL
 };
 
+static const yasm_directive rdf_objfmt_directives[] = {
+    { "library",        "nasm", dir_library,    YASM_DIR_ARG_REQUIRED },
+    { "module",         "nasm", dir_module,     YASM_DIR_ARG_REQUIRED },
+    { NULL, NULL, NULL, 0 }
+};
+
 /* Define objfmt structure -- see objfmt.h for details */
 yasm_objfmt_module yasm_rdf_LTX_objfmt = {
     "Relocatable Dynamic Object File Format (RDOFF) v2.0",
@@ -1166,13 +1061,10 @@ yasm_objfmt_module yasm_rdf_LTX_objfmt = {
     32,
     rdf_objfmt_dbgfmt_keywords,
     "null",
+    rdf_objfmt_directives,
     rdf_objfmt_create,
     rdf_objfmt_output,
     rdf_objfmt_destroy,
     rdf_objfmt_add_default_section,
-    rdf_objfmt_section_switch,
-    rdf_objfmt_extern_declare,
-    rdf_objfmt_global_declare,
-    rdf_objfmt_common_declare,
-    rdf_objfmt_directive
+    rdf_objfmt_section_switch
 };
