@@ -1,7 +1,7 @@
 /*
  * Extended Dynamic Object format
  *
- *  Copyright (C) 2004  Peter Johnson
+ *  Copyright (C) 2004-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: xdf-objfmt.c 1795 2007-02-24 22:39:55Z peter $");
+/*@unused@*/ RCSID("$Id: xdf-objfmt.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_BC_INTERNAL
@@ -87,13 +87,10 @@ typedef struct yasm_objfmt_xdf {
     yasm_objfmt_base objfmt;		    /* base structure */
 
     long parse_scnum;		    /* sect numbering in parser */
-
-    yasm_object *object;
-    yasm_symtab *symtab;
-    /*@dependent@*/ yasm_arch *arch;
 } yasm_objfmt_xdf;
 
 typedef struct xdf_objfmt_output_info {
+    yasm_object *object;
     yasm_objfmt_xdf *objfmt_xdf;
     yasm_errwarns *errwarns;
     /*@dependent@*/ FILE *f;
@@ -126,23 +123,19 @@ yasm_objfmt_module yasm_xdf_LTX_objfmt;
 
 
 static yasm_objfmt *
-xdf_objfmt_create(yasm_object *object, yasm_arch *a)
+xdf_objfmt_create(yasm_object *object)
 {
     yasm_objfmt_xdf *objfmt_xdf = yasm_xmalloc(sizeof(yasm_objfmt_xdf));
 
-    objfmt_xdf->object = object;
-    objfmt_xdf->symtab = yasm_object_get_symtab(object);
-    objfmt_xdf->arch = a;
-
     /* Only support x86 arch */
-    if (yasm__strcasecmp(yasm_arch_keyword(a), "x86") != 0) {
+    if (yasm__strcasecmp(yasm_arch_keyword(object->arch), "x86") != 0) {
 	yasm_xfree(objfmt_xdf);
 	return NULL;
     }
 
     /* Support x86 and amd64 machines of x86 arch */
-    if (yasm__strcasecmp(yasm_arch_get_machine(a), "x86") &&
-	yasm__strcasecmp(yasm_arch_get_machine(a), "amd64")) {
+    if (yasm__strcasecmp(yasm_arch_get_machine(object->arch), "x86") &&
+	yasm__strcasecmp(yasm_arch_get_machine(object->arch), "amd64")) {
 	yasm_xfree(objfmt_xdf);
 	return NULL;
     }
@@ -177,7 +170,7 @@ xdf_objfmt_output_value(yasm_value *value, unsigned char *buf,
      * cross-section, or non-PC-relative reference (those are handled below).
      */
     switch (yasm_value_output_basic(value, buf, destsize, bc, warn,
-				    info->objfmt_xdf->arch)) {
+				    info->object->arch)) {
 	case -1:
 	    return 1;
 	case 0:
@@ -237,7 +230,7 @@ xdf_objfmt_output_value(yasm_value *value, unsigned char *buf,
 	yasm_intnum_calc(intn, YASM_EXPR_ADD, intn2);
     }
 
-    retval = yasm_arch_intnum_tobytes(objfmt_xdf->arch, intn, buf, destsize,
+    retval = yasm_arch_intnum_tobytes(info->object->arch, intn, buf, destsize,
 				      valsize, 0, bc, warn);
     yasm_intnum_destroy(intn);
     return retval;
@@ -298,10 +291,6 @@ xdf_objfmt_output_section(yasm_section *sect, /*@null@*/ void *d)
     long pos;
     xdf_reloc *reloc;
 
-    /* FIXME: Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     xsd = yasm_section_get_data(sect, &xdf_section_data_cb);
     assert(xsd != NULL);
@@ -395,10 +384,6 @@ xdf_objfmt_output_secthead(yasm_section *sect, /*@null@*/ void *d)
     /*@null@*/ xdf_symrec_data *xsymd;
     unsigned char *localbuf;
 
-    /* Don't output absolute sections into the section table */
-    if (yasm_section_is_absolute(sect))
-	return 0;
-
     assert(info != NULL);
     objfmt_xdf = info->objfmt_xdf;
     xsd = yasm_section_get_data(sect, &xdf_section_data_cb);
@@ -441,8 +426,16 @@ static int
 xdf_objfmt_count_sym(yasm_symrec *sym, /*@null@*/ void *d)
 {
     /*@null@*/ xdf_objfmt_output_info *info = (xdf_objfmt_output_info *)d;
+    yasm_sym_vis vis = yasm_symrec_get_visibility(sym);
     assert(info != NULL);
-    if (info->all_syms || yasm_symrec_get_visibility(sym) != YASM_SYM_LOCAL) {
+    if (vis & YASM_SYM_COMMON) {
+	yasm_error_set(YASM_ERROR_GENERAL,
+	    N_("XDF object format does not support common variables"));
+	yasm_errwarn_propagate(info->errwarns, yasm_symrec_get_decl_line(sym));
+	return 0;
+    }
+    if (info->all_syms ||
+	(vis != YASM_SYM_LOCAL && !(vis & YASM_SYM_DLOCAL))) {
 	/* Save index in symrec data */
 	xdf_symrec_data *sym_data = yasm_xmalloc(sizeof(xdf_symrec_data));
 	sym_data->index = info->indx;
@@ -488,24 +481,9 @@ xdf_objfmt_output_sym(yasm_symrec *sym, /*@null@*/ void *d)
 	    if (sect) {
 		/*@dependent@*/ /*@null@*/ xdf_section_data *csectd;
 		csectd = yasm_section_get_data(sect, &xdf_section_data_cb);
-		if (csectd) {
+		if (csectd)
 		    scnum = csectd->scnum;
-		} else if (yasm_section_is_absolute(sect)) {
-		    yasm_expr *abs_start;
-
-		    abs_start = yasm_expr_copy(yasm_section_get_start(sect));
-		    intn = yasm_expr_get_intnum(&abs_start, 1);
-		    if (!intn) {
-			yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			    N_("absolute section start not an integer expression"));
-			yasm_errwarn_propagate(info->errwarns, abs_start->line);
-		    } else
-			value = yasm_intnum_get_uint(intn);
-		    yasm_expr_destroy(abs_start);
-
-		    flags |= XDF_SYM_EQU;
-		    scnum = -2;	/* -2 = absolute symbol */
-		} else
+		else
 		    yasm_internal_error(N_("didn't understand section"));
 		if (precbc)
 		    value += yasm_bc_next_offset(precbc);
@@ -560,14 +538,15 @@ xdf_objfmt_output_str(yasm_symrec *sym, /*@null@*/ void *d)
 }
 
 static void
-xdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
-		  /*@unused@*/ yasm_dbgfmt *df, yasm_errwarns *errwarns)
+xdf_objfmt_output(yasm_object *object, FILE *f, int all_syms,
+		  yasm_errwarns *errwarns)
 {
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
+    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)object->objfmt;
     xdf_objfmt_output_info info;
     unsigned char *localbuf;
     unsigned long symtab_count = 0;
 
+    info.object = object;
     info.objfmt_xdf = objfmt_xdf;
     info.errwarns = errwarns;
     info.f = f;
@@ -583,20 +562,20 @@ xdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     /* Get number of symbols */
     info.indx = 0;
     info.all_syms = 1;	/* force all syms into symbol table */
-    yasm_symtab_traverse(objfmt_xdf->symtab, &info, xdf_objfmt_count_sym);
+    yasm_symtab_traverse(object->symtab, &info, xdf_objfmt_count_sym);
     symtab_count = info.indx;
 
     /* Get file offset of start of string table */
     info.strtab_offset = 16+40*(objfmt_xdf->parse_scnum)+16*symtab_count;
 
     /* Output symbol table */
-    yasm_symtab_traverse(objfmt_xdf->symtab, &info, xdf_objfmt_output_sym);
+    yasm_symtab_traverse(object->symtab, &info, xdf_objfmt_output_sym);
 
     /* Output string table */
-    yasm_symtab_traverse(objfmt_xdf->symtab, &info, xdf_objfmt_output_str);
+    yasm_symtab_traverse(object->symtab, &info, xdf_objfmt_output_str);
 
     /* Section data/relocs */
-    if (yasm_object_sections_traverse(objfmt_xdf->object, &info,
+    if (yasm_object_sections_traverse(object, &info,
 				      xdf_objfmt_output_section))
 	return;
 
@@ -615,8 +594,7 @@ xdf_objfmt_output(yasm_objfmt *objfmt, FILE *f, int all_syms,
     YASM_WRITE_32_L(localbuf, info.strtab_offset-16);
     fwrite(info.buf, 16, 1, f);
 
-    yasm_object_sections_traverse(objfmt_xdf->object, &info,
-				  xdf_objfmt_output_secthead);
+    yasm_object_sections_traverse(object, &info, xdf_objfmt_output_secthead);
 
     yasm_xfree(info.buf);
 }
@@ -628,9 +606,10 @@ xdf_objfmt_destroy(yasm_objfmt *objfmt)
 }
 
 static xdf_section_data *
-xdf_objfmt_init_new_section(yasm_objfmt_xdf *objfmt_xdf, yasm_section *sect,
+xdf_objfmt_init_new_section(yasm_object *object, yasm_section *sect,
 			    const char *sectname, unsigned long line)
 {
+    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)object->objfmt;
     xdf_section_data *data;
     yasm_symrec *sym;
 
@@ -645,111 +624,115 @@ xdf_objfmt_init_new_section(yasm_objfmt_xdf *objfmt_xdf, yasm_section *sect,
     data->nreloc = 0;
     yasm_section_add_data(sect, &xdf_section_data_cb, data);
 
-    sym = yasm_symtab_define_label(objfmt_xdf->symtab, sectname,
+    sym = yasm_symtab_define_label(object->symtab, sectname,
 				   yasm_section_bcs_first(sect), 1, line);
     data->sym = sym;
     return data;
 }
 
 static yasm_section *
-xdf_objfmt_add_default_section(yasm_objfmt *objfmt)
+xdf_objfmt_add_default_section(yasm_object *object)
 {
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
     yasm_section *retval;
     xdf_section_data *xsd;
     int isnew;
 
-    retval = yasm_object_get_general(objfmt_xdf->object, ".text", 0, 0, 1, 0,
-				     &isnew, 0);
+    retval = yasm_object_get_general(object, ".text", 0, 0, 1, 0, &isnew, 0);
     if (isnew) {
-	xsd = xdf_objfmt_init_new_section(objfmt_xdf, retval, ".text", 0);
+	xsd = xdf_objfmt_init_new_section(object, retval, ".text", 0);
 	yasm_section_set_default(retval, 1);
     }
     return retval;
 }
 
-static /*@observer@*/ /*@null@*/ yasm_section *
-xdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
-			    /*@unused@*/ /*@null@*/
-			    yasm_valparamhead *objext_valparams,
-			    unsigned long line)
+static int
+xdf_helper_use(void *obj, yasm_valparam *vp, unsigned long line, void *d,
+               uintptr_t bits)
 {
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
-    yasm_valparam *vp = yasm_vps_first(valparams);
+    yasm_object *object = (yasm_object *)obj;
+    unsigned long *flags = (unsigned long *)d;
+    *flags &= ~(XDF_SECT_USE_16|XDF_SECT_USE_32|XDF_SECT_USE_64);
+    switch (bits) {
+        case 16: *flags |= XDF_SECT_USE_16; break;
+        case 32: *flags |= XDF_SECT_USE_32; break;
+        case 64: *flags |= XDF_SECT_USE_64; break;
+    };
+    yasm_arch_set_var(object->arch, "mode_bits", bits);
+    return 0;
+}
+
+static /*@observer@*/ /*@null@*/ yasm_section *
+xdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
+			  /*@unused@*/ /*@null@*/
+			  yasm_valparamhead *objext_valparams,
+			  unsigned long line)
+{
+    yasm_valparam *vp;
     yasm_section *retval;
     int isnew;
-    /*@dependent@*/ /*@null@*/ const yasm_intnum *absaddr = NULL;
-    /*@dependent@*/ /*@null@*/ const yasm_intnum *vaddr = NULL;
-    unsigned long align = 0;
-    unsigned long flags = 0;
     int flags_override = 0;
-    char *sectname;
+    const char *sectname;
     int resonly = 0;
     xdf_section_data *xsd;
+    unsigned long align = 0;
 
-    if (!vp || vp->param || !vp->val)
-	return NULL;
+    struct xdf_section_switch_data {
+        /*@only@*/ /*@null@*/ yasm_intnum *absaddr;
+        /*@only@*/ /*@null@*/ yasm_intnum *vaddr;
+        /*@only@*/ /*@null@*/ yasm_intnum *align_intn;
+        unsigned long flags;
+    } data;
 
-    sectname = vp->val;
+    static const yasm_dir_help help[] = {
+        { "use16", 0, xdf_helper_use,
+          offsetof(struct xdf_section_switch_data, flags), 16 },
+        { "use32", 0, xdf_helper_use,
+          offsetof(struct xdf_section_switch_data, flags), 32 },
+        { "use64", 0, xdf_helper_use,
+          offsetof(struct xdf_section_switch_data, flags), 64 },
+        { "bss", 0, yasm_dir_helper_flag_or,
+          offsetof(struct xdf_section_switch_data, flags), XDF_SECT_BSS },
+        { "flat", 0, yasm_dir_helper_flag_or,
+          offsetof(struct xdf_section_switch_data, flags), XDF_SECT_FLAT },
+        { "absolute", 1, yasm_dir_helper_intn,
+          offsetof(struct xdf_section_switch_data, absaddr), 0 },
+        { "virtual", 1, yasm_dir_helper_intn,
+          offsetof(struct xdf_section_switch_data, vaddr), 0 },
+        { "align", 1, yasm_dir_helper_intn,
+          offsetof(struct xdf_section_switch_data, align_intn), 0 }
+    };
 
-    while ((vp = yasm_vps_next(vp))) {
-	if (!vp->val) {
-	    yasm_warn_set(YASM_WARN_GENERAL,
-			  N_("Unrecognized numeric qualifier"));
-	    continue;
-	}
+    data.absaddr = NULL;
+    data.vaddr = NULL;
+    data.align_intn = NULL;
+    data.flags = 0;
 
-	flags_override = 1;
-	if (yasm__strcasecmp(vp->val, "use16") == 0) {
-	    flags &= ~(XDF_SECT_USE_32|XDF_SECT_USE_64);
-	    flags |= XDF_SECT_USE_16;
-	    yasm_arch_set_var(objfmt_xdf->arch, "mode_bits", 16);
-	} else if (yasm__strcasecmp(vp->val, "use32") == 0) {
-	    flags &= ~(XDF_SECT_USE_16|XDF_SECT_USE_64);
-	    flags |= XDF_SECT_USE_32;
-	    yasm_arch_set_var(objfmt_xdf->arch, "mode_bits", 32);
-	} else if (yasm__strcasecmp(vp->val, "use64") == 0) {
-	    flags &= ~(XDF_SECT_USE_16|XDF_SECT_USE_32);
-	    flags |= XDF_SECT_USE_64;
-	    yasm_arch_set_var(objfmt_xdf->arch, "mode_bits", 64);
-	} else if (yasm__strcasecmp(vp->val, "bss") == 0) {
-	    flags |= XDF_SECT_BSS;
-	} else if (yasm__strcasecmp(vp->val, "flat") == 0) {
-	    flags |= XDF_SECT_FLAT;
-	} else if (yasm__strcasecmp(vp->val, "absolute") == 0 && vp->param) {
-	    flags |= XDF_SECT_ABSOLUTE;
-	    absaddr = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!absaddr) {
-		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			       N_("argument to `%s' is not an integer"),
-			       vp->val);
+    vp = yasm_vps_first(valparams);
+    sectname = yasm_vp_string(vp);
+    if (!sectname)
 		return NULL;
-	    }
-	} else if (yasm__strcasecmp(vp->val, "virtual") == 0 && vp->param) {
-	    vaddr = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!vaddr) {
-		yasm_error_set(YASM_ERROR_NOT_CONSTANT,
-			       N_("argument to `%s' is not an integer"),
-			       vp->val);
-		return NULL;
-	    }
-	} else if (yasm__strcasecmp(vp->val, "align") == 0 && vp->param) {
-	    /*@dependent@*/ /*@null@*/ const yasm_intnum *align_expr;
+    vp = yasm_vps_next(vp);
 
-	    align_expr = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!align_expr) {
-		yasm_error_set(YASM_ERROR_VALUE,
-			       N_("argument to `%s' is not an integer"),
-			       vp->val);
-		return NULL;
-	    }
-	    align = yasm_intnum_get_uint(align_expr);
+    flags_override = yasm_dir_helper(object, vp, line, help, NELEMS(help),
+                                     &data, yasm_dir_helper_valparam_warn);
+    if (flags_override < 0)
+        return NULL;    /* error occurred */
+
+    if (data.absaddr)
+        data.flags |= XDF_SECT_ABSOLUTE;
+    if (data.align_intn) {
+        align = yasm_intnum_get_uint(data.align_intn);
+        yasm_intnum_destroy(data.align_intn);
 
             /* Alignments must be a power of two. */
             if (!is_exp2(align)) {
 		yasm_error_set(YASM_ERROR_VALUE,
 			       N_("argument to `%s' is not a power of two"),
-			       vp->val);
+                           "align");
+            if (data.vaddr)
+                yasm_intnum_destroy(data.vaddr);
+            if (data.absaddr)
+                yasm_intnum_destroy(data.absaddr);
 		return NULL;
 	    }
 
@@ -757,33 +740,34 @@ xdf_objfmt_section_switch(yasm_objfmt *objfmt, yasm_valparamhead *valparams,
 	    if (align > 4096) {
 		yasm_error_set(YASM_ERROR_VALUE,
 			       N_("XDF does not support alignments > 4096"));
+            if (data.vaddr)
+                yasm_intnum_destroy(data.vaddr);
+            if (data.absaddr)
+                yasm_intnum_destroy(data.absaddr);
 		return NULL;
 	    }
-	} else
-	    yasm_warn_set(YASM_WARN_GENERAL, N_("Unrecognized qualifier `%s'"),
-			  vp->val);
     }
 
-    retval = yasm_object_get_general(objfmt_xdf->object, sectname, 0, align, 1,
-				     resonly, &isnew, line);
+    retval = yasm_object_get_general(object, sectname, 0, align, 1, resonly,
+				     &isnew, line);
 
     if (isnew)
-	xsd = xdf_objfmt_init_new_section(objfmt_xdf, retval, sectname, line);
+	xsd = xdf_objfmt_init_new_section(object, retval, sectname, line);
     else
 	xsd = yasm_section_get_data(retval, &xdf_section_data_cb);
 
     if (isnew || yasm_section_is_default(retval)) {
 	yasm_section_set_default(retval, 0);
-	xsd->flags = flags;
-	if (absaddr) {
+        xsd->flags = data.flags;
+        if (data.absaddr) {
 	    if (xsd->addr)
 		yasm_intnum_destroy(xsd->addr);
-	    xsd->addr = yasm_intnum_copy(absaddr);
+            xsd->addr = data.absaddr;
 	}
-	if (vaddr) {
+        if (data.vaddr) {
 	    if (xsd->vaddr)
 		yasm_intnum_destroy(xsd->vaddr);
-	    xsd->vaddr = yasm_intnum_copy(vaddr);
+            xsd->vaddr = data.vaddr;
 	}
 	yasm_section_set_align(retval, align, line);
     } else if (flags_override)
@@ -822,44 +806,6 @@ xdf_section_data_print(void *data, FILE *f, int indent_level)
     fprintf(f, "%*snreloc=%ld\n", indent_level, "", xsd->nreloc);
 }
 
-static yasm_symrec *
-xdf_objfmt_extern_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			   /*@null@*/ yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
-
-    return yasm_symtab_declare(objfmt_xdf->symtab, name, YASM_SYM_EXTERN,
-			       line);
-}
-
-static yasm_symrec *
-xdf_objfmt_global_declare(yasm_objfmt *objfmt, const char *name, /*@unused@*/
-			   /*@null@*/ yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
-
-    return yasm_symtab_declare(objfmt_xdf->symtab, name, YASM_SYM_GLOBAL,
-			       line);
-}
-
-static yasm_symrec *
-xdf_objfmt_common_declare(yasm_objfmt *objfmt, const char *name,
-			   /*@only@*/ yasm_expr *size, /*@unused@*/ /*@null@*/
-			   yasm_valparamhead *objext_valparams,
-			   unsigned long line)
-{
-    yasm_objfmt_xdf *objfmt_xdf = (yasm_objfmt_xdf *)objfmt;
-
-    yasm_expr_destroy(size);
-    yasm_error_set(YASM_ERROR_GENERAL,
-	N_("XDF object format does not support common variables"));
-
-    return yasm_symtab_declare(objfmt_xdf->symtab, name, YASM_SYM_COMMON,
-			       line);
-}
-
 static void
 xdf_symrec_data_destroy(void *data)
 {
@@ -874,18 +820,6 @@ xdf_symrec_data_print(void *data, FILE *f, int indent_level)
     fprintf(f, "%*ssymtab index=%lu\n", indent_level, "", xsd->index);
 }
 
-static int
-xdf_objfmt_directive(/*@unused@*/ yasm_objfmt *objfmt,
-		      /*@unused@*/ const char *name,
-		      /*@unused@*/ /*@null@*/ yasm_valparamhead *valparams,
-		      /*@unused@*/ /*@null@*/
-		      yasm_valparamhead *objext_valparams,
-		      /*@unused@*/ unsigned long line)
-{
-    return 1;	/* no objfmt directives */
-}
-
-
 /* Define valid debug formats to use with this object format */
 static const char *xdf_objfmt_dbgfmt_keywords[] = {
     "null",
@@ -900,13 +834,10 @@ yasm_objfmt_module yasm_xdf_LTX_objfmt = {
     32,
     xdf_objfmt_dbgfmt_keywords,
     "null",
+    NULL,	/* no directives */
     xdf_objfmt_create,
     xdf_objfmt_output,
     xdf_objfmt_destroy,
     xdf_objfmt_add_default_section,
-    xdf_objfmt_section_switch,
-    xdf_objfmt_extern_declare,
-    xdf_objfmt_global_declare,
-    xdf_objfmt_common_declare,
-    xdf_objfmt_directive
+    xdf_objfmt_section_switch
 };
