@@ -1,7 +1,7 @@
 /*
  * GAS-compatible parser
  *
- *  Copyright (C) 2005  Peter Johnson
+ *  Copyright (C) 2005-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-RCSID("$Id: gas-parse.c 1795 2007-02-24 22:39:55Z peter $");
+RCSID("$Id: gas-parse.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_EXPR_INTERNAL
@@ -42,8 +42,6 @@ RCSID("$Id: gas-parse.c 1795 2007-02-24 22:39:55Z peter $");
 static yasm_bytecode *parse_line(yasm_parser_gas *parser_gas);
 static yasm_bytecode *parse_instr(yasm_parser_gas *parser_gas);
 static int parse_dirvals(yasm_parser_gas *parser_gas, yasm_valparamhead *vps);
-static int parse_dirstrvals(yasm_parser_gas *parser_gas,
-			    yasm_valparamhead *vps);
 static int parse_datavals(yasm_parser_gas *parser_gas, yasm_datavalhead *dvs);
 static int parse_strvals(yasm_parser_gas *parser_gas, yasm_datavalhead *dvs);
 static yasm_effaddr *parse_memaddr(yasm_parser_gas *parser_gas);
@@ -72,10 +70,12 @@ static yasm_bytecode *gas_parser_dir_fill
     (yasm_parser_gas *parser_gas, /*@only@*/ yasm_expr *repeat,
      /*@only@*/ /*@null@*/ yasm_expr *size,
      /*@only@*/ /*@null@*/ yasm_expr *value);
+#if 0
 static void gas_parser_directive
     (yasm_parser_gas *parser_gas, const char *name,
      yasm_valparamhead *valparams,
      /*@null@*/ yasm_valparamhead *objext_valparams);
+#endif
 
 #define is_eol_tok(tok)	((tok) == '\n' || (tok) == ';' || (tok) == 0)
 #define is_eol()	is_eol_tok(curtok)
@@ -143,60 +143,53 @@ demand_eol_(yasm_parser_gas *parser_gas)
 static int
 expect_(yasm_parser_gas *parser_gas, int token)
 {
-    static char strch[] = "expected ` '";
+    static char strch[] = "` '";
     const char *str;
 
     if (curtok == token)
 	return 1;
 
     switch (token) {
-	case INTNUM:		str = "expected integer"; break;
-	case FLTNUM:		str = "expected floating point value"; break;
-	case STRING:		str = "expected string"; break;
-	case INSN:		str = "expected instruction"; break;
-	case PREFIX:		str = "expected instruction prefix"; break;
-	case REG:		str = "expected register"; break;
-	case REGGROUP:		str = "expected register group"; break;
-	case SEGREG:		str = "expected segment register"; break;
-	case TARGETMOD:		str = "expected target modifier"; break;
-	case LEFT_OP:		str = "expected <<"; break;
-	case RIGHT_OP:		str = "expected >>"; break;
-	case ID:		str = "expected identifier"; break;
-	case LABEL:		str = "expected label"; break;
+	case INTNUM:		str = "integer"; break;
+	case FLTNUM:		str = "floating point value"; break;
+	case STRING:		str = "string"; break;
+	case INSN:		str = "instruction"; break;
+	case PREFIX:		str = "instruction prefix"; break;
+	case REG:		str = "register"; break;
+	case REGGROUP:		str = "register group"; break;
+	case SEGREG:		str = "segment register"; break;
+	case TARGETMOD:		str = "target modifier"; break;
+	case LEFT_OP:		str = "<<"; break;
+	case RIGHT_OP:		str = ">>"; break;
+	case ID:		str = "identifier"; break;
+	case LABEL:		str = "label"; break;
 	case LINE:
 	case DIR_ALIGN:
 	case DIR_ASCII:
 	case DIR_COMM:
 	case DIR_DATA:
 	case DIR_ENDR:
-	case DIR_EXTERN:
 	case DIR_EQU:
 	case DIR_FILE:
 	case DIR_FILL:
-	case DIR_GLOBAL:
-	case DIR_IDENT:
 	case DIR_LEB128:
 	case DIR_LINE:
-	case DIR_LOC:
 	case DIR_LOCAL:
 	case DIR_LCOMM:
 	case DIR_ORG:
 	case DIR_REPT:
 	case DIR_SECTION:
 	case DIR_SECTNAME:
-	case DIR_SIZE:
 	case DIR_SKIP:
-	case DIR_TYPE:
-	case DIR_WEAK:
 	case DIR_ZERO:
-	    str = "expected directive";
+	    str = "directive";
 	    break;
 	default:
-	    strch[10] = token;
+	    strch[1] = token;
 	    str = strch;
 	    break;
     }
-    yasm_error_set(YASM_ERROR_PARSE, str);
+    yasm_error_set(YASM_ERROR_PARSE, "expected %s", str);
     destroy_curtok();
     return 0;
 }
@@ -215,7 +208,7 @@ gas_parser_parse(yasm_parser_gas *parser_gas)
 
 	yasm_errwarn_propagate(parser_gas->errwarns, cur_line);
 
-	temp_bc = yasm_section_bcs_append(parser_gas->cur_section, bc);
+	temp_bc = yasm_section_bcs_append(cursect, bc);
 	if (temp_bc)
 	    parser_gas->prev_bc = temp_bc;
 	if (curtok == ';')
@@ -251,14 +244,18 @@ parse_line(yasm_parser_gas *parser_gas)
     switch (curtok) {
 	case ID:
 	    id = ID_val;
+	    parser_gas->state = INSTDIR;
 	    get_next_token(); /* ID */
 	    if (curtok == ':') {
 		/* Label */
+		parser_gas->state = INITIAL;
 		get_next_token(); /* : */
 		define_label(parser_gas, id, 0);
 		return parse_line(parser_gas);
 	    } else if (curtok == '=') {
 		/* EQU */
+		/* TODO: allow redefinition, assigning to . (same as .org) */
+		parser_gas->state = INITIAL;
 		get_next_token(); /* = */
 		e = parse_expr(parser_gas);
 		if (e)
@@ -269,7 +266,16 @@ parse_line(yasm_parser_gas *parser_gas)
 		yasm_xfree(id);
 		return NULL;
 	    }
-	    /* must be an error at this point */
+
+	    /* possibly a directive; try to parse it */
+	    parse_dirvals(parser_gas, &vps);
+	    if (!yasm_object_directive(p_object, id, "gas", &vps, NULL,
+				       cur_line)) {
+		yasm_vps_delete(&vps);
+		yasm_xfree(id);
+		return NULL;
+	    }
+	    yasm_vps_delete(&vps);
 	    if (id[0] == '.')
 		yasm_warn_set(YASM_WARN_GENERAL,
 			      N_("directive `%s' not recognized"), id);
@@ -377,8 +383,8 @@ parse_line(yasm_parser_gas *parser_gas)
 		}
 	    }
 
-	    return gas_parser_align(parser_gas, parser_gas->cur_section, bound,
-				    fill, maxskip, (int)ival);
+	    return gas_parser_align(parser_gas, cursect, bound, fill, maxskip,
+				    (int)ival);
 	}
 	case DIR_ORG:
 	    get_next_token(); /* DIR_ORG */
@@ -393,16 +399,7 @@ parse_line(yasm_parser_gas *parser_gas)
 	case DIR_LOCAL:
 	    get_next_token(); /* DIR_LOCAL */
 	    if (!expect(ID)) return NULL;
-	    yasm_symtab_declare(parser_gas->symtab, ID_val, YASM_SYM_DLOCAL,
-				cur_line);
-	    yasm_xfree(ID_val);
-	    get_next_token(); /* ID */
-	    return NULL;
-	case DIR_GLOBAL:
-	    get_next_token(); /* DIR_GLOBAL */
-	    if (!expect(ID)) return NULL;
-	    yasm_objfmt_global_declare(parser_gas->objfmt, ID_val, NULL,
-				       cur_line);
+	    yasm_symtab_declare(p_symtab, ID_val, YASM_SYM_DLOCAL, cur_line);
 	    yasm_xfree(ID_val);
 	    get_next_token(); /* ID */
 	    return NULL;
@@ -436,50 +433,29 @@ parse_line(yasm_parser_gas *parser_gas)
 	    }
 	    /* If already explicitly declared local, treat like LCOMM */
 	    if (is_lcomm
-		|| ((sym = yasm_symtab_get(parser_gas->symtab, id))
+		|| ((sym = yasm_symtab_get(p_symtab, id))
 		    && yasm_symrec_get_visibility(sym) == YASM_SYM_DLOCAL)) {
 		define_lcomm(parser_gas, id, e, align);
 	    } else if (align) {
 		/* Give third parameter as objext valparam */
-		yasm_vps_initialize(&vps);
-		vp = yasm_vp_create(NULL, align);
-		yasm_vps_append(&vps, vp);
+		yasm_valparamhead *extvps = yasm_vps_create();
+                vp = yasm_vp_create_expr(NULL, align);
+		yasm_vps_append(extvps, vp);
 
-		yasm_objfmt_common_declare(parser_gas->objfmt, id, e, &vps,
-					   cur_line);
+		sym = yasm_symtab_declare(p_symtab, id, YASM_SYM_COMMON,
+					  cur_line);
+		yasm_symrec_set_common_size(sym, e);
+		yasm_symrec_set_objext_valparams(sym, extvps);
 
-		yasm_vps_delete(&vps);
 		yasm_xfree(id);
 	    } else {
-		yasm_objfmt_common_declare(parser_gas->objfmt, id, e, NULL,
-					   cur_line);
+		sym = yasm_symtab_declare(p_symtab, id, YASM_SYM_COMMON,
+					  cur_line);
+		yasm_symrec_set_common_size(sym, e);
 		yasm_xfree(id);
 	    }
 	    return NULL;
 	}
-	case DIR_EXTERN:
-	    get_next_token(); /* DIR_EXTERN */
-	    if (!expect(ID)) return NULL;
-	    /* Go ahead and do it, even though all undef become extern */
-	    yasm_objfmt_extern_declare(parser_gas->objfmt, ID_val, NULL,
-				       cur_line);
-	    yasm_xfree(ID_val);
-	    get_next_token(); /* ID */
-	    return NULL;
-	case DIR_WEAK:
-	    get_next_token(); /* DIR_EXTERN */
-	    if (!expect(ID)) return NULL;
-
-	    yasm_vps_initialize(&vps);
-	    vp = yasm_vp_create(ID_val, NULL);
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* ID */
-
-	    yasm_objfmt_directive(parser_gas->objfmt, "weak", &vps, NULL,
-				  cur_line);
-
-	    yasm_vps_delete(&vps);
-	    return NULL;
 
 	/* Integer data definition directives */
 	case DIR_ASCII:
@@ -487,14 +463,14 @@ parse_line(yasm_parser_gas *parser_gas)
 	    get_next_token(); /* DIR_ASCII */
 	    if (!parse_strvals(parser_gas, &dvs))
 		return NULL;
-	    return yasm_bc_create_data(&dvs, 1, (int)ival, parser_gas->arch,
+	    return yasm_bc_create_data(&dvs, 1, (int)ival, p_object->arch,
 				       cur_line);
 	case DIR_DATA:
 	    ival = DIR_DATA_val;
 	    get_next_token(); /* DIR_DATA */
 	    if (!parse_datavals(parser_gas, &dvs))
 		return NULL;
-	    return yasm_bc_create_data(&dvs, ival, 0, parser_gas->arch,
+	    return yasm_bc_create_data(&dvs, ival, 0, p_object->arch,
 				       cur_line);
 	case DIR_LEB128:
 	    ival = DIR_LEB128_val;
@@ -516,7 +492,7 @@ parse_line(yasm_parser_gas *parser_gas)
 	    yasm_dvs_initialize(&dvs);
 	    yasm_dvs_append(&dvs, yasm_dv_create_expr(
 		p_expr_new_ident(yasm_expr_int(yasm_intnum_create_uint(0)))));
-	    bc = yasm_bc_create_data(&dvs, 1, 0, parser_gas->arch, cur_line);
+	    bc = yasm_bc_create_data(&dvs, 1, 0, p_object->arch, cur_line);
 	    yasm_bc_set_multiple(bc, e);
 	    return bc;
 	case DIR_SKIP:
@@ -536,7 +512,7 @@ parse_line(yasm_parser_gas *parser_gas)
 	    e_val = parse_expr(parser_gas);
 	    yasm_dvs_initialize(&dvs);
 	    yasm_dvs_append(&dvs, yasm_dv_create_expr(e_val));
-	    bc = yasm_bc_create_data(&dvs, 1, 0, parser_gas->arch, cur_line);
+	    bc = yasm_bc_create_data(&dvs, 1, 0, p_object->arch, cur_line);
 
 	    yasm_bc_set_multiple(bc, e);
 	    return bc;
@@ -624,14 +600,26 @@ parse_line(yasm_parser_gas *parser_gas)
 	}
 
 	/* Other directives */
-	case DIR_IDENT:
-	    get_next_token(); /* DIR_IDENT */
-	    if (!parse_dirstrvals(parser_gas, &vps))
-		return NULL;
-	    yasm_objfmt_directive(parser_gas->objfmt, "ident", &vps, NULL,
-				  cur_line);
-	    yasm_vps_delete(&vps);
-	    return NULL;
+        case DIR_EQU:
+            /* ID ',' expr */
+            get_next_token(); /* DIR_EQU */
+            if (!expect(ID)) return NULL;
+            id = ID_val;
+            get_next_token(); /* ID */
+            if (!expect(',')) {
+                yasm_xfree(id);
+                return NULL;
+            }
+            get_next_token(); /* ',' */
+            e = parse_expr(parser_gas);
+            if (e)
+                yasm_symtab_define_equ(p_symtab, id, e, cur_line);
+            else
+                yasm_error_set(YASM_ERROR_SYNTAX,
+                               N_("expression expected after `%s'"), ",");
+            yasm_xfree(id);
+            return NULL;
+
 	case DIR_FILE:
 	    get_next_token(); /* DIR_FILE */
 	    if (curtok == STRING) {
@@ -664,11 +652,11 @@ parse_line(yasm_parser_gas *parser_gas)
 
 		/* Pass change along to debug format */
 		yasm_vps_initialize(&vps);
-		vp = yasm_vp_create(filename, NULL);
+                vp = yasm_vp_create_string(NULL, filename);
 		yasm_vps_append(&vps, vp);
 
-		yasm_dbgfmt_directive(parser_gas->dbgfmt, "file",
-				      parser_gas->cur_section, &vps, cur_line);
+		yasm_object_directive(p_object, ".file", "gas", &vps, NULL,
+				      cur_line);
 
 		yasm_vps_delete(&vps);
 		return NULL;
@@ -678,7 +666,7 @@ parse_line(yasm_parser_gas *parser_gas)
 	    yasm_vps_initialize(&vps);
 
 	    if (!expect(INTNUM)) return NULL;
-	    vp = yasm_vp_create(NULL,
+            vp = yasm_vp_create_expr(NULL,
 				p_expr_new_ident(yasm_expr_int(INTNUM_val)));
 	    yasm_vps_append(&vps, vp);
 	    get_next_token(); /* INTNUM */
@@ -687,111 +675,11 @@ parse_line(yasm_parser_gas *parser_gas)
 		yasm_vps_delete(&vps);
 		return NULL;
 	    }
-	    vp = yasm_vp_create(STRING_val.contents, NULL);
+            vp = yasm_vp_create_string(NULL, STRING_val.contents);
 	    yasm_vps_append(&vps, vp);
 	    get_next_token(); /* STRING */
 
-	    yasm_dbgfmt_directive(parser_gas->dbgfmt, "file",
-				  parser_gas->cur_section, &vps, cur_line);
-
-	    yasm_vps_delete(&vps);
-	    return NULL;
-	case DIR_LOC:
-	    /* INTNUM INTNUM INTNUM */
-	    get_next_token(); /* DIR_LOC */
-	    yasm_vps_initialize(&vps);
-
-	    if (!expect(INTNUM)) return NULL;
-	    vp = yasm_vp_create(NULL,
-				p_expr_new_ident(yasm_expr_int(INTNUM_val)));
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* INTNUM */
-
-	    if (!expect(INTNUM)) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    vp = yasm_vp_create(NULL,
-				p_expr_new_ident(yasm_expr_int(INTNUM_val)));
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* INTNUM */
-
-	    if (!expect(INTNUM)) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    vp = yasm_vp_create(NULL,
-				p_expr_new_ident(yasm_expr_int(INTNUM_val)));
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* INTNUM */
-
-	    yasm_dbgfmt_directive(parser_gas->dbgfmt, "loc",
-				  parser_gas->cur_section, &vps, cur_line);
-
-	    yasm_vps_delete(&vps);
-	    return NULL;
-	case DIR_TYPE:
-	    /* ID ',' '@' ID */
-	    get_next_token(); /* DIR_TYPE */
-	    yasm_vps_initialize(&vps);
-
-	    if (!expect(ID)) return NULL;
-	    vp = yasm_vp_create(ID_val, NULL);
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* ID */
-
-	    if (!expect(',')) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    get_next_token(); /* ',' */
-
-	    if (!expect('@')) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    get_next_token(); /* '@' */
-
-	    if (!expect(ID)) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    vp = yasm_vp_create(ID_val, NULL);
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* ID */
-
-	    yasm_objfmt_directive(parser_gas->objfmt, "type", &vps, NULL,
-				  cur_line);
-
-	    yasm_vps_delete(&vps);
-	    return NULL;
-	case DIR_SIZE:
-	    /* ID ',' expr */
-	    get_next_token(); /* DIR_SIZE */
-	    yasm_vps_initialize(&vps);
-
-	    if (!expect(ID)) return NULL;
-	    vp = yasm_vp_create(ID_val, NULL);
-	    yasm_vps_append(&vps, vp);
-	    get_next_token(); /* ID */
-
-	    if (!expect(',')) {
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    get_next_token(); /* ',' */
-
-	    e = parse_expr(parser_gas);
-	    if (!e) {
-		yasm_error_set(YASM_ERROR_SYNTAX,
-			       N_("expression expected for `.size'"));
-		yasm_vps_delete(&vps);
-		return NULL;
-	    }
-	    vp = yasm_vp_create(NULL, e);
-	    yasm_vps_append(&vps, vp);
-
-	    yasm_objfmt_directive(parser_gas->objfmt, "size", &vps, NULL,
+	    yasm_object_directive(p_object, ".file", "gas", &vps, NULL,
 				  cur_line);
 
 	    yasm_vps_delete(&vps);
@@ -818,7 +706,7 @@ parse_instr(yasm_parser_gas *parser_gas)
 	    get_next_token();
 	    if (is_eol()) {
 		/* no operands */
-		return yasm_bc_create_insn(parser_gas->arch, insn.arch_data,
+		return yasm_bc_create_insn(p_object->arch, insn.arch_data,
 					   0, NULL, cur_line);
 	    }
 
@@ -843,7 +731,7 @@ parse_instr(yasm_parser_gas *parser_gas)
 		}
 		get_next_token();
 	    }
-	    return yasm_bc_create_insn(parser_gas->arch, insn.arch_data,
+	    return yasm_bc_create_insn(p_object->arch, insn.arch_data,
 				       num_operands, &operands, cur_line);
 	}
 	case PREFIX:
@@ -852,7 +740,7 @@ parse_instr(yasm_parser_gas *parser_gas)
 	    get_next_token(); /* PREFIX */
 	    bc = parse_instr(parser_gas);
 	    if (!bc)
-		bc = yasm_bc_create_empty_insn(parser_gas->arch, cur_line);
+		bc = yasm_bc_create_empty_insn(p_object->arch, cur_line);
 	    yasm_bc_insn_add_prefix(bc, prefix.arch_data);
 	    return bc;
 	}
@@ -862,7 +750,7 @@ parse_instr(yasm_parser_gas *parser_gas)
 	    get_next_token(); /* SEGREG */
 	    bc = parse_instr(parser_gas);
 	    if (!bc)
-		bc = yasm_bc_create_empty_insn(parser_gas->arch, cur_line);
+		bc = yasm_bc_create_empty_insn(p_object->arch, cur_line);
 	    yasm_bc_insn_add_seg_prefix(bc, segreg);
 	}
 	default:
@@ -873,45 +761,49 @@ parse_instr(yasm_parser_gas *parser_gas)
 static int
 parse_dirvals(yasm_parser_gas *parser_gas, yasm_valparamhead *vps)
 {
+    yasm_valparam *vp;
     yasm_expr *e;
-    yasm_valparam *vp;
     int num = 0;
 
     yasm_vps_initialize(vps);
 
     for (;;) {
-	e = parse_expr(parser_gas);
-	vp = yasm_vp_create(NULL, e);
-	yasm_vps_append(vps, vp);
-	num++;
-	if (curtok != ',')
-	    break;
-	get_next_token(); /* ',' */
-    }
-    return num;
-}
-
-static int
-parse_dirstrvals(yasm_parser_gas *parser_gas, yasm_valparamhead *vps)
-{
-    yasm_valparam *vp;
-    int num = 0;
-
-    yasm_vps_initialize(vps);
-
-    for (;;) {
-	if (!expect(STRING)) {
-	    yasm_vps_delete(vps);
-	    yasm_vps_initialize(vps);
-	    return 0;
+	switch (curtok) {
+	    case ID:
+		get_peek_token(parser_gas);
+                switch (parser_gas->peek_token) {
+                    case '+': case '-':
+                    case '|': case '^': case '&': case '!':
+                    case '*': case '/': case '%': case LEFT_OP: case RIGHT_OP:
+                        e = parse_expr(parser_gas);
+                        vp = yasm_vp_create_expr(NULL, e);
+                        break;
+                    default:
+		    /* Just an ID */
+                        vp = yasm_vp_create_id(NULL, ID_val, '\0');
+		    get_next_token(); /* ID */
+                        break;
+		}
+		break;
+	    case STRING:
+                vp = yasm_vp_create_string(NULL, STRING_val.contents);
+		get_next_token(); /* STRING */
+		break;
+	    case '@':
+		/* XXX: is throwing it away *really* the right thing? */
+		get_next_token(); /* @ */
+		continue;
+	    default:
+		e = parse_expr(parser_gas);
+		if (!e)
+		    return num;
+                vp = yasm_vp_create_expr(NULL, e);
+		break;
 	}
-	vp = yasm_vp_create(STRING_val.contents, NULL);
 	yasm_vps_append(vps, vp);
-	get_next_token(); /* STRING */
 	num++;
-	if (curtok != ',')
-	    break;
-	get_next_token(); /* ',' */
+	if (curtok == ',')
+	    get_next_token(); /* ',' */
     }
     return num;
 }
@@ -1001,7 +893,7 @@ parse_memaddr(yasm_parser_gas *parser_gas)
 
     if (curtok == '(') {
 	int havereg = 0;
-	uintptr_t reg;
+	uintptr_t reg = 0;
 	yasm_intnum *scale = NULL;
 
 	get_next_token(); /* '(' */
@@ -1084,7 +976,7 @@ done:
 
     if (!e1)
 	return NULL;
-    ea = yasm_arch_ea_create(parser_gas->arch, e1);
+    ea = yasm_arch_ea_create(p_object->arch, e1);
     if (strong)
 	yasm_ea_set_strong(ea, 1);
     return ea;
@@ -1135,7 +1027,7 @@ parse_operand(yasm_parser_gas *parser_gas)
 		return NULL;
 	    }
 	    get_next_token(); /* ')' */
-	    reg = yasm_arch_reggroup_get_reg(parser_gas->arch, reg, regindex);
+	    reg = yasm_arch_reggroup_get_reg(p_object->arch, reg, regindex);
 	    if (reg == 0) {
 		yasm_error_set(YASM_ERROR_SYNTAX, N_("bad register index `%u'"),
 			       regindex);
@@ -1396,26 +1288,24 @@ gas_get_section(yasm_parser_gas *parser_gas, char *name,
     yasm_section *new_section;
 
     yasm_vps_initialize(&vps);
-    vp = yasm_vp_create(name, NULL);
+    vp = yasm_vp_create_id(NULL, name, '\0');
     yasm_vps_append(&vps, vp);
 
     if (!builtin) {
-	if (flags) {
-	    gasflags = yasm_xmalloc(5+strlen(flags));
-	    strcpy(gasflags, "gas_");
-	    strcat(gasflags, flags);
-	} else
-	    gasflags = yasm__xstrdup("gas_");
-	vp = yasm_vp_create(gasflags, NULL);
+        if (flags)
+            gasflags = yasm__xstrdup(flags);
+        else
+            gasflags = yasm__xstrdup("");
+        vp = yasm_vp_create_string(yasm__xstrdup("gasflags"), gasflags);
 	yasm_vps_append(&vps, vp);
 	if (type) {
-	    vp = yasm_vp_create(type, NULL);
+            vp = yasm_vp_create_id(NULL, type, '\0');
 	    yasm_vps_append(&vps, vp);
 	}
     }
 
-    new_section = yasm_objfmt_section_switch(parser_gas->objfmt, &vps,
-					     objext_valparams, cur_line);
+    new_section = yasm_objfmt_section_switch(p_object, &vps, objext_valparams,
+					     cur_line);
 
     yasm_vps_delete(&vps);
     return new_section;
@@ -1432,7 +1322,7 @@ gas_switch_section(yasm_parser_gas *parser_gas, char *name,
     new_section = gas_get_section(parser_gas, yasm__xstrdup(name), flags, type,
 				  objext_valparams, builtin);
     if (new_section) {
-	parser_gas->cur_section = new_section;
+	cursect = new_section;
 	parser_gas->prev_bc = yasm_section_bcs_last(new_section);
     } else
 	yasm_error_set(YASM_ERROR_GENERAL, N_("invalid section name `%s'"),
@@ -1471,7 +1361,7 @@ gas_parser_align(yasm_parser_gas *parser_gas, yasm_section *sect,
 
     return yasm_bc_create_align(boundval, fillval, maxskipval,
 				yasm_section_is_code(sect) ?
-				    yasm_arch_get_fill(parser_gas->arch) : NULL,
+				    yasm_arch_get_fill(p_object->arch) : NULL,
 				cur_line);
 }
 
@@ -1506,13 +1396,13 @@ gas_parser_dir_fill(yasm_parser_gas *parser_gas, /*@only@*/ yasm_expr *repeat,
 
     yasm_dvs_initialize(&dvs);
     yasm_dvs_append(&dvs, yasm_dv_create_expr(value));
-    bc = yasm_bc_create_data(&dvs, ssize, 0, parser_gas->arch, cur_line);
+    bc = yasm_bc_create_data(&dvs, ssize, 0, p_object->arch, cur_line);
 
     yasm_bc_set_multiple(bc, repeat);
 
     return bc;
 }
-
+#if 0
 static void
 gas_parser_directive(yasm_parser_gas *parser_gas, const char *name,
 		      yasm_valparamhead *valparams,
@@ -1521,10 +1411,10 @@ gas_parser_directive(yasm_parser_gas *parser_gas, const char *name,
     unsigned long line = cur_line;
 
     /* Handle (mostly) output-format independent directives here */
-    if (!yasm_arch_parse_directive(parser_gas->arch, name, valparams,
+    if (!yasm_arch_parse_directive(p_object->arch, name, valparams,
 		    objext_valparams, parser_gas->object, line)) {
 	;
-    } else if (yasm_objfmt_directive(parser_gas->objfmt, name, valparams,
+    } else if (yasm_objfmt_directive(p_object, name, valparams,
 				     objext_valparams, line)) {
 	yasm_error_set(YASM_ERROR_GENERAL, N_("unrecognized directive [%s]"),
 		       name);
@@ -1534,3 +1424,4 @@ gas_parser_directive(yasm_parser_gas *parser_gas, const char *name,
     if (objext_valparams)
 	yasm_vps_delete(objext_valparams);
 }
+#endif
