@@ -1,7 +1,7 @@
 /*
  * GAS-compatible parser
  *
- *  Copyright (C) 2005  Peter Johnson
+ *  Copyright (C) 2005-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: gas-parser.c 1735 2007-01-18 04:09:59Z peter $");
+/*@unused@*/ RCSID("$Id: gas-parser.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #include <libyasm.h>
@@ -37,16 +37,14 @@
 
 
 static void
-gas_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
-		    yasm_objfmt *of, yasm_dbgfmt *df, FILE *f,
-		    const char *in_filename, int save_input,
-		    yasm_section *def_sect, yasm_errwarns *errwarns)
+gas_parser_do_parse(yasm_object *object, yasm_preproc *pp, FILE *f,
+		    int save_input, yasm_linemap *linemap,
+		    yasm_errwarns *errwarns)
 {
     yasm_parser_gas parser_gas;
 
     parser_gas.object = object;
-    parser_gas.linemap = yasm_object_get_linemap(parser_gas.object);
-    parser_gas.symtab = yasm_object_get_symtab(parser_gas.object);
+    parser_gas.linemap = linemap;
 
     parser_gas.in = f;
 
@@ -58,13 +56,9 @@ gas_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
     parser_gas.dir_line = 0;
 
     parser_gas.preproc = pp;
-    parser_gas.arch = a;
-    parser_gas.objfmt = of;
-    parser_gas.dbgfmt = df;
     parser_gas.errwarns = errwarns;
 
-    parser_gas.cur_section = def_sect;
-    parser_gas.prev_bc = yasm_section_bcs_first(def_sect);
+    parser_gas.prev_bc = yasm_section_bcs_first(object->cur_section);
 
     parser_gas.save_input = save_input;
     parser_gas.save_last = 0;
@@ -107,6 +101,9 @@ gas_parser_do_parse(yasm_object *object, yasm_preproc *pp, yasm_arch *a,
 
     if (parser_gas.dir_file)
 	yasm_xfree(parser_gas.dir_file);
+
+    /* Convert all undefined symbols into extern symbols */
+    yasm_symtab_parser_finalize(object->symtab, 1, errwarns);
 }
 
 /* Define valid preprocessors to use with this parser */
