@@ -1,7 +1,7 @@
-/* $Id: gas-parser.h 1774 2007-02-18 20:42:55Z peter $
+/* $Id: gas-parser.h 1876 2007-06-02 21:50:51Z peter $
  * GAS-compatible parser header file
  *
- *  Copyright (C) 2005  Peter Johnson
+ *  Copyright (C) 2005-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -54,25 +54,18 @@ enum tokentype {
     DIR_COMM,
     DIR_DATA,
     DIR_ENDR,
-    DIR_EXTERN,
     DIR_EQU,
     DIR_FILE,
     DIR_FILL,
-    DIR_GLOBAL,
-    DIR_IDENT,
     DIR_LEB128,
     DIR_LINE,
-    DIR_LOC,
     DIR_LOCAL,
     DIR_LCOMM,
     DIR_ORG,
     DIR_REPT,
     DIR_SECTION,
     DIR_SECTNAME,
-    DIR_SIZE,
     DIR_SKIP,
-    DIR_TYPE,
-    DIR_WEAK,
     DIR_ZERO,
     NONE
 };
@@ -115,7 +108,6 @@ typedef struct yasm_parser_gas {
     int debug;
 
     /*@only@*/ yasm_object *object;
-    /*@dependent@*/ yasm_section *cur_section;
 
     /* last "base" label for local (.) labels */
     /*@null@*/ char *locallabel_base;
@@ -127,13 +119,9 @@ typedef struct yasm_parser_gas {
     unsigned long dir_line;
 
     /*@dependent@*/ yasm_preproc *preproc;
-    /*@dependent@*/ yasm_arch *arch;
-    /*@dependent@*/ yasm_objfmt *objfmt;
-    /*@dependent@*/ yasm_dbgfmt *dbgfmt;
     /*@dependent@*/ yasm_errwarns *errwarns;
 
     /*@dependent@*/ yasm_linemap *linemap;
-    /*@dependent@*/ yasm_symtab *symtab;
 
     /*@null@*/ yasm_bytecode *prev_bc;
     yasm_bytecode *temp_bc;
@@ -163,7 +151,9 @@ typedef struct yasm_parser_gas {
 } yasm_parser_gas;
 
 /* shorter access names to commonly used parser_gas fields */
-#define p_symtab	(parser_gas->symtab)
+#define p_object	(parser_gas->object)
+#define p_symtab	(parser_gas->object->symtab)
+#define cursect		(parser_gas->object->cur_section)
 #define curtok		(parser_gas->token)
 #define curval		(parser_gas->tokval)
 
