@@ -1,7 +1,7 @@
 /*
  * NASM-compatible parser
  *
- *  Copyright (C) 2001  Peter Johnson, Michael Urman
+ *  Copyright (C) 2001-2007  Peter Johnson, Michael Urman
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-RCSID("$Id: nasm-parse.c 1795 2007-02-24 22:39:55Z peter $");
+RCSID("$Id: nasm-parse.c 1876 2007-06-02 21:50:51Z peter $");
 
 #define YASM_LIB_INTERNAL
 #define YASM_EXPR_INTERNAL
@@ -65,8 +65,6 @@ static void nasm_parser_directive
     (yasm_parser_nasm *parser_nasm, const char *name,
      /*@null@*/ yasm_valparamhead *valparams,
      /*@null@*/ yasm_valparamhead *objext_valparams);
-static int fix_directive_symrec(/*@null@*/ yasm_expr__item *ei,
-				/*@null@*/ void *d);
 static void define_label(yasm_parser_nasm *parser_nasm, /*@only@*/ char *name,
 			 int local);
 
@@ -139,48 +137,48 @@ demand_eol_(yasm_parser_nasm *parser_nasm)
 static int
 expect_(yasm_parser_nasm *parser_nasm, int token)
 {
-    static char strch[] = "expected ` '";
+    static char strch[] = "` '";
     const char *str;
 
     if (curtok == token)
 	return 1;
 
     switch (token) {
-	case INTNUM:		str = "expected integer"; break;
-	case FLTNUM:		str = "expected floating point value"; break;
-	case DIRECTIVE_NAME:	str = "expected directive name"; break;
-	case FILENAME:		str = "expected filename"; break;
-	case STRING:		str = "expected string"; break;
-	case SIZE_OVERRIDE:	str = "expected size override"; break;
-	case DECLARE_DATA:	str = "expected DB/DW/etc."; break;
-	case RESERVE_SPACE:	str = "expected RESB/RESW/etc."; break;
-	case INCBIN:		str = "expected INCBIN"; break;
-	case EQU:		str = "expected EQU"; break;
-	case TIMES:		str = "expected TIMES"; break;
-	case SEG:		str = "expected SEG"; break;
-	case WRT:		str = "expected WRT"; break;
-	case NOSPLIT:		str = "expected NOSPLIT"; break;
-	case STRICT:		str = "expected STRICT"; break;
-	case INSN:		str = "expected instruction"; break;
-	case PREFIX:		str = "expected instruction prefix"; break;
-	case REG:		str = "expected register"; break;
-	case SEGREG:		str = "expected segment register"; break;
-	case TARGETMOD:		str = "expected target modifier"; break;
-	case LEFT_OP:		str = "expected <<"; break;
-	case RIGHT_OP:		str = "expected >>"; break;
-	case SIGNDIV:		str = "expected //"; break;
-	case SIGNMOD:		str = "expected %%"; break;
-	case START_SECTION_ID:	str = "expected $$"; break;
-	case ID:		str = "expected identifier"; break;
-	case LOCAL_ID:		str = "expected .identifier"; break;
-	case SPECIAL_ID:	str = "expected ..identifier"; break;
-	case LINE:		str = "expected %line"; break;
+	case INTNUM:		str = "integer"; break;
+	case FLTNUM:		str = "floating point value"; break;
+	case DIRECTIVE_NAME:	str = "directive name"; break;
+	case FILENAME:		str = "filename"; break;
+	case STRING:		str = "string"; break;
+	case SIZE_OVERRIDE:	str = "size override"; break;
+	case DECLARE_DATA:	str = "DB/DW/etc."; break;
+	case RESERVE_SPACE:	str = "RESB/RESW/etc."; break;
+	case INCBIN:		str = "INCBIN"; break;
+	case EQU:		str = "EQU"; break;
+	case TIMES:		str = "TIMES"; break;
+	case SEG:		str = "SEG"; break;
+	case WRT:		str = "WRT"; break;
+	case NOSPLIT:		str = "NOSPLIT"; break;
+	case STRICT:		str = "STRICT"; break;
+	case INSN:		str = "instruction"; break;
+	case PREFIX:		str = "instruction prefix"; break;
+	case REG:		str = "register"; break;
+	case SEGREG:		str = "segment register"; break;
+	case TARGETMOD:		str = "target modifier"; break;
+	case LEFT_OP:		str = "<<"; break;
+	case RIGHT_OP:		str = ">>"; break;
+	case SIGNDIV:		str = "//"; break;
+	case SIGNMOD:		str = "%%"; break;
+	case START_SECTION_ID:	str = "$$"; break;
+	case ID:		str = "identifier"; break;
+	case LOCAL_ID:		str = ".identifier"; break;
+	case SPECIAL_ID:	str = "..identifier"; break;
+	case LINE:		str = "%line"; break;
 	default:
-	    strch[10] = token;
+	    strch[1] = token;
 	    str = strch;
 	    break;
     }
-    yasm_error_set(YASM_ERROR_PARSE, str);
+    yasm_error_set(YASM_ERROR_PARSE, "expected %s", str);
     destroy_curtok();
     return 0;
 }
@@ -197,11 +195,41 @@ nasm_parser_parse(yasm_parser_nasm *parser_nasm)
 	    demand_eol();
 	}
 
+	if (parser_nasm->abspos) {
+	    /* If we're inside an absolute section, just add to the absolute
+	     * position rather than appending bytecodes to a section.
+	     * Only RES* are allowed in an absolute section, so this is easy.
+	     */
+	    if (bc) {
+		/*@null@*/ const yasm_expr *numitems, *multiple;
+		unsigned int itemsize;
+		numitems = yasm_bc_reserve_numitems(bc, &itemsize);
+		if (numitems) {
+		    yasm_expr *e;
+		    e = yasm_expr_create(YASM_EXPR_MUL,
+			yasm_expr_expr(yasm_expr_copy(numitems)),
+			yasm_expr_int(yasm_intnum_create_uint(itemsize)),
+			cur_line);
+		    multiple = yasm_bc_get_multiple_expr(bc);
+		    if (multiple)
+			e = yasm_expr_create_tree(e, YASM_EXPR_MUL,
+						  yasm_expr_copy(multiple),
+						  cur_line);
+		    parser_nasm->abspos = yasm_expr_create_tree(
+			parser_nasm->abspos, YASM_EXPR_ADD, e, cur_line);
+		} else
+		    yasm_error_set(YASM_ERROR_SYNTAX,
+			N_("only RES* allowed within absolute section"));
+		yasm_bc_destroy(bc);
+	    }
+	    temp_bc = NULL;
+	} else {
+	    temp_bc = yasm_section_bcs_append(cursect, bc);
+	    if (temp_bc)
+		parser_nasm->prev_bc = temp_bc;
+	}
 	yasm_errwarn_propagate(parser_nasm->errwarns, cur_line);
 
-	temp_bc = yasm_section_bcs_append(parser_nasm->cur_section, bc);
-	if (temp_bc)
-	    parser_nasm->prev_bc = temp_bc;
 	if (parser_nasm->save_input)
 	    yasm_linemap_add_source(parser_nasm->linemap,
 		temp_bc,
@@ -364,63 +392,58 @@ parse_directive_valparams(yasm_parser_nasm *parser_nasm,
     for (;;) {
 	yasm_valparam *vp;
 	yasm_expr *e;
+        char *id = NULL;
 
+        /* Look for value first */
+        if (curtok == ID) {
+            get_peek_token(parser_nasm);
+            if (parser_nasm->peek_token == '=') {
+                id = ID_val;
+                get_next_token(); /* id */
+                get_next_token(); /* '=' */
+            }
+        }
+
+        /* Look for parameter */
 	switch (curtok) {
 	    case STRING:
-		vp = yasm_vp_create(STRING_val.contents, NULL);
+                vp = yasm_vp_create_string(id, STRING_val.contents);
 		get_next_token();
 		break;
 	    case ID:
-	    {
-		char *id = ID_val;
+                /* We need a peek token, but avoid error if we have one
+                 * already; we need to work whether or not we hit the
+                 * "value=" if test above.
+                 *
+                 * We cheat and peek ahead to see if this is just an ID or
+                 * the ID is part of an expression.  We assume a + or - means
+                 * that it's part of an expression (e.g. "x+y" is parsed as
+                 * the expression "x+y" and not as "x", "+y").
+                 */
+                if (parser_nasm->peek_token == NONE)
 		get_peek_token(parser_nasm);
-		if (parser_nasm->peek_token == '=') {
-		    get_next_token(); /* id */
-		    get_next_token(); /* = */
-		    e = parse_expr(parser_nasm, DIR_EXPR);
-		    if (!e) {
-			yasm_vps_delete(vps);
-			return 0;
-		    }
-		    yasm_expr__traverse_leaves_in(e, parser_nasm,
-						  fix_directive_symrec);
-		    vp = yasm_vp_create(id, e);
-		    break;
-		}
-		if (parser_nasm->peek_token == ','
-		    || parser_nasm->peek_token == ']'
-		    || parser_nasm->peek_token == ':') {
-		    /* Try to catch just IDs here first */
-		    get_next_token(); /* id */
-		    vp = yasm_vp_create(id, NULL);
+                switch (parser_nasm->peek_token) {
+                    case '|': case '^': case '&': case LEFT_OP: case RIGHT_OP:
+                    case '+': case '-':
+                    case '*': case '/': case '%': case SIGNDIV: case SIGNMOD:
 		    break;
+                    default:
+                        /* Just an id */
+                        vp = yasm_vp_create_id(id, ID_val, '$');
+                        get_next_token();
+                        goto next;
 		}
 		/*@fallthrough@*/
-	    }
 	    default:
-	    {
-		/* If direxpr is just an ID, put it in val and delete the expr.
-		 * Otherwise, we need to go through the expr and replace the
-		 * current (local) symrecs with the use of global ones.
-		 */
-		const /*@null@*/ yasm_symrec *vp_symrec;
 		e = parse_expr(parser_nasm, DIR_EXPR);
-		if (!e) {
-		    yasm_vps_delete(vps);
-		    return 0;
+                if (!e) {
+                    yasm_vps_delete(vps);
+                    return 0;
+                }
+                vp = yasm_vp_create_expr(id, e);
+                break;
 		}
-		if ((vp_symrec = yasm_expr_get_symrec(&e, 0))) {
-		    vp = yasm_vp_create(
-			yasm__xstrdup(yasm_symrec_get_name(vp_symrec)), NULL);
-		    yasm_expr_destroy(e);
-		} else {
-		    yasm_expr__traverse_leaves_in(e, parser_nasm,
-						  fix_directive_symrec);
-		    vp = yasm_vp_create(NULL, e);
-		}
-	    }
-	}
-
+next:
 	yasm_vps_append(vps, vp);
 	if (curtok == ',')
 	    get_next_token();
@@ -506,7 +529,7 @@ dv_done:
 		if (is_eol())	/* allow trailing , on list */
 		    break;
 	    }
-	    return yasm_bc_create_data(&dvs, size, 0, parser_nasm->arch,
+	    return yasm_bc_create_data(&dvs, size, 0, p_object->arch,
 				       cur_line);
 	}
 	case RESERVE_SPACE:
@@ -583,8 +606,8 @@ parse_instr(yasm_parser_nasm *parser_nasm)
 	    get_next_token();
 	    if (is_eol()) {
 		/* no operands */
-		return yasm_bc_create_insn(parser_nasm->arch, insn.arch_data,
-					   0, NULL, cur_line);
+		return yasm_bc_create_insn(p_object->arch, insn.arch_data, 0,
+					   NULL, cur_line);
 	    }
 
 	    /* parse operands */
@@ -608,7 +631,7 @@ parse_instr(yasm_parser_nasm *parser_nasm)
 		}
 		get_next_token();
 	    }
-	    return yasm_bc_create_insn(parser_nasm->arch, insn.arch_data,
+	    return yasm_bc_create_insn(p_object->arch, insn.arch_data,
 				       num_operands, &operands, cur_line);
 	}
 	case PREFIX: {
@@ -677,11 +700,28 @@ parse_operand(yasm_parser_nasm *parser_nasm)
 	    if (!op)
 		return NULL;
 	    if (op->type == YASM_INSN__OPERAND_REG &&
-		yasm_arch_get_reg_size(parser_nasm->arch, op->data.reg) != size)
+		yasm_arch_get_reg_size(p_object->arch, op->data.reg) != size)
 		yasm_error_set(YASM_ERROR_TYPE,
 			       N_("cannot override register size"));
-	    else
+	    else {
+		/* Silently override others unless a warning is turned on.
+		 * This is to allow overrides such as:
+		 *   %define arg1 dword [bp+4]
+		 *   cmp word arg1, 2
+		 * Which expands to:
+		 *   cmp word dword [bp+4], 2
+		 */
+		if (op->size != 0) {
+		    if (op->size != size)
+			yasm_warn_set(YASM_WARN_SIZE_OVERRIDE,
+			    N_("overriding operand size from %u-bit to %u-bit"),
+			    op->size, size);
+		    else
+			yasm_warn_set(YASM_WARN_SIZE_OVERRIDE,
+				      N_("double operand size override"));
+		}
 		op->size = size;
+	    }
 	    return op;
 	}
 	case TARGETMOD:
@@ -744,7 +784,7 @@ parse_memaddr(yasm_parser_nasm *parser_nasm)
 	    yasm_expr *e = parse_expr(parser_nasm, NORM_EXPR);
 	    if (!e)
 		return NULL;
-	    return yasm_arch_ea_create(parser_nasm->arch, e);
+	    return yasm_arch_ea_create(p_object->arch, e);
 	}
     }
 }
@@ -938,10 +978,8 @@ parse_expr6(yasm_parser_nasm *parser_nasm, expr_type type)
 	    e = p_expr_new_ident(yasm_expr_reg(REG_val[0]));
 	    break;
 	case ID:
-	    e = p_expr_new_ident(yasm_expr_sym(
-		yasm_symtab_define_label(p_symtab, ID_val,
-		    yasm_section_bcs_first(parser_nasm->cur_section), 0,
-		    cur_line)));
+            sym = yasm_symtab_use(p_symtab, ID_val, cur_line);
+            e = p_expr_new_ident(yasm_expr_sym(sym));
 	    yasm_xfree(ID_val);
 	    break;
 	default:
@@ -1008,15 +1046,23 @@ parse_expr6(yasm_parser_nasm *parser_nasm, expr_type type)
 	    break;
 	case '$':
 	    /* "$" references the current assembly position */
-	    sym = yasm_symtab_define_curpos(p_symtab, "$",
-					    parser_nasm->prev_bc, cur_line);
-	    e = p_expr_new_ident(yasm_expr_sym(sym));
+	    if (parser_nasm->abspos)
+		e = yasm_expr_copy(parser_nasm->abspos);
+	    else {
+		sym = yasm_symtab_define_curpos(p_symtab, "$",
+		    parser_nasm->prev_bc, cur_line);
+		e = p_expr_new_ident(yasm_expr_sym(sym));
+	    }
 	    break;
 	case START_SECTION_ID:
 	    /* "$$" references the start of the current section */
-	    sym = yasm_symtab_define_label(p_symtab, "$$",
-		yasm_section_bcs_first(parser_nasm->cur_section), 0, cur_line);
-	    e = p_expr_new_ident(yasm_expr_sym(sym));
+	    if (parser_nasm->absstart)
+		e = yasm_expr_copy(parser_nasm->absstart);
+	    else {
+		sym = yasm_symtab_define_label(p_symtab, "$$",
+		    yasm_section_bcs_first(cursect), 0, cur_line);
+		e = p_expr_new_ident(yasm_expr_sym(sym));
+	    }
 	    break;
 	default:
 	    return NULL;
@@ -1037,171 +1083,45 @@ define_label(yasm_parser_nasm *parser_nasm, char *name, int local)
 	strcpy(parser_nasm->locallabel_base, name);
     }
 
-    yasm_symtab_define_label(p_symtab, name, parser_nasm->prev_bc, 1,
-			     cur_line);
+    if (parser_nasm->abspos)
+	yasm_symtab_define_equ(p_symtab, name,
+			       yasm_expr_copy(parser_nasm->abspos), cur_line);
+    else
+	yasm_symtab_define_label(p_symtab, name, parser_nasm->prev_bc, 1,
+				 cur_line);
     yasm_xfree(name);
 }
 
-static int
-fix_directive_symrec(yasm_expr__item *ei, void *d)
-{
-    yasm_parser_nasm *parser_nasm = (yasm_parser_nasm *)d;
-    if (!ei || ei->type != YASM_EXPR_SYM)
-	return 0;
-
-    /* FIXME: Delete current symrec */
-    ei->data.sym =
-	yasm_symtab_use(p_symtab, yasm_symrec_get_name(ei->data.sym),
-			cur_line);
-
-    return 0;
-}
-
 static void
-dir_extern(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	   yasm_valparamhead *objext_valparams)
+dir_align(yasm_object *object, yasm_valparamhead *valparams,
+	  yasm_valparamhead *objext_valparams, unsigned long line)
 {
     yasm_valparam *vp = yasm_vps_first(valparams);
-    yasm_objfmt_extern_declare(parser_nasm->objfmt, vp->val, objext_valparams,
-			       cur_line);
-}
-
-static void
-dir_global(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	   yasm_valparamhead *objext_valparams)
-{
-    yasm_valparam *vp = yasm_vps_first(valparams);
-    yasm_objfmt_global_declare(parser_nasm->objfmt, vp->val, objext_valparams,
-			       cur_line);
-}
-
-static void
-dir_common(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	   yasm_valparamhead *objext_valparams)
-{
-    yasm_valparam *vp = yasm_vps_first(valparams);
-    yasm_valparam *vp2 = yasm_vps_next(vp);
-    unsigned long line = cur_line;
-
-    if (!vp2 || (!vp2->val && !vp2->param)) {
-	yasm_error_set(YASM_ERROR_SYNTAX,
-		       N_("no size specified in %s declaration"), "COMMON");
-	return;
-    }
-    if (vp2->val) {
-	yasm_objfmt_common_declare(parser_nasm->objfmt, vp->val,
-	    p_expr_new_ident(yasm_expr_sym(
-		yasm_symtab_use(p_symtab, vp2->val, line))),
-	    objext_valparams, line);
-    } else if (vp2->param) {
-	yasm_objfmt_common_declare(parser_nasm->objfmt, vp->val, vp2->param,
-				   objext_valparams, line);
-	vp2->param = NULL;
-    }
-}
-
-static void
-dir_section(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	    yasm_valparamhead *objext_valparams)
-{
-    yasm_valparam *vp = yasm_vps_first(valparams);
-    yasm_section *new_section =
-	yasm_objfmt_section_switch(parser_nasm->objfmt, valparams,
-				   objext_valparams, cur_line);
-    if (new_section) {
-	parser_nasm->cur_section = new_section;
-	parser_nasm->prev_bc = yasm_section_bcs_last(new_section);
-    } else
-	yasm_error_set(YASM_ERROR_SYNTAX, N_("invalid argument to [%s]"),
-		       "SECTION");
-}
-
-static void
-dir_absolute(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	     yasm_valparamhead *objext_valparams)
-{
-    yasm_valparam *vp = yasm_vps_first(valparams);
-    unsigned long line = cur_line;
-    /* it can be just an ID or a complete expression, so handle both. */
-    if (vp->val)
-	parser_nasm->cur_section =
-	    yasm_object_create_absolute(parser_nasm->object,
-		p_expr_new_ident(yasm_expr_sym(
-		    yasm_symtab_use(p_symtab, vp->val, line))), line);
-    else if (vp->param) {
-	parser_nasm->cur_section =
-	    yasm_object_create_absolute(parser_nasm->object, vp->param, line);
-	vp->param = NULL;
-    }
-    parser_nasm->prev_bc = yasm_section_bcs_last(parser_nasm->cur_section);
-}
-
-static void
-dir_align(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	  yasm_valparamhead *objext_valparams)
-{
-    /*@only@*/ yasm_expr *boundval;
+    yasm_expr *boundval = yasm_vp_expr(vp, object->symtab, line);
     /*@depedent@*/ yasm_intnum *boundintn;
-    yasm_valparam *vp = yasm_vps_first(valparams);
-
-    /* it can be just an ID or a complete expression, so handle both. */
-    vp = yasm_vps_first(valparams);
-    if (vp->val)
-	boundval = p_expr_new_ident(yasm_expr_sym(
-	    yasm_symtab_use(p_symtab, vp->val, cur_line)));
-    else if (vp->param) {
-	boundval = vp->param;
-	vp->param = NULL;
-    }
 
     /* Largest .align in the section specifies section alignment.
      * Note: this doesn't match NASM behavior, but is a lot more
      * intelligent!
      */
-    boundintn = yasm_expr_get_intnum(&boundval, 0);
-    if (boundintn) {
+    if (boundval && (boundintn = yasm_expr_get_intnum(&boundval, 0))) {
 	unsigned long boundint = yasm_intnum_get_uint(boundintn);
 
 	/* Alignments must be a power of two. */
 	if (is_exp2(boundint)) {
-	    if (boundint > yasm_section_get_align(parser_nasm->cur_section))
-		yasm_section_set_align(parser_nasm->cur_section, boundint,
-				       cur_line);
+	    if (boundint > yasm_section_get_align(object->cur_section))
+		yasm_section_set_align(object->cur_section, boundint, line);
 	}
     }
 
     /* As this directive is called only when nop is used as fill, always
      * use arch (nop) fill.
      */
-    parser_nasm->prev_bc =
-	yasm_section_bcs_append(parser_nasm->cur_section,
-	    yasm_bc_create_align(boundval, NULL, NULL,
-		/*yasm_section_is_code(parser_nasm->cur_section) ?*/
-		    yasm_arch_get_fill(parser_nasm->arch)/* : NULL*/,
-		cur_line));
-}
-
-static void
-dir_cpu(yasm_parser_nasm *parser_nasm, yasm_valparamhead *valparams,
-	yasm_valparamhead *objext_valparams)
-{
-    yasm_valparam *vp;
-    yasm_vps_foreach(vp, valparams) {
-	if (vp->val)
-	    yasm_arch_parse_cpu(parser_nasm->arch, vp->val, strlen(vp->val));
-	else if (vp->param) {
-	    const yasm_intnum *intcpu;
-	    intcpu = yasm_expr_get_intnum(&vp->param, 0);
-	    if (!intcpu)
-		yasm_error_set(YASM_ERROR_SYNTAX,
-			       N_("invalid argument to [%s]"), "CPU");
-	    else {
-		char strcpu[16];
-		sprintf(strcpu, "%lu", yasm_intnum_get_uint(intcpu));
-		yasm_arch_parse_cpu(parser_nasm->arch, strcpu, strlen(strcpu));
-	    }
-	}
-    }
+    yasm_section_bcs_append(object->cur_section,
+	yasm_bc_create_align(boundval, NULL, NULL,
+	    /*yasm_section_is_code(object->cur_section) ?*/
+	    yasm_arch_get_fill(object->arch)/* : NULL*/,
+	    line));
 }
 
 static void
@@ -1209,58 +1129,65 @@ nasm_parser_directive(yasm_parser_nasm *parser_nasm, const char *name,
 		      yasm_valparamhead *valparams,
 		      yasm_valparamhead *objext_valparams)
 {
-    yasm_valparam *vp;
     unsigned long line = cur_line;
-    static const struct {
-	const char *name;
-	unsigned int required_arg:1;
-	unsigned int id_only:1;
-	void (*func) (yasm_parser_nasm *, yasm_valparamhead *,
-		      yasm_valparamhead *);
-    } dirs[] = {
-	{"EXTERN", 1, 1, dir_extern},
-	{"GLOBAL", 1, 1, dir_global},
-	{"COMMON", 1, 1, dir_common},
-	{"SECTION", 1, 0, dir_section},
-	{"SEGMENT", 1, 0, dir_section},
-	{"ABSOLUTE", 1, 0, dir_absolute},
-	{"ALIGN", 1, 0, dir_align},
-	{"CPU", 1, 0, dir_cpu}
-    };
-    size_t i;
+    yasm_valparam *vp;
 
-    /* Handle (mostly) output-format independent directives here */
-    for (i=0; i<NELEMS(dirs); i++) {
-	if (yasm__strcasecmp(name, dirs[i].name) == 0) {
-	    if (dirs[i].required_arg && !valparams) {
-		yasm_error_set(YASM_ERROR_SYNTAX,
-			       N_("[%s] requires an argument"), dirs[i].name);
-		break;
-	    }
-	    if (dirs[i].id_only && (vp = yasm_vps_first(valparams))
-		&& !vp->val) {
-		yasm_error_set(YASM_ERROR_SYNTAX,
-			       N_("invalid argument to [%s]"), dirs[i].name);
-		break;
-	    }
-	    dirs[i].func(parser_nasm, valparams, objext_valparams);
-	    break;
-	}
+    if (!yasm_object_directive(p_object, name, "nasm", valparams,
+			       objext_valparams, line))
+	;
+    else if (yasm__strcasecmp(name, "absolute") == 0) {
+	vp = yasm_vps_first(valparams);
+	if (parser_nasm->absstart)
+	    yasm_expr_destroy(parser_nasm->absstart);
+	if (parser_nasm->abspos)
+	    yasm_expr_destroy(parser_nasm->abspos);
+	parser_nasm->absstart = yasm_vp_expr(vp, p_object->symtab, line);
+	parser_nasm->abspos = yasm_expr_copy(parser_nasm->absstart);
+	cursect = NULL;
+	parser_nasm->prev_bc = NULL;
+    } else if (yasm__strcasecmp(name, "align") == 0) {
+	/* Really, we shouldn't end up with an align directive in an absolute
+         * section (as it's supposed to be only used for nop fill), but handle
+	 * it gracefully anyway.
+	 */
+	if (parser_nasm->abspos) {
+	    yasm_expr *boundval, *e;
+	    vp = yasm_vps_first(valparams);
+	    boundval = yasm_vp_expr(vp, p_object->symtab, line);
+	    e = yasm_expr_create_tree(
+		yasm_expr_create_tree(yasm_expr_copy(parser_nasm->absstart),
+				      YASM_EXPR_SUB,
+				      yasm_expr_copy(parser_nasm->abspos),
+				      cur_line),
+		YASM_EXPR_AND,
+		yasm_expr_create(YASM_EXPR_SUB, yasm_expr_expr(boundval),
+				 yasm_expr_int(yasm_intnum_create_uint(1)),
+				 cur_line),
+		cur_line);
+	    parser_nasm->abspos = yasm_expr_create_tree(
+		parser_nasm->abspos, YASM_EXPR_ADD, e, cur_line);
+        } else if (!valparams) {
+            yasm_error_set(YASM_ERROR_SYNTAX,
+                           N_("directive `%s' requires an argument"), "align");
+	} else
+	    dir_align(p_object, valparams, objext_valparams, line);
+    } else
+	yasm_error_set(YASM_ERROR_SYNTAX, N_("unrecognized directive `%s'"),
+		       name);
+
+    if (parser_nasm->absstart && cursect) {
+        /* We switched to a new section.  Get out of absolute section mode. */
+        yasm_expr_destroy(parser_nasm->absstart);
+        parser_nasm->absstart = NULL;
+        if (parser_nasm->abspos) {
+            yasm_expr_destroy(parser_nasm->abspos);
+            parser_nasm->abspos = NULL;
+        }
     }
 
-    if (i != NELEMS(dirs)) {
-	;
-    } else if (!yasm_arch_parse_directive(parser_nasm->arch, name, valparams,
-		    objext_valparams, parser_nasm->object, line)) {
-	;
-    } else if (!yasm_dbgfmt_directive(parser_nasm->dbgfmt, name,
-				      parser_nasm->cur_section, valparams,
-				      line)) {
-	;
-    } else if (yasm_objfmt_directive(parser_nasm->objfmt, name, valparams,
-				     objext_valparams, line)) {
-	yasm_error_set(YASM_ERROR_SYNTAX, N_("unrecognized directive [%s]"),
-		       name);
+    if (cursect) {
+        /* In case cursect changed or a bytecode was added, update prev_bc. */
+        parser_nasm->prev_bc = yasm_section_bcs_last(cursect);
     }
 
     if (valparams)
